@@ -22,7 +22,6 @@
 
 #include "Indicators.h"
 
-
 namespace unity
 {
 namespace indicator
@@ -38,19 +37,16 @@ public:
   DBusIndicators();
   ~DBusIndicators();
 
-  void SyncGeometries(std::string const& name,
-                      EntryLocationMap const& locations);
-
-  virtual void OnEntryScroll(std::string const& entry_id, int delta);
-  virtual void OnEntryShowMenu(std::string const& entry_id, unsigned int xid,
-                               int x, int y, unsigned int button,
-                               unsigned int timestamp);
-  virtual void OnEntrySecondaryActivate(std::string const& entry_id,
-                                        unsigned int timestamp);
-  virtual void OnShowAppMenu(unsigned int xid, int x, int y,
-                             unsigned int timestamp);
+  std::vector<std::string> const& IconPaths() const;
+  void ShowEntriesDropdown(Indicator::Entries const&, Entry::Ptr const&, unsigned xid, int x, int y);
+  void SyncGeometries(std::string const& name, EntryLocationMap const& locations);
 
 protected:
+  virtual void OnEntryScroll(std::string const& entry_id, int delta);
+  virtual void OnEntryShowMenu(std::string const& entry_id, unsigned int xid, int x, int y, unsigned int button);
+  virtual void OnEntrySecondaryActivate(std::string const& entry_id);
+  virtual void OnShowAppMenu(unsigned int xid, int x, int y);
+
   DBusIndicators(std::string const& dbus_name);
   bool IsConnected() const;
 
@@ -59,6 +55,11 @@ private:
   std::unique_ptr<Impl> pimpl;
 };
 
+struct LockScreenDBusIndicators : DBusIndicators
+{
+  LockScreenDBusIndicators();
+};
+
 }
 }
 
