@@ -23,12 +23,17 @@
 
 #include <glib.h>
 #include <gio/gdesktopappinfo.h>
+#include <NuxCore/Logger.h>
 
 #include "DesktopUtilities.h"
 #include "GLibWrapper.h"
 
 namespace unity
 {
+namespace
+{
+DECLARE_LOGGER(logger, "unity.desktop.utilities");
+}
 
 std::string DesktopUtilities::GetUserDataDirectory()
 {
@@ -49,6 +54,30 @@ std::string DesktopUtilities::GetUserDataDirectory()
   return "";
 }
 
+std::string DesktopUtilities::GetUserCacheDirectory()
+{
+  const char *cache_dir = g_get_user_cache_dir();
+  auto unity_cache = glib::gchar_to_string(cache_dir).append(G_DIR_SEPARATOR_S "unity" G_DIR_SEPARATOR_S);
+
+  if (g_mkdir_with_parents(unity_cache.c_str(), 0700) >= 0)
+    return unity_cache;
+
+  LOG_ERROR(logger) << "Impossible to create unity cache folder '"<< unity_cache <<"' !";
+  return "";
+}
+
+std::string DesktopUtilities::GetUserRuntimeDirectory()
+{
+  const char *runtime_dir = g_get_user_runtime_dir();
+  auto unity_runtime = glib::gchar_to_string(runtime_dir).append(G_DIR_SEPARATOR_S "unity" G_DIR_SEPARATOR_S);
+
+  if (g_mkdir_with_parents(unity_runtime.c_str(), 0700) >= 0)
+    return unity_runtime;
+
+  LOG_ERROR(logger) << "Impossible to create unity runtime folder '"<< unity_runtime <<"' !";
+  return "";
+}
+
 std::vector<std::string> DesktopUtilities::GetSystemDataDirectories()
 {
   const char* const* system_dirs = g_get_system_data_dirs();
