@@ -1,6 +1,6 @@
 // -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright (C) 2011 Canonical Ltd
+ * Copyright (C) 2011-2013 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -15,6 +15,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Authored by: Neil Jagdish Patel <neil.patel@canonical.com>
+ *              Michal Hruby <michal.hruby@canonical.com>
+ *              Marco Trevisan <marco.trevisan@canonical.com>
  */
 
 #ifndef UNITY_DBUS_PROXY_H
@@ -26,6 +28,9 @@
 #include <sigc++/signal.h>
 #include <sigc++/trackable.h>
 
+#include "GLibWrapper.h"
+#include "Variant.h"
+
 namespace unity
 {
 namespace glib
@@ -38,6 +43,7 @@ class DBusProxy : public sigc::trackable, boost::noncopyable
 public:
   typedef std::shared_ptr<DBusProxy> Ptr;
   typedef std::function<void(GVariant*)> ReplyCallback;
+  typedef std::function<void(GVariant*, Error const&)> CallFinishedCallback;
 
   DBusProxy(std::string const& name,
             std::string const& object_path,
@@ -48,13 +54,28 @@ public:
 
   void Call(std::string const& method_name,
             GVariant* parameters = nullptr,
-            ReplyCallback callback = nullptr,
+            ReplyCallback const& callback = nullptr,
             GCancellable *cancellable = nullptr,
             GDBusCallFlags flags = G_DBUS_CALL_FLAGS_NONE,
             int timeout_msec = -1);
+  void CallBegin(std::string const& method_name,
+                 GVariant* parameters,
+                 CallFinishedCallback const& callback,
+                 GCancellable *cancellable = nullptr,
+                 GDBusCallFlags flags = G_DBUS_CALL_FLAGS_NONE,
+                 int timeout_msec = -1);
 
-  void Connect(std::string const& signal_name, ReplyCallback callback);
-  bool IsConnected();
+  bool IsConnected() const;
+
+  Variant GetProperty(std::string const& property_name) const;
+  void GetProperty(std::string const& property_name, ReplyCallback const&);
+  void SetProperty(std::string const& property_name, GVariant* value);
+
+  void Connect(std::string const& signal_name, ReplyCallback const& callback);
+  void DisconnectSignal(std::string const& signal_name = "");
+
+  void ConnectProperty(std::string const& property_name, ReplyCallback const& callback);
+  void DisconnectProperty(std::string const& property_name = "");
 
   sigc::signal<void> connected;
   sigc::signal<void> disconnected;
