@@ -33,47 +33,22 @@ namespace indicator
 class Indicators : public sigc::trackable, boost::noncopyable
 {
 public:
+  typedef std::shared_ptr<Indicators> Ptr;
   typedef std::list<Indicator::Ptr> IndicatorsList;
 
   Indicators();
   virtual ~Indicators();
 
   IndicatorsList GetIndicators() const;
+  virtual std::vector<std::string> const& IconPaths() const = 0;
 
-  /**
-   * internal
-   */
-  void ActivateEntry(std::string const& entry_id, nux::Rect const& geometry);
-
-  /**
-   * internal
-   */
-  void SetEntryShowNow(std::string const& entry_id, bool show_now);
-
-  /**
-   * internal
-   */
-  virtual void OnEntryScroll(std::string const& entry_id, int delta) = 0;
-
-  /**
-   * internal
-   */
-  virtual void OnEntryShowMenu(std::string const& entry_id, unsigned int xid,
-                               int x, int y, unsigned int button) = 0;
-
-  /**
-   * internal
-   */
-  virtual void OnEntrySecondaryActivate(std::string const& entry_id) = 0;
-
-  /**
-   * internal
-   */
-  virtual void OnShowAppMenu(unsigned int xid, int x, int y) = 0;
+  virtual void SyncGeometries(std::string const& panel, EntryLocationMap const&) = 0;
+  virtual void ShowEntriesDropdown(Indicator::Entries const&, Entry::Ptr const&, unsigned xid, int x, int y) = 0;
 
   // Signals
   sigc::signal<void, Indicator::Ptr const&> on_object_added;
   sigc::signal<void, Indicator::Ptr const&> on_object_removed;
+  sigc::signal<void> icon_paths_changed;
 
   /**
    * Service wants the view to activate an entry.
@@ -87,12 +62,7 @@ public:
    * An entry just got activated. View needs to repaint it.
    * @param entry_id entry id
    */
-  sigc::signal<void, std::string const&, nux::Rect const&> on_entry_activated;
-
-  /**
-   * internal
-   */
-  sigc::signal<void> on_synced;
+  sigc::signal<void, std::string const&, std::string const&, nux::Rect const&> on_entry_activated;
 
   /**
    * The service is about to show a menu.
@@ -104,15 +74,15 @@ public:
    */
   sigc::signal<void, std::string const&, unsigned, int, int, unsigned> on_entry_show_menu;
 
-  /**
-   * The service is about to show an appmenu.
-   * @param xid window xid
-   * @param x coordinate
-   * @param y coordinate
-   */
-  sigc::signal<void, unsigned, int, int> on_show_appmenu;
-
 protected:
+  void ActivateEntry(std::string const& panel, std::string const& entry_id, nux::Rect const& geometry);
+  void SetEntryShowNow(std::string const& entry_id, bool show_now);
+
+  virtual void OnEntryScroll(std::string const& entry_id, int delta) = 0;
+  virtual void OnEntryShowMenu(std::string const& entry_id, unsigned int xid, int x, int y, unsigned int button) = 0;
+  virtual void OnEntrySecondaryActivate(std::string const& entry_id) = 0;
+  virtual void OnShowAppMenu(unsigned int xid, int x, int y) = 0;
+
   Indicator::Ptr GetIndicator(std::string const& name);
   Indicator::Ptr AddIndicator(std::string const& name);
   void RemoveIndicator(std::string const& name);
