@@ -1,6 +1,6 @@
 // -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright (C) 2011-2012 Canonical Ltd
+ * Copyright (C) 2011-2013 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -16,30 +16,27 @@
  *
  * Authored by: Neil Jagdish Patel <neil.patel@canonical.com>
  *              Michal Hruby <michal.hruby@canonical.com>
+ *              Manuel de la Pena <manuel.delapena@canonical.com>
  */
 
 #include <NuxCore/Logger.h>
 #include <unity-protocol.h>
 
-#include "Lens.h"
 #include "Preview.h"
+#include "Scope.h"
 
 #include "ApplicationPreview.h"
 #include "GenericPreview.h"
 #include "MusicPreview.h"
 #include "MoviePreview.h"
+#include "PaymentPreview.h"
 #include "SocialPreview.h"
-#include "SeriesPreview.h"
 
 namespace unity
 {
 namespace dash
 {
-
-namespace
-{
-  nux::logging::Logger logger("unity.dash.preview");
-}
+DECLARE_LOGGER(logger, "unity.dash.preview");
 
 Preview::Ptr Preview::PreviewForProtocolObject(glib::Object<GObject> const& proto_obj)
 {
@@ -55,27 +52,31 @@ Preview::Ptr Preview::PreviewForProtocolObject(glib::Object<GObject> const& prot
 
   if (renderer_name == "preview-generic")
   {
-    return Preview::Ptr(new GenericPreview(proto_obj));
+    return std::make_shared<GenericPreview>(proto_obj);
+  }
+  else if (renderer_name == "preview-payment")
+  {
+    return std::make_shared<PaymentPreview>(proto_obj);
+  }
+  else if (renderer_name == "preview-payment")
+  {
+    return std::make_shared<PaymentPreview>(proto_obj);
   }
   else if (renderer_name == "preview-application")
   {
-    return Preview::Ptr(new ApplicationPreview(proto_obj));
+    return std::make_shared<ApplicationPreview>(proto_obj);
   }
   else if (renderer_name == "preview-music")
   {
-    return Preview::Ptr(new MusicPreview(proto_obj));
+    return std::make_shared<MusicPreview>(proto_obj);
   }
   else if (renderer_name == "preview-movie")
   {
-    return Preview::Ptr(new MoviePreview(proto_obj));
+    return std::make_shared<MoviePreview>(proto_obj);
   }
   else if (renderer_name == "preview-social")
   {
-    return Preview::Ptr(new SocialPreview(proto_obj));
-  }
-  else if (renderer_name == "preview-series")
-  {
-    return Preview::Ptr(new SeriesPreview(proto_obj));
+    return std::make_shared<SocialPreview>(proto_obj);
   }
   else
   {
@@ -85,7 +86,7 @@ Preview::Ptr Preview::PreviewForProtocolObject(glib::Object<GObject> const& prot
   return nullptr;
 }
 
-Preview::Ptr Preview::PreviewForVariant(glib::Variant &properties)
+Preview::Ptr Preview::PreviewForVariant(glib::Variant const& properties)
 {
   glib::Object<UnityProtocolPreview> preview(unity_protocol_preview_parse(properties));
   if (!preview)
@@ -98,19 +99,6 @@ Preview::Ptr Preview::PreviewForVariant(glib::Variant &properties)
   return PreviewForProtocolObject(proto_obj);
 }
 
-unity::glib::Object<GIcon> Preview::IconForString(std::string const& icon_hint)
-{
-  glib::Error error;
-  glib::Object<GIcon> icon(g_icon_new_for_string(icon_hint.c_str(), &error));
-
-  if (error)
-  {
-    LOG_WARN(logger) << "Unable to instantiate icon: " << icon_hint;
-  }
-
-  return icon;
-}
-
 class Preview::Impl
 {
 public:
@@ -123,14 +111,13 @@ public:
   std::string get_description() const { return description_; };
   unity::glib::Object<GIcon> get_image() const { return image_; };
   std::string get_image_source_uri() const { return image_source_uri_; };
-  ActionPtrList get_actions() const { return actions_list_; };
-  InfoHintPtrList get_info_hints() const { return info_hint_list_; };
-  void EmitClosed() const;
+  ActionPtrList const& get_actions() const { return actions_list_; };
+  InfoHintPtrList const& get_info_hints() const { return info_hint_list_; };
 
-  Lens* get_parent_lens() const { return parent_lens_; };
-  bool set_parent_lens(Lens* lens)
+  Scope* get_parent_scope() const { return parent_scope_; };
+  bool set_parent_scope(Scope* scope)
   {
-    parent_lens_ = lens;
+    parent_scope_ = scope;
     return false; // TODO: do we need the notifications here?
   };
 
@@ -145,12 +132,13 @@ public:
   std::string image_source_uri_;
   ActionPtrList actions_list_;
   InfoHintPtrList info_hint_list_;
-  Lens* parent_lens_;
+
+  Scope* parent_scope_;
 };
 
 Preview::Impl::Impl(Preview* owner, glib::Object<GObject> const& proto_obj)
   : owner_(owner)
-  , parent_lens_(nullptr)
+  , parent_scope_(nullptr)
 {
   if (!proto_obj)
   {
@@ -185,7 +173,7 @@ Preview::Impl::Impl(Preview* owner, glib::Object<GObject> const& proto_obj)
             static_cast<LayoutHint>(raw_action->layout_hint),
             raw_action->hints));
     }
-    
+
     int info_hints_len;
     auto info_hints = unity_protocol_preview_get_info_hints(raw_preview_, &info_hints_len);
     for (int i = 0; i < info_hints_len; i++)
@@ -219,21 +207,10 @@ void Preview::Impl::SetupGetters()
   owner_->image_source_uri.SetGetterFunction(
     sigc::mem_fun(this, &Preview::Impl::get_image_source_uri));
 
-  owner_->parent_lens.SetGetterFunction(
-      sigc::mem_fun(this, &Preview::Impl::get_parent_lens));
-  owner_->parent_lens.SetSetterFunction(
-      sigc::mem_fun(this, &Preview::Impl::set_parent_lens));
-}
-
-void Preview::Impl::EmitClosed() const
-{
-  UnityProtocolPreview *preview = UNITY_PROTOCOL_PREVIEW(raw_preview_.RawPtr());
-
-  unity_protocol_preview_begin_updates(preview);
-  unity_protocol_preview_preview_closed(raw_preview_);
-  glib::Variant properties(unity_protocol_preview_end_updates(preview),
-                           glib::StealRef());
-  owner_->Update(properties);
+  owner_->parent_scope.SetGetterFunction(
+      sigc::mem_fun(this, &Preview::Impl::get_parent_scope));
+  owner_->parent_scope.SetSetterFunction(
+      sigc::mem_fun(this, &Preview::Impl::set_parent_scope));
 }
 
 Preview::Preview(glib::Object<GObject> const& proto_obj)
@@ -245,45 +222,51 @@ Preview::~Preview()
 {
 }
 
-Preview::ActionPtrList Preview::GetActions() const
+Preview::ActionPtrList const& Preview::GetActions() const
 {
   return pimpl->get_actions();
 }
 
-Preview::InfoHintPtrList Preview::GetInfoHints() const
+Preview::ActionPtr Preview::GetActionById(std::string const& _id) const
+{
+  for (Preview::ActionPtr const& action : GetActions())
+  {
+    if (action->id == _id)
+      return action;
+  }
+  return Preview::ActionPtr();
+}
+
+Preview::InfoHintPtrList const& Preview::GetInfoHints() const
 {
   return pimpl->get_info_hints();
 }
 
-void Preview::Update(glib::Variant const& properties,
-                     glib::DBusProxy::ReplyCallback reply_callback) const
+void Preview::PerformAction(std::string const& id, glib::HintsMap const& hints, ActivateCallback const& callback, GCancellable* cancellable) const
 {
-  if (pimpl->parent_lens_)
+  ActionPtr action = GetActionById(id);
+
+  if (action && pimpl->parent_scope_)
   {
-    pimpl->parent_lens_->SignalPreview(preview_uri, properties, reply_callback);
+    auto reply_func = [id, callback] (LocalResult const& result, ScopeHandledType handled_type, glib::Error const& error)
+    {
+      if (error)
+      {
+        LOG_WARN(logger) << "Preview action '" << id << "' failed => '" << error;
+      }
+      if (callback)
+      {
+        callback(result, handled_type, error);
+      }
+    };
+
+    pimpl->parent_scope_->ActivatePreviewAction(action, preview_result, hints, reply_func, cancellable);
   }
   else
   {
-    LOG_WARN(logger) << "Unable to update Preview, parent_lens wasn't set!";
+    LOG_WARN(logger) << "Unable to perform action '" << id << "', parent_scope_ wasn't set!";
   }
 }
 
-void Preview::PerformAction(std::string const& id, Lens::Hints const& hints) const
-{
-  if (pimpl->parent_lens_)
-  {
-    pimpl->parent_lens_->ActivatePreviewAction(id, preview_uri, hints);
-  }
-  else
-  {
-    LOG_WARN(logger) << "Unable to perform action '" << id << "', parent_lens wasn't set!";
-  }
-}
-
-void Preview::EmitClosed() const
-{
-  pimpl->EmitClosed();
-}
-
 } // namespace dash
 } // namespace unity
