@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Canonical Ltd
+ * Copyright (C) 2010-2014 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -21,6 +21,7 @@
 #include "DashView.h"
 #include "DashViewPrivate.h"
 #include "FilterExpanderLabel.h"
+#include "MultiMonitor.h"
 
 #include <math.h>
 
@@ -35,7 +36,7 @@
 #include "unity-shared/DashStyle.h"
 #include "unity-shared/KeyboardUtil.h"
 #include "unity-shared/PreviewStyle.h"
-#include "unity-shared/PanelStyle.h"
+#include "unity-shared/RawPixel.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/UnitySettings.h"
 #include "unity-shared/WindowManager.h"
@@ -53,10 +54,9 @@ const int PREVIEW_ANIMATION_LENGTH = 250;
 
 const int DASH_TILE_HORIZONTAL_COUNT = 6;
 const int DASH_DEFAULT_CATEGORY_COUNT = 3;
-const int DASH_RESULT_RIGHT_PAD = 35;
-const int GROUP_HEADING_HEIGHT = 24;
 
-const int PREVIEW_ICON_SPLIT_OFFSCREEN_OFFSET = 10;
+const RawPixel DASH_RESULT_RIGHT_PAD = 35_em;
+const RawPixel PREVIEW_ICON_SPLIT_OFFSCREEN_OFFSET = 10_em;
 
 const int MAX_ENTRY_ACTIVATE_WAIT_TIMEOUT = 1000;
 }
@@ -129,6 +129,8 @@ DashView::DashView(Scopes::Ptr const& scopes, ApplicationStarter::Ptr const& app
   , animate_preview_container_value_(0.0)
   , animate_preview_value_(0.0)
   , overlay_window_buttons_(new OverlayWindowButtons())
+  , cv_(Settings::Instance().em())
+  , monitor_(0)
 {
   renderer_.SetOwner(this);
   renderer_.need_redraw.connect([this] () {
@@ -155,6 +157,8 @@ DashView::DashView(Scopes::Ptr const& scopes, ApplicationStarter::Ptr const& app
     if (visible_ && !area)
       nux::GetWindowCompositor().SetKeyFocusArea(default_focus());
   });
+
+  unity::Settings::Instance().dpi_changed.connect(sigc::mem_fun(this, &DashView::OnDPIChanged));
 }
 
 DashView::~DashView()
@@ -168,6 +172,8 @@ void DashView::SetMonitorOffset(int x, int y)
 {
   renderer_.x_offset = x;
   renderer_.y_offset = y;
+
+  top_space_->SetMinMaxSize(0, y);
 }
 
 bool DashView::IsCommandLensOpen() const
@@ -237,7 +243,7 @@ void DashView::BuildPreview(Preview::Ptr model)
     preview_displaying_ = true;
 
     // connect to nav left/right signals to request nav left/right movement.
-    preview_container_->navigate_left.connect([&] () {
+    preview_container_->navigate_left.connect([this] () {
       preview_navigation_mode_ = previews::Navigation::LEFT;
 
       // sends a message to all result views, sending the the uri of the current preview result
@@ -245,7 +251,7 @@ void DashView::BuildPreview(Preview::Ptr model)
       ubus_manager_.SendMessage(UBUS_DASH_PREVIEW_NAVIGATION_REQUEST, g_variant_new("(ivs)", -1, g_variant_ref(last_activated_result_.Variant()), stored_activated_unique_id_.c_str()));
     });
 
-    preview_container_->navigate_right.connect([&] () {
+    preview_container_->navigate_right.connect([this] () {
       preview_navigation_mode_ = previews::Navigation::RIGHT;
 
       // sends a message to all result views, sending the the uri of the current preview result
@@ -253,7 +259,7 @@ void DashView::BuildPreview(Preview::Ptr model)
       ubus_manager_.SendMessage(UBUS_DASH_PREVIEW_NAVIGATION_REQUEST, g_variant_new("(ivs)", 1, g_variant_ref(last_activated_result_.Variant()), stored_activated_unique_id_.c_str()));
     });
 
-    preview_container_->request_close.connect([&] () { ClosePreview(); });
+    preview_container_->request_close.connect([this] () { ClosePreview(); });
   }
   else
   {
@@ -303,7 +309,7 @@ void DashView::StartPreviewAnimation()
   split_animation_->SetStartValue(animate_split_value_);
   split_animation_->SetFinishValue(1.0f);
   split_animation_->SetEasingCurve(na::EasingCurve(na::EasingCurve::Type::Linear));
-  split_animation_->updated.connect([&](float const& linear_split_animate_value)
+  split_animation_->updated.connect([this](float const& linear_split_animate_value)
   {
     static na::EasingCurve split_animation_curve(na::EasingCurve::Type::InQuad);
 
@@ -319,7 +325,7 @@ void DashView::StartPreviewAnimation()
       preview_container_animation_->SetStartValue(animate_preview_container_value_);
       preview_container_animation_->SetFinishValue(1.0f);
       preview_container_animation_->SetEasingCurve(na::EasingCurve(na::EasingCurve::Type::Linear));
-      preview_container_animation_->updated.connect([&](float const& linear_preview_container_animate_value)
+      preview_container_animation_->updated.connect([this](float const& linear_preview_container_animate_value)
       {
         static na::EasingCurve preview_container_animation_curve(na::EasingCurve::Type::InQuad);
 
@@ -335,7 +341,7 @@ void DashView::StartPreviewAnimation()
           preview_animation_->SetStartValue(animate_preview_value_);
           preview_animation_->SetFinishValue(1.0f);
           preview_animation_->SetEasingCurve(na::EasingCurve(na::EasingCurve::Type::Linear));
-          preview_animation_->updated.connect([&](float const& linear_preview_animate_value)
+          preview_animation_->updated.connect([this](float const& linear_preview_animate_value)
           {
             animate_preview_value_ = linear_preview_animate_value;
             QueueDraw();
@@ -372,7 +378,7 @@ void DashView::EndPreviewAnimation()
   preview_animation_->SetStartValue(1.0f - animate_preview_value_);
   preview_animation_->SetFinishValue(1.0f);
   preview_animation_->SetEasingCurve(na::EasingCurve(na::EasingCurve::Type::Linear));
-  preview_animation_->updated.connect([&](float const& preview_value)
+  preview_animation_->updated.connect([this](float const& preview_value)
   {
     animate_preview_value_ = 1.0f - preview_value;
     QueueDraw();
@@ -386,7 +392,7 @@ void DashView::EndPreviewAnimation()
       preview_container_animation_->SetStartValue(1.0f - animate_preview_container_value_);
       preview_container_animation_->SetFinishValue(1.0f);
       preview_container_animation_->SetEasingCurve(na::EasingCurve(na::EasingCurve::Type::Linear));
-      preview_container_animation_->updated.connect([&](float const& linear_preview_container_animate_value)
+      preview_container_animation_->updated.connect([this](float const& linear_preview_container_animate_value)
       {
         static na::EasingCurve preview_container_animation_curve(na::EasingCurve::Type::InQuad);
 
@@ -401,7 +407,7 @@ void DashView::EndPreviewAnimation()
           split_animation_->SetStartValue(1.0f - animate_split_value_);
           split_animation_->SetFinishValue(1.0f);
           split_animation_->SetEasingCurve(na::EasingCurve(na::EasingCurve::Type::Linear));
-          split_animation_->updated.connect([&](float const& linear_split_animate_value)
+          split_animation_->updated.connect([this](float const& linear_split_animate_value)
           {
             static na::EasingCurve split_animation_curve(na::EasingCurve::Type::InQuad);
 
@@ -452,11 +458,20 @@ void DashView::OnPreviewAnimationFinished()
   content_view_->SetPresentRedirectedView(true);
 }
 
-void DashView::AboutToShow()
+void DashView::AboutToShow(int monitor)
 {
   visible_ = true;
   search_bar_->text_entry()->SelectAll();
 
+  if (monitor_ != monitor)
+  {
+    monitor_ = monitor;
+    cv_ = Settings::Instance().em(monitor_);
+
+    OnDPIChanged();
+    Relayout();
+  }
+
   /* Give the scopes a chance to prep data before we map them  */
   if (active_scope_view_)
   {
@@ -475,14 +490,20 @@ void DashView::AboutToShow()
     ClosePreview();
   }
 
-
   overlay_window_buttons_->Show();
 
+  renderer_.UpdateBlurBackgroundSize(content_geo_, GetRenderAbsoluteGeometry(), false);
   renderer_.AboutToShow();
 }
 
 void DashView::AboutToHide()
 {
+  if (BackgroundEffectHelper::blur_type == BLUR_STATIC)
+  { 
+    content_geo_ = {0, 0, 0, 0};
+    renderer_.UpdateBlurBackgroundSize(content_geo_, GetRenderAbsoluteGeometry(), false);
+  }
+
   visible_ = false;
   renderer_.AboutToHide();
 
@@ -510,34 +531,28 @@ void DashView::AboutToHide()
 
 void DashView::SetupViews()
 {
-  dash::Style& style = dash::Style::Instance();
-  panel::Style &panel_style = panel::Style::Instance();
-
   layout_ = new nux::VLayout();
-  layout_->SetLeftAndRightPadding(style.GetVSeparatorSize(), 0);
-  layout_->SetTopAndBottomPadding(style.GetHSeparatorSize(), 0);
   SetLayout(layout_);
-  layout_->AddLayout(new nux::SpaceLayout(0, 0, panel_style.panel_height, panel_style.panel_height), 0);
+
+  top_space_ = new nux::SpaceLayout(0, 0, renderer_.y_offset(), renderer_.y_offset());
+  layout_->AddLayout(top_space_, 0);
 
   content_layout_ = new DashLayout(NUX_TRACKER_LOCATION);
-  content_layout_->SetTopAndBottomPadding(style.GetDashViewTopPadding(), 0);
 
   content_view_ = new DashContentView(NUX_TRACKER_LOCATION);
   content_view_->SetLayout(content_layout_);
   layout_->AddView(content_view_, 1, nux::MINOR_POSITION_START, nux::MINOR_SIZE_FULL);
 
   search_bar_layout_ = new nux::HLayout();
-  search_bar_layout_->SetLeftAndRightPadding(style.GetSearchBarLeftPadding(), 0);
   content_layout_->AddLayout(search_bar_layout_, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FULL);
 
-  search_bar_ = new SearchBar();
+  search_bar_ = new SearchBar(true);
+  search_bar_->scale = cv_->DPIScale();
   AddChild(search_bar_);
-  search_bar_->SetMinimumHeight(style.GetSearchBarHeight());
-  search_bar_->SetMaximumHeight(style.GetSearchBarHeight());
   search_bar_->activated.connect(sigc::mem_fun(this, &DashView::OnEntryActivated));
   search_bar_->search_changed.connect(sigc::mem_fun(this, &DashView::OnSearchChanged));
   search_bar_->live_search_reached.connect(sigc::mem_fun(this, &DashView::OnLiveSearchReached));
-  search_bar_->showing_filters.changed.connect([&] (bool showing)
+  search_bar_->showing_filters.changed.connect([this] (bool showing)
   {
     if (active_scope_view_)
     {
@@ -555,6 +570,42 @@ void DashView::SetupViews()
   AddChild(scope_bar_);
   scope_bar_->scope_activated.connect(sigc::mem_fun(this, &DashView::OnScopeBarActivated));
   content_layout_->AddView(scope_bar_, 0, nux::MINOR_POSITION_CENTER);
+
+  OnDPIChanged();
+}
+
+void DashView::OnDPIChanged()
+{
+  double scale = cv_->DPIScale();
+
+  UpdateDashViewSize();
+
+  for (auto& scope : scope_views_)
+    scope.second->scale = scale;
+
+  search_bar_->scale = scale;
+  scope_bar_->scale = scale;
+}
+
+void DashView::UpdateDashViewSize()
+{
+  dash::Style const& style = dash::Style::Instance();
+
+  RawPixel const v_separator_size = style.GetVSeparatorSize();
+  RawPixel const h_separator_size = style.GetHSeparatorSize();
+  RawPixel const view_top_padding = style.GetDashViewTopPadding();
+
+  RawPixel const search_bar_left_padding = style.GetSearchBarLeftPadding();
+  RawPixel const search_bar_height       = style.GetSearchBarHeight();
+
+  layout_->SetLeftAndRightPadding(v_separator_size.CP(cv_), 0);
+  layout_->SetTopAndBottomPadding(h_separator_size.CP(cv_), 0);
+
+  content_layout_->SetTopAndBottomPadding(view_top_padding.CP(cv_), 0);
+
+  search_bar_layout_->SetLeftAndRightPadding(search_bar_left_padding.CP(cv_), 0);
+  search_bar_->SetMinimumHeight(search_bar_height.CP(cv_));
+  search_bar_->SetMaximumHeight(search_bar_height.CP(cv_));
 }
 
 void DashView::SetupUBusConnections()
@@ -575,23 +626,34 @@ void DashView::Relayout()
   content_geo_ = GetBestFitGeometry(geo);
   dash::Style& style = dash::Style::Instance();
 
+  RawPixel const top_padding = style.GetDashViewTopPadding();
+
   // kinda hacky, but it makes sure the content isn't so big that it throws
   // the bottom of the dash off the screen
   // not hugely happy with this, so FIXME
-  scopes_layout_->SetMaximumHeight (std::max(0, content_geo_.height - search_bar_->GetGeometry().height - scope_bar_->GetGeometry().height - style.GetDashViewTopPadding()));
-  scopes_layout_->SetMinimumHeight (std::max(0, content_geo_.height - search_bar_->GetGeometry().height - scope_bar_->GetGeometry().height - style.GetDashViewTopPadding()));
+  scopes_layout_->SetMaximumHeight (std::max(0, content_geo_.height -
+                                                search_bar_->GetGeometry().height -
+                                                scope_bar_->GetGeometry().height -
+                                                top_padding.CP(cv_)));
+
+  scopes_layout_->SetMinimumHeight (std::max(0, content_geo_.height -
+                                                search_bar_->GetGeometry().height -
+                                                scope_bar_->GetGeometry().height -
+                                                top_padding.CP(cv_)));
 
   layout_->SetMinMaxSize(content_geo_.width, content_geo_.y + content_geo_.height);
 
   // Minus the padding that gets added to the left
-  float tile_width = style.GetTileWidth();
-  style.SetDefaultNColumns(floorf((content_geo_.width - 32) / tile_width));
+  RawPixel const tile_width = style.GetTileWidth();
+  style.SetDefaultNColumns(floorf((content_geo_.width - cv_->CP(32)) / tile_width.CP(cv_)));
 
   ubus_manager_.SendMessage(UBUS_DASH_SIZE_CHANGED, g_variant_new("(ii)", content_geo_.width, content_geo_.height));
 
   if (preview_displaying_)
     preview_container_->SetGeometry(layout_->GetGeometry());
 
+  renderer_.UpdateBlurBackgroundSize(content_geo_, GetRenderAbsoluteGeometry(), false);
+
   QueueDraw();
 }
 
@@ -601,67 +663,73 @@ void DashView::Relayout()
 nux::Geometry DashView::GetBestFitGeometry(nux::Geometry const& for_geo)
 {
   dash::Style& style = dash::Style::Instance();
-  panel::Style &panel_style = panel::Style::Instance();
+  int panel_height = renderer_.y_offset;
 
   int width = 0, height = 0;
-  int tile_width = style.GetTileWidth();
-  int tile_height = style.GetTileHeight();
-  int category_height = (style.GetPlacesGroupTopSpace() + style.GetCategoryIconSize()  + style.GetPlacesGroupResultTopPadding() + tile_height);
+  RawPixel const tile_width  = style.GetTileWidth();
+  RawPixel const tile_height = style.GetTileHeight();
+  RawPixel const group_top_space    = style.GetPlacesGroupTopSpace();
+  RawPixel const category_icon_size = style.GetCategoryIconSize();
+  RawPixel const group_top_padding  = style.GetPlacesGroupResultTopPadding();
+
+  int category_height = (group_top_space.CP(cv_) + category_icon_size.CP(cv_) +
+                         group_top_padding.CP(cv_) + tile_height.CP(cv_));
+
   int half = for_geo.width / 2;
 
   // if default dash size is bigger than half a screens worth of items, go for that.
-  while ((width += tile_width) < half)
+  while ((width += tile_width.CP(cv_)) < half)
     ;
 
-  width = std::max(width, tile_width * DASH_TILE_HORIZONTAL_COUNT);
-  width += style.GetVSeparatorSize();
-  width += style.GetPlacesGroupResultLeftPadding() + DASH_RESULT_RIGHT_PAD;
+  RawPixel const v_separator_size   = style.GetVSeparatorSize();
+  RawPixel const group_left_padding = style.GetPlacesGroupResultLeftPadding();
 
+  width = std::max(width, tile_width.CP(cv_) * DASH_TILE_HORIZONTAL_COUNT);
+  width += v_separator_size.CP(cv_);
+  width += group_left_padding.CP(cv_) + DASH_RESULT_RIGHT_PAD.CP(cv_);
 
-  height = style.GetHSeparatorSize();
-  height += style.GetDashViewTopPadding();
+  RawPixel const h_separator_size = style.GetHSeparatorSize();
+  RawPixel const top_padding      = style.GetDashViewTopPadding();
+
+  height = h_separator_size.CP(cv_);
+  height += top_padding.CP(cv_);
   height += search_bar_->GetGeometry().height;
   height += category_height * DASH_DEFAULT_CATEGORY_COUNT; // adding three categories
   height += scope_bar_->GetGeometry().height;
 
   // width/height shouldn't be bigger than the geo available.
   width = std::min(width, for_geo.width); // launcher width is taken into account in for_geo.
-  height = std::min(height, for_geo.height - panel_style.panel_height); // panel height is not taken into account in for_geo.
+  height = std::min(height, for_geo.height - panel_height); // panel height is not taken into account in for_geo.
 
   if (style.always_maximised)
   {
     width = std::max(0, for_geo.width);
-    height = std::max(0, for_geo.height - panel_style.panel_height);
+    height = std::max(0, for_geo.height - panel_height);
   }
-  return nux::Geometry(0, panel_style.panel_height, width, height);
+  return nux::Geometry(0, panel_height, width, height);
 }
 
 void DashView::Draw(nux::GraphicsEngine& graphics_engine, bool force_draw)
 {
-  panel::Style &panel_style = panel::Style::Instance();
-
-  nux::Geometry renderer_geo_abs(GetAbsoluteGeometry());
-  renderer_geo_abs.y += panel_style.panel_height;
-  renderer_geo_abs.height -= panel_style.panel_height;
-
+  nux::Geometry const& renderer_geo_abs(GetRenderAbsoluteGeometry());
   nux::Geometry renderer_geo(GetGeometry());
-  renderer_geo.y += panel_style.panel_height;
-  renderer_geo.height += panel_style.panel_height;
+  renderer_geo.y += renderer_.y_offset;
+  renderer_geo.height += renderer_.y_offset;
 
   renderer_.DrawFull(graphics_engine, content_geo_, renderer_geo_abs, renderer_geo, false);
 }
 
 void DashView::DrawContent(nux::GraphicsEngine& graphics_engine, bool force_draw)
 {
-  panel::Style& panel_style = panel::Style::Instance();
+  int renderer_y_offset = renderer_.y_offset();
 
   nux::Geometry renderer_geo_abs(GetAbsoluteGeometry());
-  renderer_geo_abs.y += panel_style.panel_height;
-  renderer_geo_abs.height -= panel_style.panel_height;
+  renderer_geo_abs.y += renderer_y_offset;
+  renderer_geo_abs.height -= renderer_y_offset;
 
   nux::Geometry renderer_geo(GetGeometry());
-  renderer_geo.y += panel_style.panel_height;
-  renderer_geo.height += panel_style.panel_height;
+  renderer_geo.y += renderer_y_offset;
+  renderer_geo.height += renderer_y_offset;
 
   renderer_.DrawInner(graphics_engine, content_geo_, renderer_geo_abs, renderer_geo);
 
@@ -738,7 +806,7 @@ void DashView::DrawDashSplit(nux::GraphicsEngine& graphics_engine, nux::Geometry
     texxform.voffset = (scope_bar_->GetY() - content_view_->GetY())/(float)content_view_->GetHeight();
 
     int start_y = scope_bar_->GetY();
-    int final_y = geo_layout.y + geo_layout.height + PREVIEW_ICON_SPLIT_OFFSCREEN_OFFSET;
+    int final_y = geo_layout.y + geo_layout.height + PREVIEW_ICON_SPLIT_OFFSCREEN_OFFSET.CP(cv_);
 
     int scope_y = (1.0f - animate_split_value_) * start_y + (animate_split_value_ * final_y);
 
@@ -762,7 +830,7 @@ void DashView::DrawDashSplit(nux::GraphicsEngine& graphics_engine, nux::Geometry
       texxform.voffset = (search_bar_->GetY() - content_view_->GetY())/(float)content_view_->GetHeight();
 
       start_y = search_bar_->GetY();
-      final_y = geo_layout.y - search_bar_->GetHeight() - PREVIEW_ICON_SPLIT_OFFSCREEN_OFFSET;
+      final_y = geo_layout.y - search_bar_->GetHeight() - PREVIEW_ICON_SPLIT_OFFSCREEN_OFFSET.CP(cv_);
 
       graphics_engine.QRP_1Tex
       (
@@ -780,7 +848,7 @@ void DashView::DrawDashSplit(nux::GraphicsEngine& graphics_engine, nux::Geometry
       texxform.voffset = (search_bar_->GetY() - content_view_->GetY())/(float)content_view_->GetHeight();
 
       int start_x = active_scope_view_->filter_bar()->GetX();
-      int final_x = content_view_->GetX() + content_view_->GetWidth() + PREVIEW_ICON_SPLIT_OFFSCREEN_OFFSET;
+      int final_x = content_view_->GetX() + content_view_->GetWidth() + PREVIEW_ICON_SPLIT_OFFSCREEN_OFFSET.CP(cv_);
 
       int filter_x = (1.0f - animate_split_value_) * start_x + (animate_split_value_ * final_x);
 
@@ -804,7 +872,7 @@ void DashView::DrawDashSplit(nux::GraphicsEngine& graphics_engine, nux::Geometry
       texxform.voffset = (search_bar_->GetY() - content_view_->GetY())/(float)content_view_->GetHeight();
 
       int start_y = search_bar_->GetY();
-      int final_y = geo_layout.y - search_bar_->GetHeight() - PREVIEW_ICON_SPLIT_OFFSCREEN_OFFSET;
+      int final_y = geo_layout.y - search_bar_->GetHeight() - PREVIEW_ICON_SPLIT_OFFSCREEN_OFFSET.CP(cv_);
 
       graphics_engine.QRP_1Tex
       (
@@ -838,8 +906,8 @@ void DashView::DrawPreviewContainer(nux::GraphicsEngine& graphics_engine)
   // Triangle pointed at preview item
   if (opening_column_x_ != -1)
   {
-    int final_width = 14;
-    int final_height = 12;
+    int final_width  = cv_->CP(14);
+    int final_height = cv_->CP(12);
 
     int x_center = geo_content.x + (opening_column_x_ - geo_abs.x) + opening_column_width_ / 2;
     int start_y = geo_abs_preview.y - geo_abs.y;
@@ -1063,13 +1131,16 @@ void DashView::DrawPreview(nux::GraphicsEngine& graphics_engine, bool force_draw
 
 void DashView::OnMouseButtonDown(int x, int y, unsigned long button, unsigned long key)
 {
-  dash::Style& style = dash::Style::Instance();
   nux::Geometry geo(content_geo_);
 
   if (Settings::Instance().form_factor() == FormFactor::DESKTOP)
   {
-    geo.width += style.GetDashRightTileWidth();
-    geo.height += style.GetDashBottomTileHeight();
+    dash::Style& style = dash::Style::Instance();
+    RawPixel const right_title_width  = style.GetDashRightTileWidth();
+    RawPixel const bottom_title_width = style.GetDashBottomTileHeight();
+
+    geo.width  += right_title_width.CP(cv_);
+    geo.height += bottom_title_width.CP(cv_);
   }
 }
 
@@ -1211,6 +1282,7 @@ void DashView::OnScopeAdded(Scope::Ptr const& scope, int position)
 
   nux::ObjectPtr<ScopeView> view(new ScopeView(scope, search_bar_->show_filters()));
   AddChild(view.GetPointer());
+  view->scale = cv_->DPIScale();
   view->SetVisible(false);
   view->result_activated.connect(sigc::mem_fun(this, &DashView::OnResultActivated));
 
@@ -1220,10 +1292,10 @@ void DashView::OnScopeAdded(Scope::Ptr const& scope, int position)
   // set form factor used for the searches
   scope->form_factor = "desktop";
   scope->activated.connect(sigc::mem_fun(this, &DashView::OnResultActivatedReply));
-  scope->connected.changed.connect([&] (bool value) { });
+  scope->connected.changed.connect([this] (bool value) { });
 
   // Hook up to the new preview infrastructure
-  scope->preview_ready.connect([&] (LocalResult const& result, Preview::Ptr model)
+  scope->preview_ready.connect([this] (LocalResult const& result, Preview::Ptr model)
   {
     // HACK: Atm we don't support well the fact that a preview can be sent from
     // an ActionResponse and therefore transition does not work, this hack allows
@@ -1422,10 +1494,14 @@ std::string DashView::GetName() const
   return "DashView";
 }
 
-void DashView::AddProperties(GVariantBuilder* builder)
+void DashView::AddProperties(debug::IntrospectionData& introspection)
 {
   dash::Style& style = dash::Style::Instance();
   int num_rows = 1; // The search bar
+  std::vector<bool> button_on_monitor;
+
+  RawPixel const right_title_width  = style.GetDashRightTileWidth();
+  RawPixel const bottom_title_width = style.GetDashBottomTileHeight();
 
   if (active_scope_view_.IsValid())
     num_rows += active_scope_view_->GetNumRows();
@@ -1439,16 +1515,18 @@ void DashView::AddProperties(GVariantBuilder* builder)
   else if (Settings::Instance().form_factor() == FormFactor::TV)
     form_factor = "tv";
 
-  unity::variant::BuilderWrapper wrapper(builder);
-  wrapper.add(nux::Geometry(GetAbsoluteX(), GetAbsoluteY(), content_geo_.width, content_geo_.height));
-  wrapper.add("num_rows", num_rows);
-  wrapper.add("form_factor", form_factor);
-  wrapper.add("right-border-width", style.GetDashRightTileWidth());
-  wrapper.add("bottom-border-height", style.GetDashBottomTileHeight());
-  wrapper.add("preview_displaying", preview_displaying_);
-  wrapper.add("preview_animation", animate_split_value_ * animate_preview_container_value_ * animate_preview_value_);
-  wrapper.add("dash_maximized", style.always_maximised());
-  wrapper.add("overlay_window_buttons_shown", overlay_window_buttons_->IsVisible());
+  for (unsigned i = 0; i < monitors::MAX; ++i)
+    button_on_monitor.push_back(overlay_window_buttons_->IsVisibleOnMonitor(i));
+
+  introspection.add(nux::Geometry(GetAbsoluteX(), GetAbsoluteY(), content_geo_.width, content_geo_.height))
+               .add("num_rows", num_rows)
+               .add("form_factor", form_factor)
+               .add("right-border-width", right_title_width.CP(cv_))
+               .add("bottom-border-height", bottom_title_width.CP(cv_))
+               .add("preview_displaying", preview_displaying_)
+               .add("preview_animation", animate_split_value_ * animate_preview_container_value_ * animate_preview_value_)
+               .add("dash_maximized", style.always_maximised())
+               .add("overlay_window_buttons_shown", glib::Variant::FromVector(button_on_monitor));
 }
 
 nux::Area* DashView::KeyNavIteration(nux::KeyNavDirection direction)
@@ -1677,5 +1755,13 @@ nux::Geometry const& DashView::GetContentGeometry() const
   return content_geo_;
 }
 
+nux::Geometry DashView::GetRenderAbsoluteGeometry() const
+{
+  nux::Geometry renderer_geo_abs(GetAbsoluteGeometry());
+  renderer_geo_abs.y += renderer_.y_offset;
+  renderer_geo_abs.height -= renderer_.y_offset;
+  return renderer_geo_abs;
+}
+
 }
 }
