@@ -27,14 +27,13 @@
 #include <glib.h>
 #include "config.h"
 #include <glib/gi18n-lib.h>
-
-#include <UnityCore/Variant.h>
 #include <UnityCore/GLibWrapper.h>
 
 #include "unity-shared/StaticCairoText.h"
 #include "unity-shared/UBusWrapper.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/GraphicsUtils.h"
+#include "unity-shared/RawPixel.h"
 
 #include "ResultView.h"
 #include "ResultViewGrid.h"
@@ -56,12 +55,16 @@ const nux::Color kExpandDefaultTextColor(1.0f, 1.0f, 1.0f, 0.5f);
 const float kExpandDefaultIconOpacity = 0.5f;
 
 // Category  highlight
-const int kHighlightHeight = 24;
-const int kHighlightRightPadding = 10 - 3; // -3 because the scrollbar is not a real overlay scrollbar!
-const int kHighlightLeftPadding = 10;
+const RawPixel kHighlightRightPadding =  7_em; // FIXME 10 - 3 because the scrollbar is not a real overlay scrollbar!
+const RawPixel kHighlightHeight       = 24_em;
+const RawPixel kHighlightLeftPadding  = 10_em;
+const RawPixel SPACE_BETWEEN_CHILDREN = 10_em;
+const RawPixel TEXT_INTERNAL_MARGIN   = 15_em;
+const RawPixel EXPAND_INTERNAL_MARGIN =  8_em;
+const double DEFAULT_SCALE            = 1.0;
 
 // Font
-const char* const NAME_LABEL_FONT = "Ubuntu 13"; // 17px = 13
+const char* const NAME_LABEL_FONT     = "Ubuntu 13"; // 17px = 13
 const char* const EXPANDER_LABEL_FONT = "Ubuntu 10"; // 13px = 10
 
 }
@@ -114,6 +117,7 @@ NUX_IMPLEMENT_OBJECT_TYPE(PlacesGroup);
 
 PlacesGroup::PlacesGroup(dash::StyleInterface& style)
   : nux::View(NUX_TRACKER_LOCATION),
+    scale(DEFAULT_SCALE),
     _style(style),
     _child_layout(nullptr),
     _child_view(nullptr),
@@ -123,13 +127,14 @@ PlacesGroup::PlacesGroup(dash::StyleInterface& style)
     _n_visible_items_in_unexpand_mode(0),
     _n_total_items(0),
     _coverflow_enabled(false),
-    disabled_header_count_(false)
+    _disabled_header_count(false)
 {
   SetAcceptKeyNavFocusOnMouseDown(false);
   SetAcceptKeyNavFocusOnMouseEnter(false);
+  scale.changed.connect(sigc::mem_fun(this, &PlacesGroup::UpdateScale));
 
   nux::BaseTexture* arrow = _style.GetGroupExpandIcon();
-  
+
   _background = _style.GetCategoryBackground();
   _background_nofilters = _style.GetCategoryBackgroundNoFilters();
 
@@ -147,23 +152,22 @@ PlacesGroup::PlacesGroup(dash::StyleInterface& style)
 
   _group_layout = new nux::VLayout("", NUX_TRACKER_LOCATION);
 
-  int top_space = style.GetPlacesGroupTopSpace();
-  _group_layout->AddLayout(new nux::SpaceLayout(top_space, top_space, top_space, top_space), 0);
+  // Spacelayout size is updated in UpdatePlacesGroupSize
+  _space_layout = new nux::SpaceLayout(0, 0, 0, 0);
+  _group_layout->AddLayout(_space_layout, 0);
 
   _header_view = new HeaderView(NUX_TRACKER_LOCATION);
   _group_layout->AddView(_header_view, 0, nux::MINOR_POSITION_START, nux::MINOR_SIZE_FULL);
 
   _header_layout = new nux::HLayout(NUX_TRACKER_LOCATION);
   _header_layout->SetLeftAndRightPadding(_style.GetCategoryHeaderLeftPadding(), 0);
-  _header_layout->SetSpaceBetweenChildren(10);
   _header_view->SetLayout(_header_layout);
 
-  _icon = new IconTexture("", _style.GetCategoryIconSize());
-  _icon->SetMinMaxSize(_style.GetCategoryIconSize(), _style.GetCategoryIconSize());
+  RawPixel const icon_size = _style.GetCategoryIconSize();
+  _icon = new IconTexture("", icon_size.CP(scale()));
   _header_layout->AddView(_icon, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FIX);
 
   _text_layout = new nux::HLayout(NUX_TRACKER_LOCATION);
-  _text_layout->SetHorizontalInternalMargin(15);
   _header_layout->AddLayout(_text_layout, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_MATCHCONTENT);
 
   _name = new StaticCairoText("", NUX_TRACKER_LOCATION);
@@ -173,7 +177,6 @@ PlacesGroup::PlacesGroup(dash::StyleInterface& style)
   _text_layout->AddView(_name, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_MATCHCONTENT);
 
   _expand_layout = new nux::HLayout(NUX_TRACKER_LOCATION);
-  _expand_layout->SetHorizontalInternalMargin(8);
   _text_layout->AddLayout(_expand_layout, 0, nux::MINOR_POSITION_END, nux::MINOR_SIZE_MATCHCONTENT);
 
   _expand_label_layout = new nux::HLayout(NUX_TRACKER_LOCATION);
@@ -203,7 +206,7 @@ PlacesGroup::PlacesGroup(dash::StyleInterface& style)
   _expand_label->mouse_click.connect(sigc::mem_fun(this, &PlacesGroup::RecvMouseClick));
   _expand_icon->mouse_click.connect(sigc::mem_fun(this, &PlacesGroup::RecvMouseClick));
 
-  key_nav_focus_change.connect([&](nux::Area* area, bool has_focus, nux::KeyNavDirection direction)
+  key_nav_focus_change.connect([this](nux::Area* area, bool has_focus, nux::KeyNavDirection direction)
   {
     if (!has_focus)
       return;
@@ -213,6 +216,47 @@ PlacesGroup::PlacesGroup(dash::StyleInterface& style)
     else
       nux::GetWindowCompositor().SetKeyFocusArea(GetHeaderFocusableView(), direction);
   });
+
+  UpdatePlacesGroupSize();
+}
+
+void
+PlacesGroup::UpdatePlacesGroupSize()
+{
+   RawPixel const icon_size = _style.GetCategoryIconSize();
+   RawPixel const group_top = _style.GetPlacesGroupTopSpace();
+
+   int top_space = group_top.CP(scale());
+   _space_layout->SetMinimumSize(top_space, top_space);
+   _space_layout->SetMaximumSize(top_space, top_space);
+
+  _header_layout->SetSpaceBetweenChildren(SPACE_BETWEEN_CHILDREN.CP(scale()));
+
+  _icon->SetMinMaxSize(icon_size.CP(scale()), icon_size.CP(scale()));
+
+  _text_layout->SetHorizontalInternalMargin(TEXT_INTERNAL_MARGIN.CP(scale()));
+  _expand_layout->SetHorizontalInternalMargin(EXPAND_INTERNAL_MARGIN.CP(scale()));
+}
+
+void
+PlacesGroup::UpdateScale(double scale)
+{
+  RawPixel const icon_size = _style.GetCategoryIconSize();
+
+  _name->SetScale(scale);
+  _expand_label->SetScale(scale);
+
+  _icon->SetSize(icon_size.CP(scale));
+  _icon->ReLoadIcon();
+
+  // FIXME _expand_icon, needs some work here. Not as easy as _icon
+
+  if (_child_view)
+    _child_view->scale = scale;
+
+  ComputeContentSize();
+  UpdatePlacesGroupSize();
+  UpdateResultViewPadding();
 }
 
 void
@@ -234,6 +278,8 @@ PlacesGroup::OnLabelFocusChanged(nux::Area* label, bool has_focus, nux::KeyNavDi
 }
 
 void
+
+// FIXME _expand_icon, needs some work here. Not as easy as _icon
 PlacesGroup::SetName(std::string const& name)
 {
   if (_cached_name != name)
@@ -245,7 +291,7 @@ PlacesGroup::SetName(std::string const& name)
 
 void PlacesGroup::SetHeaderCountVisible(bool disable)
 {
-  disabled_header_count_ = !disable;
+  _disabled_header_count = !disable;
   Relayout();
 }
 
@@ -267,6 +313,19 @@ PlacesGroup::SetIcon(std::string const& path_to_emblem)
   _icon->SetByIconName(path_to_emblem, _style.GetCategoryIconSize());
 }
 
+void
+PlacesGroup::UpdateResultViewPadding()
+{
+  if (_child_layout)
+  {
+    RawPixel const result_top_padding  = _style.GetPlacesGroupResultTopPadding();
+    RawPixel const result_left_padding = _style.GetPlacesGroupResultLeftPadding();
+
+    _child_layout->SetTopAndBottomPadding(result_top_padding.CP(scale()), 0);
+    _child_layout->SetLeftAndRightPadding(result_left_padding.CP(scale()), 0);
+  }
+}
+
 void
 PlacesGroup::SetChildView(dash::ResultView* view)
 {
@@ -281,15 +340,15 @@ PlacesGroup::SetChildView(dash::ResultView* view)
   AddChild(view);
 
   _child_view = view;
+  _child_view->scale = scale();
 
   _child_layout = new nux::VLayout();
   _child_layout->AddView(_child_view, 0);
 
-  _child_layout->SetTopAndBottomPadding(_style.GetPlacesGroupResultTopPadding(),0);
-  _child_layout->SetLeftAndRightPadding(_style.GetPlacesGroupResultLeftPadding(), 0);
+  UpdateResultViewPadding();
   _group_layout->AddLayout(_child_layout, 1);
 
-  view->results_per_row.changed.connect([&] (int results_per_row)
+  view->results_per_row.changed.connect([this] (int results_per_row)
   {
     _n_visible_items_in_unexpand_mode = results_per_row;
     RefreshLabel();
@@ -313,7 +372,7 @@ void PlacesGroup::SetChildLayout(nux::Layout* layout)
 void
 PlacesGroup::RefreshLabel()
 {
-  if (disabled_header_count_)
+  if (_disabled_header_count)
   {
     _expand_icon->SetVisible(false);
     _expand_label->SetVisible(false);
@@ -403,7 +462,10 @@ long PlacesGroup::ComputeContentSize()
   // only the width matters
   if (_cached_geometry.GetWidth() != geo.GetWidth())
   {
-    _focus_layer.reset(_style.FocusOverlay(geo.width - kHighlightLeftPadding - kHighlightRightPadding, kHighlightHeight));
+    _focus_layer.reset(_style.FocusOverlay(geo.width - 
+                                           kHighlightLeftPadding.CP(scale()) -
+                                           kHighlightRightPadding.CP(scale()),
+                                           kHighlightHeight.CP(scale())));
     _cached_geometry = geo;
   }
   return ret;
@@ -421,8 +483,11 @@ void PlacesGroup::Draw(nux::GraphicsEngine& graphics_engine,
   if (ShouldBeHighlighted() && _focus_layer)
   {
     nux::Geometry geo(_header_layout->GetGeometry());
-    geo.width = base.width - kHighlightRightPadding - kHighlightLeftPadding;
-    geo.x += kHighlightLeftPadding;
+    geo.width = base.width -
+                kHighlightRightPadding.CP(scale()) -
+                kHighlightLeftPadding.CP(scale());
+
+    geo.x += kHighlightLeftPadding.CP(scale());
 
     _focus_layer->SetGeometry(geo);
     _focus_layer->Renderlayer(graphics_engine);
@@ -641,14 +706,13 @@ std::string PlacesGroup::GetName() const
   return "PlacesGroup";
 }
 
-void PlacesGroup::AddProperties(GVariantBuilder* builder)
+void PlacesGroup::AddProperties(debug::IntrospectionData& wrapper)
 {
-  unity::variant::BuilderWrapper wrapper(builder);
-
   wrapper.add("header-x", _header_view->GetAbsoluteX());
   wrapper.add("header-y", _header_view->GetAbsoluteY());
   wrapper.add("header-width", _header_view->GetAbsoluteWidth());
   wrapper.add("header-height", _header_view->GetAbsoluteHeight());
+  wrapper.add("header-geo", _header_view->GetAbsoluteGeometry());
   wrapper.add("header-has-keyfocus", HeaderHasKeyFocus());
   wrapper.add("header-is-highlighted", ShouldBeHighlighted());
   wrapper.add("name", _name->GetText());
@@ -656,22 +720,24 @@ void PlacesGroup::AddProperties(GVariantBuilder* builder)
   wrapper.add("is-expanded", GetExpanded());
   wrapper.add("expand-label-is-visible", _expand_label->IsVisible());
   wrapper.add("expand-label-y", _expand_label->GetAbsoluteY());
+  wrapper.add("expand-label-geo", _expand_label->GetAbsoluteGeometry());
   wrapper.add("expand-label-baseline", _expand_label->GetBaseline());
   wrapper.add("name-label-y", _name->GetAbsoluteY());
   wrapper.add("name-label-baseline", _name->GetBaseline());
+  wrapper.add("name-label-geo", _name->GetAbsoluteGeometry());
 }
 
 glib::Variant PlacesGroup::GetCurrentFocus() const
 {
   if (_header_view && _header_view->HasKeyFocus())
   {
-    return g_variant_new_string("HeaderView");
+    return glib::Variant("HeaderView");
   }
   else if (_child_view && _child_view->HasKeyFocus())
   {
     return g_variant_new("(si)", "ResultView", _child_view->GetSelectedIndex());
   }
-  return glib::Variant();
+  return nullptr;
 }
 
 void PlacesGroup::SetCurrentFocus(glib::Variant const& variant)
