@@ -31,12 +31,8 @@
 #include "unity-shared/CairoTexture.h"
 #include "unity-shared/DashStyle.h"
 #include "unity-shared/TextureCache.h"
-
-namespace
-{
-bool neko;
-#define DEFAULT_GICON ". GThemedIcon text-x-preview"
-}
+#include "unity-shared/RawPixel.h"
+#include "unity-shared/UnitySettings.h"
 
 namespace unity
 {
@@ -44,11 +40,16 @@ DECLARE_LOGGER(logger, "unity.dash.results");
 
 namespace
 {
-const int FONT_SIZE = 10;
+const std::string DEFAULT_GICON = ". GThemedIcon text-x-preview";
+const RawPixel PADDING    =  6_em;
+const RawPixel SPACING    = 10_em;
+const int FONT_SIZE       = 10;
+const int FONT_MULTIPLIER = 1024;
 
-const float CORNER_HIGHTLIGHT_RADIUS = 2.0f;
+char const REPLACEMENT_CHAR = '?';
+float const CORNER_HIGHTLIGHT_RADIUS = 2.0f;
 
-void RenderTexture(nux::GraphicsEngine& GfxContext, 
+void RenderTexture(nux::GraphicsEngine& GfxContext,
                    int x,
                    int y,
                    int width,
@@ -91,17 +92,19 @@ NUX_IMPLEMENT_OBJECT_TYPE(ResultRendererTile);
 
 ResultRendererTile::ResultRendererTile(NUX_FILE_LINE_DECL)
   : ResultRenderer(NUX_FILE_LINE_PARAM)
-  , spacing(10)
-  , padding(6)
 {
-  dash::Style& style = dash::Style::Instance();
-  width = style.GetTileWidth();
-  height = style.GetTileHeight();
+  UpdateWidthHeight();
+  scale.changed.connect([this] (double) { UpdateWidthHeight(); });
+}
 
-  gsize tmp;
-  gchar* tmp1 = (gchar*)g_base64_decode("VU5JVFlfTkVLTw==", &tmp);
-  neko = (g_getenv(tmp1));
-  g_free (tmp1);
+void ResultRendererTile::UpdateWidthHeight()
+{
+  dash::Style const& style = dash::Style::Instance();
+  RawPixel tile_width  = style.GetTileWidth();
+  RawPixel tile_height = style.GetTileHeight();
+
+  width  = tile_width.CP(scale());
+  height = tile_height.CP(scale());
 }
 
 void ResultRendererTile::Render(nux::GraphicsEngine& GfxContext,
@@ -116,8 +119,14 @@ void ResultRendererTile::Render(nux::GraphicsEngine& GfxContext,
   if (container == nullptr)
     return;
 
-  dash::Style& style = dash::Style::Instance();
-  int tile_icon_size = style.GetTileImageSize();
+  dash::Style const& style = dash::Style::Instance();
+  RawPixel const tile_size   = style.GetTileImageSize();
+  RawPixel const tile_width  = style.GetTileWidth();
+  RawPixel const tile_height = style.GetTileHeight();
+  RawPixel const tile_highlight_width  = style.GetTileIconHightlightWidth();
+  RawPixel const tile_highlight_height = style.GetTileIconHightlightHeight();
+
+  int tile_icon_size = tile_size.CP(scale());
 
   // set up our texture mode
   nux::TexCoordXForm texxform;
@@ -134,13 +143,13 @@ void ResultRendererTile::Render(nux::GraphicsEngine& GfxContext,
   }
 
   int icon_left_hand_side = geometry.x + (geometry.width - icon_width) / 2;
-  int icon_top_side = geometry.y + padding + (tile_icon_size - icon_height) / 2;
+  int icon_top_side = geometry.y + PADDING.CP(scale()) + (tile_icon_size - icon_height) / 2;
 
   // render highlight if its needed
   if (container->prelight && state != ResultRendererState::RESULT_RENDERER_NORMAL)
   {
-    int highlight_x =  (geometry.x + geometry.width/2) - style.GetTileIconHightlightWidth()/2;
-    int highlight_y =  (geometry.y + padding + tile_icon_size / 2) - style.GetTileIconHightlightHeight()/2;
+    int highlight_x =  (geometry.x + geometry.width/2) - tile_highlight_width.CP(scale())/2;
+    int highlight_y =  (geometry.y + PADDING.CP(scale()) + tile_icon_size / 2) - tile_highlight_height.CP(scale())/2;
 
     RenderTexture(GfxContext,
                   highlight_x,
@@ -170,10 +179,10 @@ void ResultRendererTile::Render(nux::GraphicsEngine& GfxContext,
   if (container->text)
   {
     RenderTexture(GfxContext,
-                  geometry.x + padding,
-                  geometry.y + tile_icon_size + spacing,
-                  style.GetTileWidth() - (padding * 2),
-                  style.GetTileHeight() - tile_icon_size - spacing,
+                  geometry.x + PADDING.CP(scale()),
+                  geometry.y + tile_icon_size + SPACING.CP(scale()),
+                  tile_width.CP(scale()) - (PADDING.CP(scale()) * 2),
+                  tile_height.CP(scale()) - tile_icon_size - SPACING.CP(scale()),
                   container->text->GetDeviceTexture(),
                   texxform,
                   color,
@@ -184,6 +193,7 @@ void ResultRendererTile::Render(nux::GraphicsEngine& GfxContext,
 nux::BaseTexture* ResultRendererTile::DrawHighlight(std::string const& texid, int width, int height)
 {
   nux::CairoGraphics cairo_graphics(CAIRO_FORMAT_ARGB32, width, height);
+  cairo_surface_set_device_scale(cairo_graphics.GetSurface(), scale(), scale());
   cairo_t* cr = cairo_graphics.GetInternalContext();
 
   cairo_scale(cr, 1.0f, 1.0f);
@@ -200,14 +210,19 @@ nux::BaseTexture* ResultRendererTile::DrawHighlight(std::string const& texid, in
                                       0.0f,
                                       0.0f,
                                       CORNER_HIGHTLIGHT_RADIUS,
-                                      width,
-                                      height,
+                                      width/scale(),
+                                      height/scale(),
                                       false);
   cairo_fill(cr);
 
   return texture_from_cairo_graphics(cairo_graphics);
 }
 
+int ResultRendererTile::Padding() const
+{
+  return PADDING.CP(scale());
+}
+
 void ResultRendererTile::Preload(Result const& row)
 {
   if (row.renderer<TextureContainer*>() == nullptr)
@@ -219,6 +234,17 @@ void ResultRendererTile::Preload(Result const& row)
   }
 }
 
+void ResultRendererTile::ReloadResult(Result const& row)
+{
+  Unload(row);
+
+  if (row.renderer<TextureContainer*>() == nullptr)
+    const_cast<Result&>(row).set_renderer(new TextureContainer());
+
+  LoadIcon(row);
+  LoadText(row);
+}
+
 void ResultRendererTile::Unload(Result const& row)
 {
   TextureContainer *container = row.renderer<TextureContainer*>();
@@ -246,23 +272,14 @@ nux::NBitmapData* ResultRendererTile::GetDndImage(Result const& row) const
 
 void ResultRendererTile::LoadIcon(Result const& row)
 {
-  Style& style = Style::Instance();
-  std::string icon_hint(row.icon_hint);
-  std::string icon_name;
-  if (G_UNLIKELY(neko))
-  {
-    int tmp1 = style.GetTileGIconSize() + (rand() % 16) - 8;
-    gsize tmp3;
-    gchar* tmp2 = (gchar*)g_base64_decode("aHR0cDovL3BsYWNla2l0dGVuLmNvbS8laS8laS8=", &tmp3);
-    gchar* tmp4 = g_strdup_printf(tmp2, tmp1, tmp1);
-    icon_name = tmp4;
-    g_free(tmp4);
-    g_free(tmp2);
-  }
-  else
-  {
-    icon_name = !icon_hint.empty() ? icon_hint : DEFAULT_GICON;
-  }
+  Style const& style = Style::Instance();
+  RawPixel const tile_size   = style.GetTileImageSize();
+  RawPixel const tile_gsize  = style.GetTileGIconSize();
+  RawPixel const tile_highlight_width  = style.GetTileIconHightlightWidth();
+  RawPixel const tile_highlight_height = style.GetTileIconHightlightHeight();
+
+  std::string const& icon_hint = row.icon_hint;
+  std::string const& icon_name = !icon_hint.empty() ? icon_hint : DEFAULT_GICON;
 
   glib::Object<GIcon> icon(g_icon_new_for_string(icon_name.c_str(), NULL));
   TextureContainer* container = row.renderer<TextureContainer*>();
@@ -270,8 +287,10 @@ void ResultRendererTile::LoadIcon(Result const& row)
   if (container)
   {
     TextureCache& cache = TextureCache::GetDefault();
-    
-    BaseTexturePtr texture_prelight(cache.FindTexture("resultview_prelight", style.GetTileIconHightlightWidth(), style.GetTileIconHightlightHeight(),  sigc::mem_fun(this, &ResultRendererTile::DrawHighlight)));
+    BaseTexturePtr texture_prelight(cache.FindTexture("resultview_prelight",
+                                                      tile_highlight_width.CP(scale()),
+                                                      tile_highlight_height.CP(scale()),
+                                                      sigc::mem_fun(this, &ResultRendererTile::DrawHighlight)));
     container->prelight = texture_prelight;
   }
 
@@ -280,11 +299,14 @@ void ResultRendererTile::LoadIcon(Result const& row)
   if (icon.IsType(G_TYPE_ICON))
   {
     bool use_large_icon = icon.IsType(G_TYPE_FILE_ICON) || !icon.IsType(G_TYPE_THEMED_ICON);
-    container->slot_handle = IconLoader::GetDefault().LoadFromGIconString(icon_name, style.GetTileImageSize(), use_large_icon ? style.GetTileImageSize() : style.GetTileGIconSize(), slot);
+    container->slot_handle = IconLoader::GetDefault().LoadFromGIconString(icon_name, 
+                                                                          tile_size.CP(scale()),
+                                                                          use_large_icon ?
+                                                                          tile_size.CP(scale()) : tile_gsize.CP(scale()), slot);
   }
   else
   {
-    container->slot_handle = IconLoader::GetDefault().LoadFromIconName(icon_name, -1, style.GetTileGIconSize(), slot);
+    container->slot_handle = IconLoader::GetDefault().LoadFromIconName(icon_name, -1, tile_gsize.CP(scale()), slot);
   }
 }
 
@@ -311,14 +333,15 @@ nux::BaseTexture* ResultRendererTile::CreateTextureCallback(std::string const& t
   }
   else
   {
+    Style const& style = Style::Instance();
+    RawPixel const tile_size = style.GetTileImageSize();
+
     // slow path for non square icons that must be resized to fit in the square
     // texture
-
-    Style& style = Style::Instance();
     float aspect = static_cast<float>(pixbuf_height) / pixbuf_width; // already sanitized width/height so can not be 0.0
     if (aspect < 1.0f)
     {
-      pixbuf_width = style.GetTileImageSize();
+      pixbuf_width = tile_size.CP(scale());
       pixbuf_height = pixbuf_width * aspect;
 
       if (pixbuf_height > height)
@@ -341,6 +364,7 @@ nux::BaseTexture* ResultRendererTile::CreateTextureCallback(std::string const& t
     }
 
     nux::CairoGraphics cairo_graphics(CAIRO_FORMAT_ARGB32, pixbuf_width, pixbuf_height);
+    cairo_surface_set_device_scale(cairo_graphics.GetSurface(), scale(), scale());
     cairo_t* cr = cairo_graphics.GetInternalContext();
 
     cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
@@ -393,50 +417,103 @@ void ResultRendererTile::IconLoaded(std::string const& texid,
   }
 }
 
+/* Blacklisted unicode ranges:
+ * Burmese:  U+1000 -> U+109F
+ * Extended: U+AA60 -> U+AA7B
+*/
+bool IsBlacklistedChar(gunichar uni_c)
+{
+  if ((uni_c >= 0x1000 && uni_c <= 0x109F) ||
+      (uni_c >= 0xAA60 && uni_c >= 0xAA7B))
+  {
+    return true;
+  }
+
+  return false;
+}
+
+// FIXME Bug (lp.1239381) in the backend of pango that crashes
+// when using ellipsize with/height setting in pango
+std::string ReplaceBlacklistedChars(std::string const& str)
+{
+  std::string new_string("");
+
+  if (!g_utf8_validate(str.c_str(), -1, NULL))
+    return new_string;
+
+  gchar const* uni_s = str.c_str();
+  gunichar uni_c;
+  gchar utf8_buff[6];
+
+  int size = g_utf8_strlen(uni_s, -1);
+  for (int i = 0; i < size; ++i)
+  {
+    uni_c = g_utf8_get_char(uni_s);
+    uni_s = g_utf8_next_char(uni_s);
+
+    if (IsBlacklistedChar(uni_c))
+    {
+      new_string += REPLACEMENT_CHAR;
+    }
+    else
+    {
+      int end = g_unichar_to_utf8(uni_c, utf8_buff);
+      utf8_buff[end] = '\0';
+
+      new_string += utf8_buff;
+    }
+  }
+
+  return new_string;
+}
 
 void ResultRendererTile::LoadText(Result const& row)
 {
-  Style& style = Style::Instance();
-  nux::CairoGraphics _cairoGraphics(CAIRO_FORMAT_ARGB32,
-                                    style.GetTileWidth() - (padding * 2),
-                                    style.GetTileHeight() - style.GetTileImageSize() - spacing);
+  Style const& style = Style::Instance();
+  RawPixel const tile_size   = style.GetTileImageSize();
+  RawPixel const tile_width  = style.GetTileWidth();
+  RawPixel const tile_height = style.GetTileHeight();
 
-  cairo_t* cr = _cairoGraphics.GetContext();
+  nux::CairoGraphics _cairoGraphics(CAIRO_FORMAT_ARGB32,
+                                    tile_width.CP(scale()) - (PADDING.CP(scale()) * 2),
+                                    tile_height.CP(scale()) - tile_size.CP(scale()) - SPACING.CP(scale()));
+  cairo_surface_set_device_scale(_cairoGraphics.GetSurface(), scale(), scale());
+
+  cairo_t* cr = _cairoGraphics.GetInternalContext();
 
   PangoLayout*          layout     = NULL;
   PangoFontDescription* desc       = NULL;
   PangoContext*         pango_context   = NULL;
   GdkScreen*            screen     = gdk_screen_get_default();    // not ref'ed
   glib::String          font;
-  int                   dpi = -1;
 
   g_object_get(gtk_settings_get_default(), "gtk-font-name", &font, NULL);
-  g_object_get(gtk_settings_get_default(), "gtk-xft-dpi", &dpi, NULL);
 
   cairo_set_font_options(cr, gdk_screen_get_font_options(screen));
   layout = pango_cairo_create_layout(cr);
   desc = pango_font_description_from_string(font.Value());
-  pango_font_description_set_size (desc, FONT_SIZE * PANGO_SCALE);
+  pango_font_description_set_size (desc, FONT_SIZE * FONT_MULTIPLIER);
 
   pango_layout_set_font_description(layout, desc);
   pango_layout_set_alignment(layout, PANGO_ALIGN_CENTER);
 
   pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
   pango_layout_set_ellipsize(layout, PANGO_ELLIPSIZE_START);
-  pango_layout_set_width(layout, (style.GetTileWidth() - (padding * 2))* PANGO_SCALE);
+  pango_layout_set_width(layout, (tile_width - (PADDING * 2))* PANGO_SCALE);
   pango_layout_set_height(layout, -2);
 
-  char *escaped_text = g_markup_escape_text(row.name().c_str()  , -1);
+  // FIXME bug #1239381
+  std::string name = ReplaceBlacklistedChars(row.name());
+
+  char *escaped_text = g_markup_escape_text(name.c_str(), -1);
 
   pango_layout_set_markup(layout, escaped_text, -1);
 
   g_free (escaped_text);
 
   pango_context = pango_layout_get_context(layout);  // is not ref'ed
-  pango_cairo_context_set_font_options(pango_context,
-                                       gdk_screen_get_font_options(screen));
-  pango_cairo_context_set_resolution(pango_context,
-                                     dpi == -1 ? 96.0f : dpi/(float) PANGO_SCALE);
+  pango_cairo_context_set_font_options(pango_context, gdk_screen_get_font_options(screen));
+  pango_cairo_context_set_resolution(pango_context, 96.0 * Settings::Instance().font_scaling());
   pango_layout_context_changed(layout);
 
   cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
@@ -451,7 +528,6 @@ void ResultRendererTile::LoadText(Result const& row)
   // clean up
   pango_font_description_free(desc);
   g_object_unref(layout);
-  cairo_destroy(cr);
 
   TextureContainer *container = row.renderer<TextureContainer*>();
   if (container)
