@@ -60,6 +60,7 @@ public:
   NUX_DECLARE_OBJECT_TYPE(ResultRendererTile, ResultRenderer);
 
   ResultRendererTile(NUX_FILE_LINE_PROTO);
+  virtual ~ResultRendererTile() {}
 
   virtual void Render(nux::GraphicsEngine& GfxContext,
                       Result& row,
@@ -71,11 +72,11 @@ public:
 
   virtual void Preload(Result const& row);
   virtual void Unload(Result const& row);
-  
   virtual nux::NBitmapData* GetDndImage(Result const& row) const;
 
-  int spacing;
-  int padding;
+  void ReloadResult(Result const& row);
+
+  int Padding() const;
 
 protected:
   virtual void LoadText(Result const& row);
@@ -92,6 +93,8 @@ private:
                                           glib::Object<GdkPixbuf> const& pixbuf);
   nux::BaseTexture* DrawHighlight(std::string const& texid,
                                   int width, int height);
+
+  void UpdateWidthHeight();
 };
 
 }
