@@ -28,12 +28,12 @@
 #include <gdk/gdk.h>
 #include <unity-protocol.h>
 
-#include <UnityCore/Variant.h>
 #include "unity-shared/IntrospectableWrappers.h"
 #include "unity-shared/Timer.h"
 #include "unity-shared/UBusWrapper.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/GraphicsUtils.h"
+#include "unity-shared/RawPixel.h"
 #include "unity-shared/UnitySettings.h"
 #include "unity-shared/WindowManager.h"
 #include "ResultViewGrid.h"
@@ -55,6 +55,9 @@ namespace
   const float FOCUSED_ICON_SATURATION_REF = 0.5f;
 
   const int DOUBLE_CLICK_SPEED = 500; //500 ms (double-click speed hardcoded to 400 ms in nux)
+
+  const RawPixel WIDTH_PADDING   = 25_em;
+  const RawPixel SCROLLBAR_WIDTH =  3_em;
 }
 
 NUX_IMPLEMENT_OBJECT_TYPE(ResultViewGrid);
@@ -81,16 +84,17 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
   EnableDoubleClick(true);
   SetAcceptKeyNavFocusOnMouseDown(false);
 
-  auto needredraw_lambda = [&](int value) { NeedRedraw(); };
+  auto needredraw_lambda = [this](int value) { NeedRedraw(); };
   horizontal_spacing.changed.connect(needredraw_lambda);
   vertical_spacing.changed.connect(needredraw_lambda);
   padding.changed.connect(needredraw_lambda);
   selected_index_.changed.connect(needredraw_lambda);
-  expanded.changed.connect([&](bool value) { if (value) all_results_preloaded_ = false; });
-  results_per_row.changed.connect([&](int value) { if (value > 0) all_results_preloaded_ = false; });
+  expanded.changed.connect([this](bool value) { if (value) all_results_preloaded_ = false; });
+  results_per_row.changed.connect([this](int value) { if (value > 0) all_results_preloaded_ = false; });
+  scale.changed.connect(sigc::mem_fun(this, &ResultViewGrid::UpdateScale));
 
   key_nav_focus_change.connect(sigc::mem_fun(this, &ResultViewGrid::OnKeyNavFocusChange));
-  key_nav_focus_activate.connect([&] (nux::Area *area)
+  key_nav_focus_activate.connect([this] (nux::Area *area)
   {
     Activate(focused_result_, selected_index_, ResultView::ActivateType::DIRECT);
   });
@@ -99,7 +103,7 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
   mouse_click.connect(sigc::mem_fun(this, &ResultViewGrid::MouseClick));
   mouse_double_click.connect(sigc::mem_fun(this, &ResultViewGrid::MouseDoubleClick));
 
-  mouse_down.connect([&](int x, int y, unsigned long mouse_state, unsigned long button_state)
+  mouse_down.connect([this](int x, int y, unsigned long mouse_state, unsigned long button_state)
   {
     last_mouse_down_x_ = x;
     last_mouse_down_y_ = y;
@@ -107,7 +111,7 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
     mouse_over_index_ = index;
   });
 
-  mouse_leave.connect([&](int x, int y, unsigned long mouse_state, unsigned long button_state)
+  mouse_leave.connect([this](int x, int y, unsigned long mouse_state, unsigned long button_state)
   {
     mouse_over_index_ = -1;
     mouse_last_x_ = -1;
@@ -123,7 +127,7 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
     g_variant_get (data, "(ii)", &recorded_dash_width_, &recorded_dash_height_);
   });
 
-  ubus_.RegisterInterest(UBUS_DASH_PREVIEW_NAVIGATION_REQUEST, [&] (GVariant* data) {
+  ubus_.RegisterInterest(UBUS_DASH_PREVIEW_NAVIGATION_REQUEST, [this] (GVariant* data) {
     int nav_mode = 0;
     GVariant* local_result_variant = NULL;
     glib::String proposed_unique_id;
@@ -348,7 +352,7 @@ void ResultViewGrid::SizeReallocate()
 
   int width = (items_per_row * renderer_->width) + (padding*2) + ((items_per_row - 1) * horizontal_spacing);
   int geo_width = GetBaseWidth();
-  int extra_width = geo_width - (width + 25-3);
+  int extra_width = geo_width - (width + WIDTH_PADDING.CP(scale()) - SCROLLBAR_WIDTH.CP(scale()));
 
   if (items_per_row != 1)
     extra_horizontal_spacing_ = extra_width / (items_per_row - 1);
@@ -1007,6 +1011,12 @@ ResultViewGrid::SetSelectedIndex(int index)
   selected_index_ = index;
 }
 
+void
+ResultViewGrid::UpdateScale(double scale)
+{
+  UpdateRenderTextures();
+}
+
 void
 ResultViewGrid::UpdateRenderTextures()
 {
