@@ -1,6 +1,6 @@
 // -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright (C) 2010, 2011 Canonical Ltd
+ * Copyright (C) 2010-2014 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -31,6 +31,7 @@
 #include "unity-shared/UBusWrapper.h"
 #include "unity-shared/PlacesOverlayVScrollBar.h"
 #include "unity-shared/GraphicsUtils.h"
+#include "unity-shared/RawPixel.h"
 
 #include "config.h"
 #include <glib/gi18n-lib.h>
@@ -44,8 +45,10 @@ DECLARE_LOGGER(focus_logger, "unity.dash.scopeview.focus");
 
 namespace
 {
-const int CARD_VIEW_GAP_VERT  = 24; // pixels
-const int CARD_VIEW_GAP_HORIZ = 25; // pixels
+const RawPixel CARD_VIEW_GAP_VERT  = 24_em; // pixels
+const RawPixel CARD_VIEW_GAP_HORIZ = 25_em; // pixels
+const RawPixel FOCUSED_OFFSET      = 30_em;
+const double DEFAULT_SCALE         = 1.0;
 }
 
 // This is so we can access some protected members in scrollview.
@@ -59,7 +62,7 @@ public:
   {
     SetVScrollBar(scroll_bar);
 
-    OnVisibleChanged.connect([&] (nux::Area* /*area*/, bool visible) {
+    OnVisibleChanged.connect([this] (nux::Area* /*area*/, bool visible) {
       if (m_horizontal_scrollbar_enable)
         _hscrollbar->SetVisible(visible);
       if (m_vertical_scrollbar_enable)
@@ -150,6 +153,7 @@ ScopeView::ScopeView(Scope::Ptr const& scope, nux::Area* show_filters)
 : nux::View(NUX_TRACKER_LOCATION)
 , filters_expanded(false)
 , can_refine_search(false)
+, scale(DEFAULT_SCALE)
 , scope_(scope)
 , no_results_active_(false)
 , last_good_filter_model_(-1)
@@ -163,6 +167,7 @@ ScopeView::ScopeView(Scope::Ptr const& scope, nux::Area* show_filters)
   search_string.SetGetterFunction(sigc::mem_fun(this, &ScopeView::get_search_string));
   filters_expanded.changed.connect(sigc::mem_fun(this, &ScopeView::OnScopeFilterExpanded));
   view_type.changed.connect(sigc::mem_fun(this, &ScopeView::OnViewTypeChanged));
+  scale.changed.connect(sigc::mem_fun(this, &ScopeView::UpdateScale));
 
   auto conn = nux::GetWindowCompositor().key_nav_focus_change.connect(sigc::mem_fun(this, &ScopeView::OnCompositorKeyNavFocusChanged));
   key_nav_focus_connection_ = conn_manager_.Add(conn);
@@ -178,7 +183,7 @@ ScopeView::ScopeView(Scope::Ptr const& scope, nux::Area* show_filters)
     conn_manager_.Add(scope_->filters.changed.connect(sigc::mem_fun(this, &ScopeView::SetupFilters)));
     SetupFilters(scope->filters);
 
-    scope_->connected.changed.connect([&](bool is_connected)
+    scope_->connected.changed.connect([this](bool is_connected)
     {
       // We need to search again if we were reconnected after being connected before.
       if (scope_connected_ && !is_connected)
@@ -193,7 +198,7 @@ ScopeView::ScopeView(Scope::Ptr const& scope, nux::Area* show_filters)
     });
   }
 
-  ubus_manager_.RegisterInterest(UBUS_RESULT_VIEW_KEYNAV_CHANGED, [&] (GVariant* data) {
+  ubus_manager_.RegisterInterest(UBUS_RESULT_VIEW_KEYNAV_CHANGED, [this] (GVariant* data) {
     // we get this signal when a result view keynav changes,
     // its a bad way of doing this but nux ABI needs to be broken
     // to do it properly
@@ -211,8 +216,8 @@ ScopeView::ScopeView(Scope::Ptr const& scope, nux::Area* show_filters)
             (expand_label && expand_label->HasKeyFocus()))
         {
           focused_pos.x += child->GetGeometry().x;
-          focused_pos.y += child->GetGeometry().y - 30;
-          focused_pos.height += 30;
+          focused_pos.y += child->GetGeometry().y - FOCUSED_OFFSET.CP(scale());
+          focused_pos.height += FOCUSED_OFFSET.CP(scale());
           scroll_view_->ScrollToPosition(focused_pos);
           break;
         }
@@ -220,7 +225,7 @@ ScopeView::ScopeView(Scope::Ptr const& scope, nux::Area* show_filters)
     }
   });
 
-  OnVisibleChanged.connect([&] (nux::Area* area, bool visible) {
+  OnVisibleChanged.connect([this] (nux::Area* area, bool visible) {
     scroll_view_->SetVisible(visible);
 
     if ((filters_expanded && visible) || !visible)
@@ -230,10 +235,7 @@ ScopeView::ScopeView(Scope::Ptr const& scope, nux::Area* show_filters)
 
 void ScopeView::SetupViews(nux::Area* show_filters)
 {
-  dash::Style& style = dash::Style::Instance();
-
   layout_ = new nux::HLayout(NUX_TRACKER_LOCATION);
-  layout_->SetSpaceBetweenChildren(style.GetSpaceBetweenScopeAndFilters());
 
   scroll_view_ = new ScopeScrollView(new PlacesOverlayVScrollBar(NUX_TRACKER_LOCATION),
                                     NUX_TRACKER_LOCATION);
@@ -261,18 +263,42 @@ void ScopeView::SetupViews(nux::Area* show_filters)
   fscroll_view_->SetLayout(fscroll_layout_);
 
   filter_bar_ = new FilterBar();
-  int width = style.GetFilterBarWidth() +
-              style.GetFilterBarLeftPadding() +
-              style.GetFilterBarRightPadding();
 
-  fscroll_view_->SetMinimumWidth(width + style.GetFilterViewRightPadding());
-  fscroll_view_->SetMaximumWidth(width + style.GetFilterViewRightPadding());
-  filter_bar_->SetMinimumWidth(width);
-  filter_bar_->SetMaximumWidth(width);
   AddChild(filter_bar_);
   fscroll_layout_->AddView(filter_bar_, 0);
 
   SetLayout(layout_);
+
+  UpdateScopeViewSize();
+}
+
+void ScopeView::UpdateScopeViewSize()
+{
+  dash::Style const& style = dash::Style::Instance();
+
+  RawPixel const scope_filter_space = style.GetSpaceBetweenScopeAndFilters();
+  RawPixel const right_padding      = style.GetFilterViewRightPadding();
+  RawPixel const filter_width       = style.GetFilterBarWidth() +
+                                      style.GetFilterBarLeftPadding() +
+                                      style.GetFilterBarRightPadding();
+
+  double scale = this->scale();
+  layout_->SetSpaceBetweenChildren(scope_filter_space.CP(scale));
+
+  fscroll_view_->SetMinimumWidth(filter_width.CP(scale) + right_padding.CP(scale));
+  fscroll_view_->SetMaximumWidth(filter_width.CP(scale) + right_padding.CP(scale));
+  filter_bar_->SetMinimumWidth(filter_width.CP(scale));
+  filter_bar_->SetMaximumWidth(filter_width.CP(scale));
+}
+
+void ScopeView::UpdateScale(double scale)
+{
+  UpdateScopeViewSize();
+
+  for (auto& group : category_views_)
+    group->scale = scale;
+
+  filter_bar_->scale = scale;
 }
 
 void ScopeView::SetupCategories(Categories::Ptr const& categories)
@@ -435,6 +461,7 @@ void ScopeView::OnCategoryAdded(Category const& category)
   group->SetIcon(icon_hint);
   group->SetExpanded(false);
   group->SetVisible(false);
+  group->scale = scale();
 
   int view_index = category_order_.size();
   auto find_view_index = std::find(category_order_.begin(), category_order_.end(), index);
@@ -459,8 +486,8 @@ void ScopeView::OnCategoryAdded(Category const& category)
   {
     results_view = new ResultViewGrid(NUX_TRACKER_LOCATION);
     results_view->SetModelRenderer(new ResultRendererHorizontalTile(NUX_TRACKER_LOCATION));
-    static_cast<ResultViewGrid*> (results_view)->horizontal_spacing = CARD_VIEW_GAP_HORIZ;
-    static_cast<ResultViewGrid*> (results_view)->vertical_spacing = CARD_VIEW_GAP_VERT;
+    static_cast<ResultViewGrid*> (results_view)->horizontal_spacing = CARD_VIEW_GAP_HORIZ.CP(scale());
+    static_cast<ResultViewGrid*> (results_view)->vertical_spacing   = CARD_VIEW_GAP_VERT.CP(scale());
   }
   else
   {
@@ -757,7 +784,7 @@ void ScopeView::QueueCategoryCountsCheck()
 {
   if (!model_updated_timeout_)
   {
-    model_updated_timeout_.reset(new glib::Idle([&] () {
+    model_updated_timeout_.reset(new glib::Idle([this] () {
       // Check if all results so far are from one category
       // If so, then expand that category.
       CheckCategoryCounts();
@@ -830,7 +857,7 @@ bool ScopeView::PerformSearch(std::string const& search_query, SearchCallback co
   if (scope_)
   {
     // 150ms to hide the no reults message if its take a while to return results
-    hide_message_delay_.reset(new glib::Timeout(150, [&] () {
+    hide_message_delay_.reset(new glib::Timeout(150, [this] () {
       HideResultsMessage();
       return false;
     }));
@@ -1142,9 +1169,9 @@ std::string ScopeView::GetName() const
   return "ScopeView";
 }
 
-void ScopeView::AddProperties(GVariantBuilder* builder)
+void ScopeView::AddProperties(debug::IntrospectionData& introspection)
 {
-  unity::variant::BuilderWrapper(builder)
+  introspection
     .add("name", scope_->id)
     .add("scope-name", scope_->name)
     .add("visible", IsVisible())
