@@ -25,7 +25,6 @@
 #include <Nux/BaseWindow.h>
 #include <NuxCore/Logger.h>
 
-#include <UnityCore/Variant.h>
 #include <UnityCore/GLibWrapper.h>
 #include <UnityCore/DesktopUtilities.h>
 
@@ -35,6 +34,7 @@
 #include "unity-shared/DesktopApplicationManager.h"
 #include "unity-shared/GnomeFileManager.h"
 #include "unity-shared/UBusMessages.h"
+#include "unity-shared/UScreen.h"
 
 #include <glib/gi18n-lib.h>
 #include <gio/gdesktopappinfo.h>
@@ -69,7 +69,6 @@ ApplicationLauncherIcon::ApplicationLauncherIcon(ApplicationPtr const& app)
   : SimpleLauncherIcon(IconType::APPLICATION)
   , _startup_notification_timestamp(0)
   , _last_scroll_timestamp(0)
-  , _last_scroll_direction(ScrollDirection::DOWN)
   , _progressive_scroll(0)
   , use_custom_bg_color_(false)
   , bg_color_(nux::color::White)
@@ -89,6 +88,7 @@ ApplicationLauncherIcon::ApplicationLauncherIcon(ApplicationPtr const& app)
   wm.window_moved.connect(sigc::mem_fun(this, &ApplicationLauncherIcon::OnWindowMoved));
   wm.screen_viewport_switch_ended.connect(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowState));
   wm.terminate_expo.connect(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowState));
+  UScreen::GetDefault()->changed.connect(sigc::hide(sigc::hide(sigc::mem_fun(this, &ApplicationLauncherIcon::EnsureWindowState))));
 
   EnsureWindowState();
 }
@@ -226,11 +226,11 @@ void ApplicationLauncherIcon::SetupApplicationSignalsConnections()
   }));
 }
 
-bool ApplicationLauncherIcon::GetQuirk(AbstractLauncherIcon::Quirk quirk) const
+bool ApplicationLauncherIcon::GetQuirk(AbstractLauncherIcon::Quirk quirk, int monitor) const
 {
   if (quirk == Quirk::ACTIVE)
   {
-    if (!SimpleLauncherIcon::GetQuirk(Quirk::ACTIVE))
+    if (!SimpleLauncherIcon::GetQuirk(Quirk::ACTIVE, monitor))
       return false;
 
     if (app_->type() == "webapp")
@@ -242,7 +242,7 @@ bool ApplicationLauncherIcon::GetQuirk(AbstractLauncherIcon::Quirk quirk) const
     return app_->OwnsWindow(WindowManager::Default().GetActiveWindow());
   }
 
-  return SimpleLauncherIcon::GetQuirk(quirk);
+  return SimpleLauncherIcon::GetQuirk(quirk, monitor);
 }
 
 void ApplicationLauncherIcon::Remove()
@@ -395,13 +395,37 @@ void ApplicationLauncherIcon::ActivateLauncherIcon(ActionArg arg)
     {
       if (scaleWasActive) // #5 above
       {
-        Focus(arg);
+        if (minimize_window_on_click())
+        {
+          for (auto const& win : GetWindows(WindowFilter::ON_CURRENT_DESKTOP))
+            wm.Minimize(win->window_id());
+        }
+        else
+        {
+          Focus(arg);
+        }
       }
       else // #2 above
       {
         if (arg.source != ActionArg::Source::SWITCHER)
         {
-          Spread(true, 0, false);
+          bool minimized = false;
+
+          if (minimize_window_on_click)
+          {
+            WindowList const& windows = GetWindows(WindowFilter::ON_CURRENT_DESKTOP);
+
+            if (windows.size() == 1)
+            {
+              wm.Minimize(windows[0]->window_id());
+              minimized = true;
+            }
+          }
+
+          if (!minimized)
+          {
+            Spread(true, 0, false);
+          }
         }
       }
     }
@@ -491,11 +515,21 @@ std::vector<Window> ApplicationLauncherIcon::WindowsForMonitor(int monitor)
 
 void ApplicationLauncherIcon::OnWindowMinimized(guint32 xid)
 {
-  if (!app_->OwnsWindow(xid))
-    return;
+  for (auto const& window: app_->GetWindows())
+  {
+    if (xid == window->window_id())
+    {
+      int monitor = GetCenterForMonitor(window->monitor()).first;
 
-  Present(0.5f, 600);
-  UpdateQuirkTimeDelayed(300, Quirk::SHIMMER);
+      if (monitor >= 0)
+      {
+        Present(0.5f, 600, monitor);
+        FullyAnimateQuirkDelayed(300, Quirk::SHIMMER, monitor);
+      }
+
+      break;
+    }
+  }
 }
 
 void ApplicationLauncherIcon::OnWindowMoved(guint32 moved_win)
@@ -586,22 +620,20 @@ std::string ApplicationLauncherIcon::DesktopFile() const
   return app_->desktop_file();
 }
 
-void ApplicationLauncherIcon::AddProperties(GVariantBuilder* builder)
+void ApplicationLauncherIcon::AddProperties(debug::IntrospectionData& introspection)
 {
-  SimpleLauncherIcon::AddProperties(builder);
-
-  GVariantBuilder xids_builder;
-  g_variant_builder_init(&xids_builder, G_VARIANT_TYPE ("au"));
+  SimpleLauncherIcon::AddProperties(introspection);
 
+  std::vector<Window> xids;
   for (auto const& window : GetWindows())
-    g_variant_builder_add(&xids_builder, "u", window->window_id());
+    xids.push_back(window->window_id());
 
-  variant::BuilderWrapper(builder)
+  introspection
     .add("desktop_file", DesktopFile())
     .add("desktop_id", app_->desktop_id())
-    .add("xids", g_variant_builder_end(&xids_builder))
+    .add("xids", glib::Variant::FromVector(xids))
     .add("sticky", IsSticky())
-    .add("startup_notification_timestamp", (uint64_t)_startup_notification_timestamp);
+    .add("startup_notification_timestamp", _startup_notification_timestamp);
 }
 
 void ApplicationLauncherIcon::OpenInstanceWithUris(std::set<std::string> const& uris, Time timestamp)
@@ -650,7 +682,7 @@ void ApplicationLauncherIcon::OpenInstanceWithUris(std::set<std::string> const&
     LOG_WARN(logger) << error;
   }
 
-  UpdateQuirkTime(Quirk::STARTING);
+  FullyAnimateQuirk(Quirk::STARTING);
 }
 
 void ApplicationLauncherIcon::OpenInstanceLauncherIcon(Time timestamp)
@@ -691,7 +723,7 @@ bool ApplicationLauncherIcon::Spread(bool current_desktop, int state, bool force
 
 void ApplicationLauncherIcon::EnsureWindowState()
 {
-  std::vector<bool> monitors(monitors::MAX);
+  std::bitset<monitors::MAX> monitors;
 
   for (auto& window: app_->GetWindows())
   {
@@ -703,18 +735,20 @@ void ApplicationLauncherIcon::EnsureWindowState()
       // If monitor is -1 (or negative), show on all monitors.
       if (monitor < 0)
       {
-        for (unsigned j = 0; j < monitors::MAX; j++)
-          monitors[j] = true;
+        monitors.set();
+        break;
       }
       else
+      {
         monitors[monitor] = true;
+      }
     }
   }
 
   for (unsigned i = 0; i < monitors::MAX; i++)
     SetWindowVisibleOnMonitor(monitors[i], i);
 
-  EmitNeedsRedraw();
+  WindowsChanged.emit();
 }
 
 void ApplicationLauncherIcon::UpdateDesktopQuickList()
@@ -1031,6 +1065,7 @@ AbstractLauncherIcon::MenuItemsVector ApplicationLauncherIcon::GetMenus()
 
     glib::Object<DbusmenuMenuitem> item(dbusmenu_menuitem_new());
     dbusmenu_menuitem_property_set(item, DBUSMENU_MENUITEM_PROP_LABEL, bold_app_name.c_str());
+    dbusmenu_menuitem_property_set(item, DBUSMENU_MENUITEM_PROP_ACCESSIBLE_DESC, app_name.Str().c_str());
     dbusmenu_menuitem_property_set_bool(item, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
     dbusmenu_menuitem_property_set_bool(item, QuicklistMenuItem::MARKUP_ENABLED_PROPERTY, TRUE);
 
@@ -1074,7 +1109,7 @@ AbstractLauncherIcon::MenuItemsVector ApplicationLauncherIcon::GetMenus()
   return result;
 }
 
-void ApplicationLauncherIcon::UpdateIconGeometries(std::vector<nux::Point3> center)
+void ApplicationLauncherIcon::UpdateIconGeometries(std::vector<nux::Point3> const& centers)
 {
   if (app_->type() == "webapp")
     return;
@@ -1084,18 +1119,23 @@ void ApplicationLauncherIcon::UpdateIconGeometries(std::vector<nux::Point3> cent
   for (auto& window : app_->GetWindows())
   {
     Window xid = window->window_id();
-    int monitor = window->monitor();
-    monitor = std::max<int>(0, std::min<int>(center.size() - 1, monitor));
+    int monitor = GetCenterForMonitor(window->monitor()).first;
 
-    geo.x = center[monitor].x - icon_size / 2;
-    geo.y = center[monitor].y - icon_size / 2;
+    if (monitor < 0)
+    {
+      WindowManager::Default().SetWindowIconGeometry(xid, nux::Geometry());
+      continue;
+    }
+
+    geo.x = centers[monitor].x - icon_size / 2;
+    geo.y = centers[monitor].y - icon_size / 2;
     WindowManager::Default().SetWindowIconGeometry(xid, geo);
   }
 }
 
-void ApplicationLauncherIcon::OnCenterStabilized(std::vector<nux::Point3> center)
+void ApplicationLauncherIcon::OnCenterStabilized(std::vector<nux::Point3> const& centers)
 {
-  UpdateIconGeometries(center);
+  UpdateIconGeometries(centers);
 }
 
 void ApplicationLauncherIcon::UpdateRemoteUri()
@@ -1263,19 +1303,17 @@ const std::set<std::string> ApplicationLauncherIcon::GetSupportedTypes()
 
 void PerformScrollUp(WindowList const& windows, unsigned int progressive_scroll)
 {
-  if (!progressive_scroll)
-  {
-    windows.at(1)->Focus();
-    return;
-  }
-  else if (progressive_scroll == 1)
+  if (progressive_scroll == windows.size() - 1)
   {
+    //RestackAbove to preserve Global Stacking Order
+    WindowManager::Default().RestackBelow(windows.at(0)->window_id(), windows.at(1)->window_id());
+    WindowManager::Default().RestackBelow(windows.at(1)->window_id(), windows.at(0)->window_id());
     windows.back()->Focus();
     return;
   }
 
-  WindowManager::Default().RestackBelow(windows.at(0)->window_id(), windows.at(windows.size() - progressive_scroll + 1)->window_id());
-  windows.at(windows.size() - progressive_scroll + 1)->Focus();
+  WindowManager::Default().RestackBelow(windows.at(0)->window_id(), windows.at(progressive_scroll + 1)->window_id());
+  windows.at(progressive_scroll + 1)->Focus();
 }
 
 void PerformScrollDown(WindowList const& windows, unsigned int progressive_scroll)
@@ -1295,26 +1333,33 @@ void ApplicationLauncherIcon::PerformScroll(ScrollDirection direction, Time time
 {
   if (timestamp - _last_scroll_timestamp < 150)
     return;
-  else if (timestamp - _last_scroll_timestamp > 1500 || direction != _last_scroll_direction)
+  else if (timestamp - _last_scroll_timestamp > 1500)
     _progressive_scroll = 0;
 
   _last_scroll_timestamp = timestamp;
-  _last_scroll_direction = direction;
 
   auto const& windows = GetWindowsOnCurrentDesktopInStackingOrder();
+
   if (windows.empty())
     return;
 
-  if (!IsActive())
+  if (scroll_inactive_icons && !IsActive())
   {
     windows.at(0)->Focus();
     return;
   }
 
+  if (!scroll_inactive_icons && !IsActive())
+    return;
+
   if (windows.size() <= 1)
     return;
 
-  ++_progressive_scroll;
+  if (direction == ScrollDirection::DOWN)
+    ++_progressive_scroll;
+  else
+    //--_progressive_scroll; but roll to the top of windows
+    _progressive_scroll += windows.size() - 1;
   _progressive_scroll %= windows.size();
 
   switch(direction)
