@@ -42,8 +42,9 @@ namespace
 
 NUX_IMPLEMENT_OBJECT_TYPE(CairoBaseWindow);
 
-CairoBaseWindow::CairoBaseWindow()
-  : use_blurred_background_(!Settings::Instance().GetLowGfxMode())
+CairoBaseWindow::CairoBaseWindow(int monitor)
+  : cv_(Settings::Instance().em(monitor))
+  , use_blurred_background_(!Settings::Instance().GetLowGfxMode())
   , compute_blur_bkg_(use_blurred_background_)
   , fade_animator_(FADE_DURATION)
 {
@@ -53,7 +54,10 @@ CairoBaseWindow::CairoBaseWindow()
   fade_animator_.updated.connect(sigc::mem_fun(this, &BaseWindow::SetOpacity));
   fade_animator_.finished.connect([this] {
     if (animation::GetDirection(fade_animator_) == animation::Direction::BACKWARD)
+    {
       ShowWindow(false);
+      hidden.emit();
+    }
   });
 }
 
@@ -69,17 +73,17 @@ void CairoBaseWindow::Hide()
   animation::StartOrReverse(fade_animator_, animation::Direction::BACKWARD);
 }
 
-bool CairoBaseWindow::HasBlurredBackground() const
-{
-  return use_blurred_background_;
-}
-
-void CairoBaseWindow::NeedSoftRedraw()
+void CairoBaseWindow::RedrawBlur()
 {
   compute_blur_bkg_ = true;
   QueueDraw();
 }
 
+bool CairoBaseWindow::HasBlurredBackground() const
+{
+  return use_blurred_background_;
+}
+
 void CairoBaseWindow::Draw(nux::GraphicsEngine& gfxContext, bool forceDraw)
 {
   nux::Geometry base(GetGeometry());
