@@ -43,6 +43,8 @@
 #include "unity-shared/UScreen.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/TimeUtil.h"
+#include "unity-shared/PanelStyle.h"
+#include "unity-shared/UnitySettings.h"
 
 namespace unity
 {
@@ -82,7 +84,6 @@ namespace
 {
   const int launcher_minimum_show_duration = 1250;
   const int shortcuts_show_delay = 750;
-  const int ignore_repeat_shortcut_duration = 250;
 
   const std::string KEYPRESS_TIMEOUT = "keypress-timeout";
   const std::string LABELS_TIMEOUT = "label-show-timeout";
@@ -118,7 +119,6 @@ Controller::Impl::Impl(Controller* parent, XdndManager::Ptr const& xdnd_manager,
   , reactivate_keynav(false)
   , keynav_restore_window_(true)
   , launcher_key_press_time_(0)
-  , last_dnd_monitor_(-1)
   , dbus_server_(DBUS_NAME)
 {
 #ifdef USE_X11
@@ -127,7 +127,7 @@ Controller::Impl::Impl(Controller* parent, XdndManager::Ptr const& xdnd_manager,
 
   UScreen* uscreen = UScreen::GetDefault();
   EnsureLaunchers(uscreen->GetPrimaryMonitor(), uscreen->GetMonitors());
-
+  uscreen->changed.connect(sigc::mem_fun(this, &Controller::Impl::EnsureLaunchers));
   SetupIcons();
 
   remote_model_.entry_added.connect(sigc::mem_fun(this, &Impl::OnLauncherEntryRemoteAdded));
@@ -139,6 +139,7 @@ Controller::Impl::Impl(Controller* parent, XdndManager::Ptr const& xdnd_manager,
 
   HudLauncherIcon* hud = new HudLauncherIcon(hide_mode);
   RegisterIcon(AbstractLauncherIcon::Ptr(hud));
+  hud_icon_ = hud;
 
   TrashLauncherIcon* trash = new TrashLauncherIcon();
   RegisterIcon(AbstractLauncherIcon::Ptr(trash));
@@ -148,8 +149,6 @@ Controller::Impl::Impl(Controller* parent, XdndManager::Ptr const& xdnd_manager,
     hud->SetHideMode(mode);
   });
 
-  uscreen->changed.connect(sigc::mem_fun(this, &Controller::Impl::OnScreenChanged));
-
   WindowManager& wm = WindowManager::Default();
   wm.window_focus_changed.connect(sigc::mem_fun(this, &Controller::Impl::OnWindowFocusChanged));
   wm.viewport_layout_changed.connect(sigc::group(sigc::mem_fun(this, &Controller::Impl::UpdateNumWorkspaces), sigc::_1 * sigc::_2));
@@ -166,8 +165,7 @@ Controller::Impl::Impl(Controller* parent, XdndManager::Ptr const& xdnd_manager,
 
     if (selected)
     {
-      ubus.SendMessage(UBUS_LAUNCHER_SELECTION_CHANGED,
-                       g_variant_new_string(selected->tooltip_text().c_str()));
+      ubus.SendMessage(UBUS_LAUNCHER_SELECTION_CHANGED, glib::Variant(selected->tooltip_text()));
     }
   });
 
@@ -201,6 +199,10 @@ void Controller::Impl::EnsureLaunchers(int primary, std::vector<nux::Geometry> c
   unsigned int launchers_size = launchers.size();
   unsigned int last_launcher = 0;
 
+  // Reset the icon centers: only the used icon centers must contain valid values
+  for (auto const& icon : *model_)
+    icon->ResetCenters();
+
   for (unsigned int i = 0; i < num_launchers; ++i, ++last_launcher)
   {
     if (i >= launchers_size)
@@ -248,16 +250,11 @@ void Controller::Impl::EnsureLaunchers(int primary, std::vector<nux::Geometry> c
   launchers.resize(num_launchers);
 }
 
-void Controller::Impl::OnScreenChanged(int primary_monitor, std::vector<nux::Geometry>& monitors)
-{
-  EnsureLaunchers(primary_monitor, monitors);
-}
-
 void Controller::Impl::OnWindowFocusChanged(guint32 xid)
 {
   static bool keynav_first_focus = false;
 
-  if (parent_->IsOverlayOpen() || launcher_->GetParent()->GetInputWindowId() == xid)
+  if (parent_->IsOverlayOpen() || CurrentLauncher()->GetParent()->GetInputWindowId() == xid)
     keynav_first_focus = false;
 
   if (keynav_first_focus)
@@ -276,8 +273,7 @@ void Controller::Impl::OnDndStarted(std::string const& data, int monitor)
 {
   if (parent_->multiple_launchers)
   {
-    last_dnd_monitor_ = monitor;
-    launchers[last_dnd_monitor_]->DndStarted(data);
+    launchers[monitor]->DndStarted(data);
   }
   else
   {
@@ -289,8 +285,8 @@ void Controller::Impl::OnDndFinished()
 {
   if (parent_->multiple_launchers)
   {
-    launchers[last_dnd_monitor_]->DndFinished();
-    last_dnd_monitor_ = -1;
+    if (xdnd_manager_->Monitor() >= 0)
+      launchers[xdnd_manager_->Monitor()]->DndFinished();
   }
   else
   {
@@ -298,14 +294,15 @@ void Controller::Impl::OnDndFinished()
   }
 }
 
-void Controller::Impl::OnDndMonitorChanged(int monitor)
+void Controller::Impl::OnDndMonitorChanged(std::string const& data, int old_monitor, int new_monitor)
 {
   if (parent_->multiple_launchers)
   {
-    launchers[last_dnd_monitor_]->UnsetDndQuirk();
-    last_dnd_monitor_ = monitor;
-    launchers[last_dnd_monitor_]->SetDndQuirk();
- }
+    if (old_monitor >= 0)
+      launchers[old_monitor]->UnsetDndQuirk();
+
+    launchers[new_monitor]->DndStarted(data);
+  }
 }
 
 Launcher* Controller::Impl::CreateLauncher()
@@ -761,7 +758,7 @@ void Controller::Impl::RegisterIcon(AbstractLauncherIcon::Ptr const& icon, int p
 
   if (icon->GetIconType() == AbstractLauncherIcon::IconType::APPLICATION)
   {
-    icon->visibility_changed.connect(sigc::mem_fun(this, &Impl::SortAndUpdate));
+    icon->visibility_changed.connect(sigc::hide(sigc::mem_fun(this, &Impl::SortAndUpdate)));
     SortAndUpdate();
   }
 
@@ -939,7 +936,6 @@ SoftwareCenterLauncherIcon::Ptr Controller::Impl::CreateSCLauncherIcon(std::stri
     return result;
 
   result = new SoftwareCenterLauncherIcon(app, aptdaemon_trans_id, icon_path);
-  result->Stick(false);
 
   return result;
 }
@@ -954,6 +950,7 @@ void Controller::Impl::AddRunningApps()
     if (!app->seen())
     {
       AbstractLauncherIcon::Ptr icon(new ApplicationLauncherIcon(app));
+      icon->SkipQuirkAnimation(AbstractLauncherIcon::Quirk::VISIBLE);
       RegisterIcon(icon, ++sort_priority_);
     }
   }
@@ -965,7 +962,10 @@ void Controller::Impl::AddDevices()
   for (auto const& icon : device_section_.GetIcons())
   {
     if (!icon->IsSticky() && !fav_store.IsFavorite(icon->RemoteUri()))
+    {
+      icon->SkipQuirkAnimation(AbstractLauncherIcon::Quirk::VISIBLE);
       RegisterIcon(icon, ++sort_priority_);
+    }
   }
 }
 
@@ -1014,7 +1014,14 @@ void Controller::Impl::SetupIcons()
     }
 
     LOG_INFO(logger) << "Adding favourite: " << fav_uri;
-    RegisterIcon(CreateFavoriteIcon(fav_uri), ++sort_priority_);
+
+    auto const& icon = CreateFavoriteIcon(fav_uri);
+
+    if (icon)
+    {
+      icon->SkipQuirkAnimation(AbstractLauncherIcon::Quirk::VISIBLE);
+      RegisterIcon(icon, ++sort_priority_);
+    }
   }
 
   if (!running_apps_added)
@@ -1053,12 +1060,13 @@ Controller::Controller(XdndManager::Ptr const& xdnd_manager, ui::EdgeBarrierCont
  , multiple_launchers(true)
  , pimpl(new Impl(this, xdnd_manager, edge_barriers))
 {
-  multiple_launchers.changed.connect([&](bool value) -> void {
+  multiple_launchers.changed.connect([this] (bool value) {
     UScreen* uscreen = UScreen::GetDefault();
     auto monitors = uscreen->GetMonitors();
     int primary = uscreen->GetPrimaryMonitor();
     pimpl->EnsureLaunchers(primary, monitors);
     options()->show_for_all = !value;
+    pimpl->hud_icon_->SetSingleLauncher(!value, primary);
   });
 }
 
@@ -1152,7 +1160,7 @@ void Controller::HandleLauncherKeyPress(int when)
 {
   pimpl->launcher_key_press_time_ = when;
 
-  auto show_launcher = [&]()
+  auto show_launcher = [this]
   {
     if (pimpl->keyboard_launcher_.IsNull())
       pimpl->keyboard_launcher_ = pimpl->CurrentLauncher();
@@ -1165,7 +1173,7 @@ void Controller::HandleLauncherKeyPress(int when)
   };
   pimpl->sources_.AddTimeout(options()->super_tap_duration, show_launcher, local::KEYPRESS_TIMEOUT);
 
-  auto show_shortcuts = [&]()
+  auto show_shortcuts = [this]
   {
     if (!pimpl->launcher_keynav)
     {
@@ -1191,7 +1199,10 @@ bool Controller::AboutToShowDash(int was_tap, int when) const
 void Controller::HandleLauncherKeyRelease(bool was_tap, int when)
 {
   int tap_duration = when - pimpl->launcher_key_press_time_;
-  if (tap_duration < options()->super_tap_duration && was_tap)
+  WindowManager& wm = WindowManager::Default();
+
+  if (tap_duration < options()->super_tap_duration && was_tap &&
+      !wm.IsTopWindowFullscreenOnMonitorWithMouse())
   {
     LOG_DEBUG(logger) << "Quick tap, sending activation request.";
     pimpl->SendHomeActivationRequest();
@@ -1221,7 +1232,7 @@ void Controller::HandleLauncherKeyRelease(bool was_tap, int when)
     {
       int time_left = local::launcher_minimum_show_duration - ms_since_show;
 
-      auto hide_launcher = [&]()
+      auto hide_launcher = [this]
       {
         if (pimpl->keyboard_launcher_.IsValid())
         {
@@ -1240,25 +1251,26 @@ void Controller::HandleLauncherKeyRelease(bool was_tap, int when)
   }
 }
 
-bool Controller::HandleLauncherKeyEvent(Display *display, unsigned int key_sym, unsigned long key_code, unsigned long key_state, char* key_string, Time timestamp)
+bool Controller::HandleLauncherKeyEvent(unsigned long key_state, unsigned int key_sym, Time timestamp)
 {
-  LauncherModel::iterator it;
+  Display* display = nux::GetGraphicsDisplay()->GetX11Display();
+  // Turn the key_sym back to a keycode, this turns keypad key_sym to the correct top row key_code
+  unsigned int key_code = XKeysymToKeycode(display, key_sym);
 
   // Shortcut to start launcher icons. Only relies on Keycode, ignore modifier
-  for (it = pimpl->model_->begin(); it != pimpl->model_->end(); ++it)
+  for (auto const& icon : *pimpl->model_)
   {
-    if ((XKeysymToKeycode(display, (*it)->GetShortcut()) == key_code) ||
-        ((gchar)((*it)->GetShortcut()) == key_string[0]))
+    unsigned int shortcut_code = XKeysymToKeycode(display, icon->GetShortcut());
+    if (shortcut_code == key_code)
     {
-      struct timespec last_action_time = (*it)->GetQuirkTime(AbstractLauncherIcon::Quirk::LAST_ACTION);
-      struct timespec current;
-      TimeUtil::SetTimeStruct(&current);
-      if (TimeUtil::TimeDelta(&current, &last_action_time) > local::ignore_repeat_shortcut_duration)
+      if ((key_state & nux::KEY_MODIFIER_SHIFT) &&
+          icon->GetIconType() == AbstractLauncherIcon::IconType::APPLICATION)
       {
-        if (g_ascii_isdigit((gchar)(*it)->GetShortcut()) && (key_state & ShiftMask))
-          (*it)->OpenInstance(ActionArg(ActionArg::Source::LAUNCHER, 0, timestamp));
-        else
-          (*it)->Activate(ActionArg(ActionArg::Source::LAUNCHER, 0, timestamp));
+        icon->OpenInstance(ActionArg(ActionArg::Source::LAUNCHER_KEYBINDING, 0, timestamp));
+      }
+      else
+      {
+        icon->Activate(ActionArg(ActionArg::Source::LAUNCHER_KEYBINDING, 0, timestamp));
       }
 
       // disable the "tap on super" check
@@ -1304,12 +1316,12 @@ void Controller::KeyNavActivate()
   if (pimpl->launcher_grabbed)
   {
     pimpl->ubus.SendMessage(UBUS_LAUNCHER_START_KEY_NAV,
-                            g_variant_new_int32(pimpl->keyboard_launcher_->monitor));
+                            glib::Variant(pimpl->keyboard_launcher_->monitor()));
   }
   else
   {
     pimpl->ubus.SendMessage(UBUS_LAUNCHER_START_KEY_SWITCHER,
-                            g_variant_new_int32(pimpl->keyboard_launcher_->monitor));
+                            glib::Variant(pimpl->keyboard_launcher_->monitor()));
   }
 
   AbstractLauncherIcon::Ptr const& selected = pimpl->model_->Selection();
@@ -1317,7 +1329,7 @@ void Controller::KeyNavActivate()
   if (selected)
   {
     pimpl->ubus.SendMessage(UBUS_LAUNCHER_SELECTION_CHANGED,
-                            g_variant_new_string(selected->tooltip_text().c_str()));
+                            glib::Variant(selected->tooltip_text()));
   }
 }
 
@@ -1330,7 +1342,7 @@ void Controller::KeyNavNext()
   if (selected)
   {
     pimpl->ubus.SendMessage(UBUS_LAUNCHER_SELECTION_CHANGED,
-                            g_variant_new_string(selected->tooltip_text().c_str()));
+                            glib::Variant(selected->tooltip_text()));
   }
 }
 
@@ -1343,7 +1355,7 @@ void Controller::KeyNavPrevious()
   if (selected)
   {
     pimpl->ubus.SendMessage(UBUS_LAUNCHER_SELECTION_CHANGED,
-                            g_variant_new_string(selected->tooltip_text().c_str()));
+                            glib::Variant(selected->tooltip_text()));
   }
 }
 
@@ -1368,12 +1380,12 @@ void Controller::KeyNavTerminate(bool activate)
     pimpl->launcher_event_outside_connection_->disconnect();
     pimpl->launcher_grabbed = false;
     pimpl->ubus.SendMessage(UBUS_LAUNCHER_END_KEY_NAV,
-                            g_variant_new_boolean(pimpl->keynav_restore_window_));
+                            glib::Variant(pimpl->keynav_restore_window_));
   }
   else
   {
     pimpl->ubus.SendMessage(UBUS_LAUNCHER_END_KEY_SWITCHER,
-                            g_variant_new_boolean(pimpl->keynav_restore_window_));
+                            glib::Variant(pimpl->keynav_restore_window_));
   }
 
   if (activate)
@@ -1406,6 +1418,12 @@ bool Controller::IsOverlayOpen() const
   return false;
 }
 
+void Controller::ClearTooltips()
+{
+  for (auto launcher_ptr : pimpl->launchers)
+    launcher_ptr->ClearTooltip();
+}
+
 std::string
 Controller::GetName() const
 {
@@ -1413,12 +1431,12 @@ Controller::GetName() const
 }
 
 void
-Controller::AddProperties(GVariantBuilder* builder)
+Controller::AddProperties(debug::IntrospectionData& introspection)
 {
   timespec current;
   clock_gettime(CLOCK_MONOTONIC, &current);
 
-  variant::BuilderWrapper(builder)
+  introspection
   .add("key_nav_is_active", KeyNavIsActive())
   .add("key_nav_launcher_monitor", pimpl->keyboard_launcher_.IsValid() ?  pimpl->keyboard_launcher_->monitor : -1)
   .add("key_nav_selection", pimpl->model_->SelectionIndex())
