@@ -23,11 +23,11 @@
 #include <NuxCore/Color.h>
 #include <NuxCore/Logger.h>
 
-#include "Launcher.h"
 #include "LauncherIcon.h"
 #include "unity-shared/AnimationUtils.h"
 #include "unity-shared/CairoTexture.h"
-#include "unity-shared/TimeUtil.h"
+#include "unity-shared/UnitySettings.h"
+#include "unity-shared/UScreen.h"
 
 #include "QuicklistManager.h"
 #include "QuicklistMenuItem.h"
@@ -50,6 +50,8 @@ DECLARE_LOGGER(logger, "unity.launcher.icon");
 
 namespace
 {
+const int IGNORE_REPEAT_SHORTCUT_DURATION = 250;
+
 const std::string DEFAULT_ICON = "application-default-icon";
 const std::string MONO_TEST_ICON = "gnome-home";
 const std::string UNITY_THEME_NAME = "unity-icon-theme";
@@ -71,27 +73,20 @@ LauncherIcon::LauncherIcon(IconType type)
   , _present_urgency(0)
   , _progress(0.0f)
   , _sort_priority(DefaultPriority(type))
+  , _order(0)
   , _last_monitor(0)
   , _background_color(nux::color::White)
   , _glow_color(nux::color::White)
   , _shortcut(0)
+  , _allow_quicklist_to_show(true)
   , _center(monitors::MAX)
-  , _has_visible_window(monitors::MAX, false)
-  , _is_visible_on_monitor(monitors::MAX, true)
+  , _quirks(monitors::MAX)
+  , _quirk_animations(monitors::MAX, decltype(_quirk_animations)::value_type(unsigned(Quirk::LAST)))
   , _last_stable(monitors::MAX)
   , _saved_center(monitors::MAX)
-  , _allow_quicklist_to_show(true)
 {
-  for (unsigned i = 0; i < unsigned(Quirk::LAST); ++i)
-  {
-    _quirks[i] = false;
-    _quirk_times[i].tv_sec = 0;
-    _quirk_times[i].tv_nsec = 0;
-  }
-
   tooltip_enabled = true;
   tooltip_enabled.changed.connect(sigc::mem_fun(this, &LauncherIcon::OnTooltipEnabledChanged));
-  tooltip_text.SetSetterFunction(sigc::mem_fun(this, &LauncherIcon::SetTooltipText));
 
   position = Position::FLOATING;
   removed = false;
@@ -103,23 +98,44 @@ LauncherIcon::LauncherIcon(IconType type)
   mouse_down.connect(sigc::mem_fun(this, &LauncherIcon::RecvMouseDown));
   mouse_up.connect(sigc::mem_fun(this, &LauncherIcon::RecvMouseUp));
   mouse_click.connect(sigc::mem_fun(this, &LauncherIcon::RecvMouseClick));
+
+  for (unsigned i = 0; i < monitors::MAX; ++i)
+  {
+    for (unsigned j = 0; j < static_cast<unsigned>(Quirk::LAST); ++j)
+    {
+      _quirk_animations[i][j] = std::make_shared<Animation>();
+      _quirk_animations[i][j]->updated.connect([this, i, j] (float value) { EmitNeedsRedraw(i); });
+    }
+
+    // Center must have set a default value
+    animation::SetValue(GetQuirkAnimation(Quirk::CENTER_SAVED, i), animation::Direction::FORWARD);
+  }
 }
 
 void LauncherIcon::LoadTooltip()
 {
-  _tooltip = new Tooltip();
-  _tooltip->SetOpacity(0.0f);
-  AddChild(_tooltip.GetPointer());
+  int monitor = _last_monitor;
+  if (monitor < 0)
+    monitor = 0;
 
+  _tooltip = new Tooltip(monitor);
+  _tooltip->SetOpacity(0.0f);
   _tooltip->text = tooltip_text();
+  _tooltip->hidden.connect([this] { _tooltip.Release(); });
+  debug::Introspectable::AddChild(_tooltip.GetPointer());
 }
 
 void LauncherIcon::LoadQuicklist()
 {
-  _quicklist = new QuicklistView();
-  AddChild(_quicklist.GetPointer());
+  int monitor = _last_monitor;
+  if (monitor < 0)
+    monitor = 0;
 
-  _quicklist->mouse_down_outside_pointer_grab_area.connect([&] (int x, int y, unsigned long button_flags, unsigned long key_flags)
+  _quicklist = new QuicklistView(monitor);
+  _quicklist->hidden.connect([this] { _quicklist.Release(); });
+  debug::Introspectable::AddChild(_quicklist.GetPointer());
+
+  _quicklist->mouse_down_outside_pointer_grab_area.connect([this] (int x, int y, unsigned long button_flags, unsigned long key_flags)
   {
     _allow_quicklist_to_show = false;
   });
@@ -127,19 +143,14 @@ void LauncherIcon::LoadQuicklist()
   QuicklistManager::Default()->RegisterQuicklist(_quicklist);
 }
 
-const bool
-LauncherIcon::WindowVisibleOnMonitor(int monitor)
+const bool LauncherIcon::WindowVisibleOnMonitor(int monitor)
 {
   return _has_visible_window[monitor];
 }
 
 const bool LauncherIcon::WindowVisibleOnViewport()
 {
-  for (unsigned i = 0; i < monitors::MAX; ++i)
-    if (_has_visible_window[i])
-      return true;
-
-  return false;
+  return _has_visible_window.any();
 }
 
 std::string
@@ -148,25 +159,42 @@ LauncherIcon::GetName() const
   return "LauncherIcon";
 }
 
-void
-LauncherIcon::AddProperties(GVariantBuilder* builder)
+void LauncherIcon::AddProperties(debug::IntrospectionData& introspection)
 {
-  GVariantBuilder monitors_builder;
-  g_variant_builder_init(&monitors_builder, G_VARIANT_TYPE ("ab"));
+  std::vector<bool> monitors_active,
+                    monitors_visible,
+                    monitors_urgent,
+                    monitors_running,
+                    monitors_starting,
+                    monitors_desaturated,
+                    monitors_presented;
 
   for (unsigned i = 0; i < monitors::MAX; ++i)
-    g_variant_builder_add(&monitors_builder, "b", IsVisibleOnMonitor(i));
+  {
+    monitors_active.push_back(GetQuirk(Quirk::ACTIVE, i));
+    monitors_visible.push_back(IsVisibleOnMonitor(i));
+    monitors_urgent.push_back(GetQuirk(Quirk::URGENT, i));
+    monitors_running.push_back(GetQuirk(Quirk::RUNNING, i));
+    monitors_starting.push_back(GetQuirk(Quirk::STARTING, i));
+    monitors_desaturated.push_back(GetQuirk(Quirk::DESAT, i));
+    monitors_presented.push_back(GetQuirk(Quirk::PRESENTED, i));
+  }
 
-  unity::variant::BuilderWrapper(builder)
-  .add("center_x", _center[0].x)
-  .add("center_y", _center[0].y)
-  .add("center_z", _center[0].z)
+  introspection
+  .add("center", _center[unity::UScreen::GetDefault()->GetMonitorWithMouse()])
   .add("related_windows", Windows().size())
   .add("icon_type", unsigned(_icon_type))
   .add("tooltip_text", tooltip_text())
   .add("sort_priority", _sort_priority)
   .add("shortcut", _shortcut)
-  .add("monitors_visibility", g_variant_builder_end(&monitors_builder))
+  .add("order", _order)
+  .add("monitors_active", glib::Variant::FromVector(monitors_active))
+  .add("monitors_visibility", glib::Variant::FromVector(monitors_visible))
+  .add("monitors_urgent", glib::Variant::FromVector(monitors_urgent))
+  .add("monitors_running", glib::Variant::FromVector(monitors_running))
+  .add("monitors_starting", glib::Variant::FromVector(monitors_starting))
+  .add("monitors_desaturated", glib::Variant::FromVector(monitors_desaturated))
+  .add("monitors_presented", glib::Variant::FromVector(monitors_presented))
   .add("active", GetQuirk(Quirk::ACTIVE))
   .add("visible", GetQuirk(Quirk::VISIBLE))
   .add("urgent", GetQuirk(Quirk::URGENT))
@@ -176,30 +204,45 @@ LauncherIcon::AddProperties(GVariantBuilder* builder)
   .add("presented", GetQuirk(Quirk::PRESENTED));
 }
 
-void
-LauncherIcon::Activate(ActionArg arg)
+bool LauncherIcon::IsActionArgValid(ActionArg const& arg)
 {
+  if (arg.source != ActionArg::Source::LAUNCHER_KEYBINDING)
+    return true;
+
+  time::Spec now;
+  now.SetToNow();
+
+  return (now.TimeDelta(_last_action) > IGNORE_REPEAT_SHORTCUT_DURATION);
+}
+
+void LauncherIcon::Activate(ActionArg arg)
+{
+  if (!IsActionArgValid(arg))
+    return;
+
   /* Launcher Icons that handle spread will adjust the spread state
    * accordingly, for all other icons we should terminate spread */
   WindowManager& wm = WindowManager::Default();
-  if (wm.IsScaleActive() && !HandlesSpread ())
+  if (wm.IsScaleActive() && !HandlesSpread())
     wm.TerminateScale();
 
   ActivateLauncherIcon(arg);
 
-  UpdateQuirkTime(Quirk::LAST_ACTION);
+  _last_action.SetToNow();
 }
 
-void
-LauncherIcon::OpenInstance(ActionArg arg)
+void LauncherIcon::OpenInstance(ActionArg arg)
 {
+  if (!IsActionArgValid(arg))
+    return;
+
   WindowManager& wm = WindowManager::Default();
   if (wm.IsScaleActive())
     wm.TerminateScale();
 
   OpenInstanceLauncherIcon(arg.timestamp);
 
-  UpdateQuirkTime(Quirk::LAST_ACTION);
+  _last_action.SetToNow();
 }
 
 nux::Color LauncherIcon::BackgroundColor() const
@@ -314,10 +357,9 @@ BaseTexturePtr LauncherIcon::TextureFromGtkTheme(std::string icon_name, int size
     icon_name = DEFAULT_ICON;
 
   default_theme = gtk_icon_theme_get_default();
+  result = TextureFromSpecificGtkTheme(default_theme, icon_name, size, update_glow_colors);
 
-  // FIXME: we need to create some kind of -unity postfix to see if we are looking to the unity-icon-theme
-  // for dedicated unity icons, then remove the postfix and degrade to other icon themes if not found
-  if (icon_name.find("workspace-switcher") == 0)
+  if (!result)
     result = TextureFromSpecificGtkTheme(GetUnityTheme(), icon_name, size, update_glow_colors);
 
   if (!result)
@@ -414,23 +456,6 @@ BaseTexturePtr LauncherIcon::TextureFromPath(std::string const& icon_name, int s
   return BaseTexturePtr();
 }
 
-bool LauncherIcon::SetTooltipText(std::string& target, std::string const& value)
-{
-  auto const& escaped = glib::String(g_markup_escape_text(value.c_str(), -1)).Str();
-
-  if (escaped != target)
-  {
-    target = escaped;
-
-    if (_tooltip)
-      _tooltip->text = target;
-
-    return true;
-  }
-
-  return false;
-}
-
 void LauncherIcon::OnTooltipEnabledChanged(bool value)
 {
   if (!value)
@@ -452,7 +477,8 @@ guint64 LauncherIcon::GetShortcut()
 
 nux::Point LauncherIcon::GetTipPosition(int monitor) const
 {
-  return nux::Point(_center[monitor].x + icon_size()/2 + 1, _center[monitor].y);
+  auto const& converter = Settings::Instance().em(monitor);
+  return nux::Point(_center[monitor].x + converter->CP(icon_size()) / 2 + 1, _center[monitor].y);
 }
 
 void LauncherIcon::ShowTooltip()
@@ -472,6 +498,10 @@ void LauncherIcon::ShowTooltip()
 void LauncherIcon::RecvMouseEnter(int monitor)
 {
   _last_monitor = monitor;
+
+  // FIXME We need to look at why we need to set the last_monitor to -1 when it leaves.
+  // As it would be nice to not have to re-create the tooltip everytime now :(
+  LoadTooltip();
 }
 
 void LauncherIcon::RecvMouseLeave(int monitor)
@@ -484,20 +514,17 @@ bool LauncherIcon::OpenQuicklist(bool select_first_item, int monitor)
 {
   MenuItemsVector const& menus = Menus();
 
-  if (!_quicklist)
-    LoadQuicklist();
-
   if (menus.empty())
     return false;
 
+  LoadQuicklist();
+
   if (_tooltip)
   {
     // Hide the tooltip without fade animation
     _tooltip->ShowWindow(false);
   }
 
-  _quicklist->RemoveAllMenuItem();
-
   for (auto const& menu_item : menus)
   {
     QuicklistMenuItem* ql_item = nullptr;
@@ -652,17 +679,28 @@ void LauncherIcon::SetCenter(nux::Point3 const& new_center, int monitor)
     }
 
     return false;
-  }, CENTER_STABILIZE_TIMEOUT);
+  }, CENTER_STABILIZE_TIMEOUT + std::to_string(monitor));
 }
 
-nux::Point3
-LauncherIcon::GetCenter(int monitor)
+void LauncherIcon::ResetCenters(int monitor)
+{
+  if (monitor < 0)
+  {
+    for (unsigned i = 0; i < monitors::MAX; ++i)
+      _center[i].Set(0, 0, 0);
+  }
+  else
+  {
+    _center[monitor].Set(0, 0, 0);
+  }
+}
+
+nux::Point3 LauncherIcon::GetCenter(int monitor)
 {
   return _center[monitor];
 }
 
-nux::Point3
-LauncherIcon::GetSavedCenter(int monitor)
+nux::Point3 LauncherIcon::GetSavedCenter(int monitor)
 {
   return _saved_center[monitor];
 }
@@ -672,37 +710,45 @@ std::vector<nux::Point3> LauncherIcon::GetCenters()
   return _center;
 }
 
-void
-LauncherIcon::SaveCenter()
+void LauncherIcon::SaveCenter()
 {
   _saved_center = _center;
-  UpdateQuirkTime(Quirk::CENTER_SAVED);
+  FullyAnimateQuirk(Quirk::CENTER_SAVED, 0);
 }
 
-void
-LauncherIcon::SetWindowVisibleOnMonitor(bool val, int monitor)
+std::pair<int, nux::Point3> LauncherIcon::GetCenterForMonitor(int monitor) const
+{
+  monitor = CLAMP(monitor, 0, static_cast<int>(_center.size() - 1));
+
+  if (_center[monitor].x && _center[monitor].y)
+    return {monitor, _center[monitor]};
+
+  for (unsigned i = 0; i < _center.size(); ++i)
+  {
+    if (_center[i].x && _center[i].y)
+      return {i, _center[i]};
+  }
+
+  return {-1, nux::Point3()};
+}
+
+void LauncherIcon::SetWindowVisibleOnMonitor(bool val, int monitor)
 {
   if (_has_visible_window[monitor] == val)
     return;
 
   _has_visible_window[monitor] = val;
-  EmitNeedsRedraw();
+  EmitNeedsRedraw(monitor);
 }
 
-void
-LauncherIcon::SetVisibleOnMonitor(int monitor, bool visible)
+void LauncherIcon::SetVisibleOnMonitor(int monitor, bool visible)
 {
-  if (_is_visible_on_monitor[monitor] == visible)
-    return;
-
-  _is_visible_on_monitor[monitor] = visible;
-  EmitNeedsRedraw();
+  SetQuirk(Quirk::VISIBLE, visible, monitor);
 }
 
-bool
-LauncherIcon::IsVisibleOnMonitor(int monitor) const
+bool LauncherIcon::IsVisibleOnMonitor(int monitor) const
 {
-  return _is_visible_on_monitor[monitor];
+  return GetQuirk(Quirk::VISIBLE, monitor);
 }
 
 float LauncherIcon::PresentUrgency()
@@ -710,41 +756,38 @@ float LauncherIcon::PresentUrgency()
   return _present_urgency;
 }
 
-void
-LauncherIcon::Present(float present_urgency, int length)
+void LauncherIcon::Present(float present_urgency, int length, int monitor)
 {
-  if (GetQuirk(Quirk::PRESENTED))
+  if (GetQuirk(Quirk::PRESENTED, monitor))
     return;
 
   if (length >= 0)
   {
-    _source_manager.AddTimeout(length, [this] {
-      if (!GetQuirk(Quirk::PRESENTED))
+    _source_manager.AddTimeout(length, [this, monitor] {
+      if (!GetQuirk(Quirk::PRESENTED, monitor))
         return false;
 
-      Unpresent();
+      Unpresent(monitor);
       return false;
-    }, PRESENT_TIMEOUT);
+    }, PRESENT_TIMEOUT + std::to_string(monitor));
   }
 
   _present_urgency = CLAMP(present_urgency, 0.0f, 1.0f);
-  SetQuirk(Quirk::PRESENTED, true);
-  SetQuirk(Quirk::UNFOLDED, true);
+  SetQuirk(Quirk::PRESENTED, true, monitor);
+  SetQuirk(Quirk::UNFOLDED, true, monitor);
 }
 
-void
-LauncherIcon::Unpresent()
+void LauncherIcon::Unpresent(int monitor)
 {
-  if (!GetQuirk(Quirk::PRESENTED))
+  if (!GetQuirk(Quirk::PRESENTED, monitor))
     return;
 
-  _source_manager.Remove(PRESENT_TIMEOUT);
-  SetQuirk(Quirk::PRESENTED, false);
-  SetQuirk(Quirk::UNFOLDED, false);
+  _source_manager.Remove(PRESENT_TIMEOUT + std::to_string(monitor));
+  SetQuirk(Quirk::PRESENTED, false, monitor);
+  SetQuirk(Quirk::UNFOLDED, false, monitor);
 }
 
-void
-LauncherIcon::Remove()
+void LauncherIcon::Remove()
 {
   if (_quicklist && _quicklist->IsVisible())
       _quicklist->Hide();
@@ -762,91 +805,139 @@ LauncherIcon::Remove()
   removed = true;
 }
 
-void
-LauncherIcon::SetSortPriority(int priority)
+void LauncherIcon::SetSortPriority(int priority)
 {
   _sort_priority = priority;
 }
 
-int
-LauncherIcon::SortPriority()
+int LauncherIcon::SortPriority()
 {
   return _sort_priority;
 }
 
+void LauncherIcon::SetOrder(int order)
+{
+  _order = order;
+}
+
 LauncherIcon::IconType
 LauncherIcon::GetIconType() const
 {
   return _icon_type;
 }
 
-bool
-LauncherIcon::GetQuirk(LauncherIcon::Quirk quirk) const
+bool LauncherIcon::GetQuirk(LauncherIcon::Quirk quirk, int monitor) const
 {
-  return _quirks[unsigned(quirk)];
+  if (monitor < 0)
+  {
+    for (unsigned i = 0; i < monitors::MAX; ++i)
+    {
+      if (!_quirks[i][unsigned(quirk)])
+        return false;
+    }
+
+    return true;
+  }
+
+  return _quirks[monitor][unsigned(quirk)];
 }
 
-void
-LauncherIcon::SetQuirk(LauncherIcon::Quirk quirk, bool value)
+void LauncherIcon::SetQuirk(LauncherIcon::Quirk quirk, bool value, int monitor)
 {
-  if (_quirks[unsigned(quirk)] == value)
-    return;
+  bool changed = false;
 
-  _quirks[unsigned(quirk)] = value;
-  if (quirk == Quirk::VISIBLE)
-    TimeUtil::SetTimeStruct(&(_quirk_times[unsigned(quirk)]), &(_quirk_times[unsigned(quirk)]), Launcher::ANIM_DURATION_SHORT);
-  else
-    clock_gettime(CLOCK_MONOTONIC, &(_quirk_times[unsigned(quirk)]));
-  EmitNeedsRedraw();
-
-  // Present on urgent as a general policy
-  if (quirk == Quirk::VISIBLE && value)
-    Present(0.5f, 1500);
-  if (quirk == Quirk::URGENT)
+  if (monitor < 0)
   {
-    if (value)
+    for (unsigned i = 0; i < monitors::MAX; ++i)
     {
-      Present(0.5f, 1500);
+      if (_quirks[i][unsigned(quirk)] != value)
+      {
+        _quirks[i][unsigned(quirk)] = value;
+        animation::StartOrReverseIf(GetQuirkAnimation(quirk, i), value);
+        changed = true;
+      }
+    }
+  }
+  else
+  {
+    if (_quirks[monitor][unsigned(quirk)] != value)
+    {
+      _quirks[monitor][unsigned(quirk)] = value;
+      animation::StartOrReverseIf(GetQuirkAnimation(quirk, monitor), value);
+      changed = true;
     }
   }
 
-  if (quirk == Quirk::VISIBLE)
+  if (!changed)
+    return;
+
+  // Present on urgent and visible as a general policy
+  if (value && (quirk == Quirk::URGENT || quirk == Quirk::VISIBLE))
   {
-     visibility_changed.emit();
+    Present(0.5f, 1500, monitor);
+  }
+
+  if (quirk == Quirk::VISIBLE)
+    visibility_changed.emit(monitor);
+
+  QuirksChanged.emit();
+}
+
+void LauncherIcon::FullyAnimateQuirkDelayed(guint ms, LauncherIcon::Quirk quirk, int monitor)
+{
+  _source_manager.AddTimeout(ms, [this, quirk, monitor] {
+    FullyAnimateQuirk(quirk, monitor);
+    return false;
+  }, QUIRK_DELAY_TIMEOUT + std::to_string(unsigned(quirk)) + std::to_string(monitor));
+}
+
+void LauncherIcon::FullyAnimateQuirk(LauncherIcon::Quirk quirk, int monitor)
+{
+  if (monitor < 0)
+  {
+    for (unsigned i = 0; i < monitors::MAX; ++i)
+      animation::Start(GetQuirkAnimation(quirk, i), animation::Direction::FORWARD);
+  }
+  else
+  {
+    animation::Start(GetQuirkAnimation(quirk, monitor), animation::Direction::FORWARD);
   }
 }
 
-void
-LauncherIcon::UpdateQuirkTimeDelayed(guint ms, LauncherIcon::Quirk quirk)
+void LauncherIcon::SkipQuirkAnimation(LauncherIcon::Quirk quirk, int monitor)
 {
-  _source_manager.AddTimeout(ms, [&, quirk] {
-    UpdateQuirkTime(quirk);
-    return false;
-  }, QUIRK_DELAY_TIMEOUT);
+  if (monitor < 0)
+  {
+    for (unsigned i = 0; i < monitors::MAX; ++i)
+    {
+      animation::Skip(GetQuirkAnimation(quirk, i));
+    }
+  }
+  else
+  {
+    animation::Skip(GetQuirkAnimation(quirk, monitor));
+  }
 }
 
-void
-LauncherIcon::UpdateQuirkTime(LauncherIcon::Quirk quirk)
+float LauncherIcon::GetQuirkProgress(Quirk quirk, int monitor) const
 {
-  clock_gettime(CLOCK_MONOTONIC, &(_quirk_times[unsigned(quirk)]));
-  EmitNeedsRedraw();
+  return GetQuirkAnimation(quirk, monitor).GetCurrentValue();
 }
 
-void
-LauncherIcon::ResetQuirkTime(LauncherIcon::Quirk quirk)
+void LauncherIcon::SetQuirkDuration(Quirk quirk, unsigned duration, int monitor)
 {
-  _quirk_times[unsigned(quirk)].tv_sec = 0;
-  _quirk_times[unsigned(quirk)].tv_nsec = 0;
+  if (monitor < 0)
+  {
+    for (unsigned i = 0; i < monitors::MAX; ++i)
+      GetQuirkAnimation(quirk, i).SetDuration(duration);
+  }
+  else
+  {
+    GetQuirkAnimation(quirk, monitor).SetDuration(duration);
+  }
 }
 
-struct timespec
-LauncherIcon::GetQuirkTime(LauncherIcon::Quirk quirk)
-{
-  return _quirk_times[unsigned(quirk)];
-}
-
-void
-LauncherIcon::SetProgress(float progress)
+void LauncherIcon::SetProgress(float progress)
 {
   if (progress == _progress)
     return;
@@ -855,8 +946,7 @@ LauncherIcon::SetProgress(float progress)
   EmitNeedsRedraw();
 }
 
-float
-LauncherIcon::GetProgress()
+float LauncherIcon::GetProgress()
 {
   return _progress;
 }
@@ -1131,10 +1221,22 @@ glib::Object<DbusmenuMenuitem> LauncherIcon::GetRemoteMenus() const
   return root;
 }
 
-void LauncherIcon::EmitNeedsRedraw()
+void LauncherIcon::EmitNeedsRedraw(int monitor)
 {
   if (OwnsTheReference() && GetReferenceCount() > 0)
-    needs_redraw.emit(AbstractLauncherIcon::Ptr(this));
+  {
+    if (monitor < 0)
+    {
+      needs_redraw.emit(AbstractLauncherIcon::Ptr(this), monitor);
+    }
+    else
+    {
+      auto const& visibilty = GetQuirkAnimation(Quirk::VISIBLE, monitor);
+
+      if (visibilty.GetCurrentValue() > 0.0f || visibilty.CurrentState() == na::Animation::State::Running)
+        needs_redraw.emit(AbstractLauncherIcon::Ptr(this), monitor);
+    }
+  }
 }
 
 void LauncherIcon::EmitRemove()
