@@ -32,33 +32,22 @@ namespace launcher
 // Start private implementation
 //
 
-class VolumeImp::Impl
+class VolumeImp::Impl : public sigc::trackable
 {
 public:
-  Impl(glib::Object<GVolume> const& volume,
-       FileManagerOpener::Ptr const& file_manager_opener,
-       DeviceNotificationDisplay::Ptr const& device_notification_display,
-       VolumeImp* parent)
+  Impl(glib::Object<GVolume> const& volume, VolumeImp* parent)
     : parent_(parent)
-    , cancellable_(g_cancellable_new())
     , volume_(volume)
-    , file_manager_opener_(file_manager_opener)
-    , device_notification_display_(device_notification_display)
   {
     signal_volume_changed_.Connect(volume_, "changed", [this] (GVolume*) {
       parent_->changed.emit();
     });
 
     signal_volume_removed_.Connect(volume_, "removed", [this] (GVolume*) {
-          parent_->removed.emit();
+      parent_->removed.emit();
     });
   }
 
-  ~Impl()
-  {
-    g_cancellable_cancel(cancellable_);
-  }
-
   bool CanBeEjected() const
   {
     return g_volume_can_eject(volume_) != FALSE;
@@ -117,71 +106,44 @@ public:
     return static_cast<bool>(mount);
   }
 
-  void EjectAndShowNotification()
+  void Eject()
   {
     if (!CanBeEjected())
       return;
 
     glib::Object<GMountOperation> mount_op(gtk_mount_operation_new(nullptr));
 
-    g_volume_eject_with_operation(volume_,
-                                  (GMountUnmountFlags)0,
-                                  mount_op,
-                                  cancellable_,
-                                  (GAsyncReadyCallback)OnEjectReady,
-                                  this);
+    g_volume_eject_with_operation(volume_, G_MOUNT_UNMOUNT_NONE, mount_op, cancellable_,
+    [] (GObject* object, GAsyncResult* res, gpointer data) {
+      if (g_volume_eject_with_operation_finish(G_VOLUME(object), res, nullptr))
+        static_cast<Impl*>(data)->parent_->ejected.emit();
+    }, this);
   }
 
-  static void OnEjectReady(GObject* object, GAsyncResult* result, Impl* self)
-  {
-    if (g_volume_eject_with_operation_finish(self->volume_, result, nullptr))
-    {
-      self->device_notification_display_->Display(self->GetIconName(), self->GetName());
-    }
-  }
-
-  void MountAndOpenInFileManager()
-  {
-    if (!IsMounted())
-      MountAndOnFinishOpenInFileManager();
-    else
-      OpenInFileManager();
-  }
-
-  void MountAndOnFinishOpenInFileManager()
+  void Mount()
   {
     glib::Object<GMountOperation> mount_op(gtk_mount_operation_new(nullptr));
 
-    g_volume_mount(volume_,
-                   (GMountMountFlags) 0,
-                   mount_op,
-                   cancellable_,
-                   (GAsyncReadyCallback) &Impl::OnMountFinish,
-                   this);
+    g_volume_mount(volume_, G_MOUNT_MOUNT_NONE, mount_op, cancellable_,
+    [] (GObject* object, GAsyncResult* res, gpointer data) {
+      if (g_volume_mount_finish(G_VOLUME(object), res, nullptr))
+        static_cast<Impl*>(data)->parent_->mounted.emit();
+    }, this);
   }
 
-  static void OnMountFinish(GObject* object,
-                            GAsyncResult* result,
-                            Impl* self)
-  {
-    if (g_volume_mount_finish(self->volume_, result, nullptr))
-      self->OpenInFileManager();
-  }
-
-  void OpenInFileManager()
-  {
-    file_manager_opener_->Open(GetUri());
-  }
-
-  std::string GetUri()
+  std::string GetUri() const
   {
     glib::Object<GMount> mount(g_volume_get_mount(volume_));
+
+    if (!mount.IsType(G_TYPE_MOUNT))
+      return std::string();
+
     glib::Object<GFile> root(g_mount_get_root(mount));
 
-    if (root.IsType(G_TYPE_FILE))
-      return glib::String(g_file_get_uri(root)).Str();
-    else
-     return std::string();
+    if (!root.IsType(G_TYPE_FILE))
+      return std::string();
+
+    return glib::String(g_file_get_uri(root)).Str();
   }
 
   void StopDrive()
@@ -192,10 +154,11 @@ public:
     glib::Object<GDrive> drive(g_volume_get_drive(volume_));
     glib::Object<GMountOperation> mount_op(gtk_mount_operation_new(NULL));
 
-    g_drive_stop(drive,
-                 (GMountUnmountFlags)0,
-                 mount_op,
-                 cancellable_, nullptr, nullptr);
+    g_drive_stop(drive, G_MOUNT_UNMOUNT_NONE, mount_op, cancellable_,
+    [] (GObject* object, GAsyncResult* res, gpointer data) {
+      if (g_drive_stop_finish(G_DRIVE(object), res, nullptr))
+        static_cast<Impl*>(data)->parent_->stopped.emit();
+    }, this);
   }
 
   void Unmount()
@@ -206,17 +169,16 @@ public:
     glib::Object<GMount> mount(g_volume_get_mount(volume_));
     glib::Object<GMountOperation> op(gtk_mount_operation_new(nullptr));
 
-    g_mount_unmount_with_operation(mount,
-                                   (GMountUnmountFlags)0,
-                                   op,
-                                   cancellable_, nullptr, nullptr);
+    g_mount_unmount_with_operation(mount, G_MOUNT_UNMOUNT_NONE, op, cancellable_,
+    [] (GObject* object, GAsyncResult* res, gpointer data) {
+      if (g_mount_unmount_with_operation_finish(G_MOUNT(object), res, nullptr))
+        static_cast<Impl*>(data)->parent_->unmounted.emit();
+    }, this);
   }
 
   VolumeImp* parent_;
-  glib::Object<GCancellable> cancellable_;
+  glib::Cancellable cancellable_;
   glib::Object<GVolume> volume_;
-  FileManagerOpener::Ptr file_manager_opener_;
-  DeviceNotificationDisplay::Ptr device_notification_display_;
 
   glib::Signal<void, GVolume*> signal_volume_changed_;
   glib::Signal<void, GVolume*> signal_volume_removed_;
@@ -226,10 +188,8 @@ public:
 // End private implementation
 //
 
-VolumeImp::VolumeImp(glib::Object<GVolume> const& volume,
-                     FileManagerOpener::Ptr const& file_manager_opener,
-                     DeviceNotificationDisplay::Ptr const& device_notification_display)
-  : pimpl(new Impl(volume, file_manager_opener, device_notification_display, this))
+VolumeImp::VolumeImp(glib::Object<GVolume> const& volume)
+  : pimpl(new Impl(volume, this))
 {}
 
 VolumeImp::~VolumeImp()
@@ -265,6 +225,11 @@ std::string VolumeImp::GetIdentifier() const
   return pimpl->GetIdentifier();
 }
 
+std::string VolumeImp::GetUri() const
+{
+  return pimpl->GetUri();
+}
+
 bool VolumeImp::HasSiblings() const
 {
   return pimpl->HasSiblings();
@@ -275,14 +240,14 @@ bool VolumeImp::IsMounted() const
   return pimpl->IsMounted();
 }
 
-void VolumeImp::MountAndOpenInFileManager()
+void VolumeImp::Mount()
 {
-  pimpl->MountAndOpenInFileManager();
+  pimpl->Mount();
 }
 
-void VolumeImp::EjectAndShowNotification()
+void VolumeImp::Eject()
 {
-  pimpl->EjectAndShowNotification();
+  pimpl->Eject();
 }
 
 void VolumeImp::StopDrive()
