@@ -19,31 +19,26 @@
  */
 
 #include <Nux/Nux.h>
-#include <Nux/HLayout.h>
-#include <Nux/VLayout.h>
-
-#include <NuxGraphics/GLThread.h>
-#include <Nux/BaseWindow.h>
-#include <Nux/WindowCompositor.h>
-#include <UnityCore/Variant.h>
+#include <NuxCore/Logger.h>
+#include <UnityCore/ConnectionManager.h>
 #include <UnityCore/GTKWrapper.h>
 
-#include <glib.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
-#include <boost/algorithm/string.hpp>
+
+#include "PanelIndicatorEntryView.h"
 
 #include "unity-shared/CairoTexture.h"
-#include "PanelIndicatorEntryView.h"
 #include "unity-shared/PanelStyle.h"
+#include "unity-shared/RawPixel.h"
 #include "unity-shared/WindowManager.h"
-
+#include "unity-shared/UnitySettings.h"
 
 namespace unity
 {
-
 namespace
 {
+DECLARE_LOGGER(logger, "unity.panel.indicator.entry");
 const int DEFAULT_SPACING = 3;
 }
 
@@ -53,16 +48,15 @@ PanelIndicatorEntryView::PanelIndicatorEntryView(Entry::Ptr const& proxy, int pa
                                                  IndicatorEntryType type)
   : TextureArea(NUX_TRACKER_LOCATION)
   , proxy_(proxy)
-  , spacing_(DEFAULT_SPACING)
-  , left_padding_(padding < 0 ? 0 : padding)
-  , right_padding_(left_padding_)
   , type_(type)
-  , entry_texture_(nullptr)
+  , monitor_(0)
   , opacity_(1.0f)
   , draw_active_(false)
   , overlay_showing_(false)
   , disabled_(false)
   , focused_(true)
+  , padding_(padding < 0 ? 0 : padding)
+  , cv_(unity::Settings::Instance().em(monitor_))
 {
   proxy_->active_changed.connect(sigc::mem_fun(this, &PanelIndicatorEntryView::OnActiveChanged));
   proxy_->updated.connect(sigc::mem_fun(this, &PanelIndicatorEntryView::Refresh));
@@ -70,12 +64,22 @@ PanelIndicatorEntryView::PanelIndicatorEntryView(Entry::Ptr const& proxy, int pa
   InputArea::mouse_down.connect(sigc::mem_fun(this, &PanelIndicatorEntryView::OnMouseDown));
   InputArea::mouse_up.connect(sigc::mem_fun(this, &PanelIndicatorEntryView::OnMouseUp));
 
-  InputArea::SetAcceptMouseWheelEvent(true);
+  if (type_ == INDICATOR)
+  {
+    InputArea::SetAcceptMouseWheelEvent(true);
+    InputArea::mouse_wheel.connect(sigc::mem_fun(this, &PanelIndicatorEntryView::OnMouseWheel));
+  }
 
   if (type_ != MENU)
-    InputArea::mouse_wheel.connect(sigc::mem_fun(this, &PanelIndicatorEntryView::OnMouseWheel));
+  {
+    icon_theme_changed_.Connect(gtk_icon_theme_get_default(), "changed", [this] (GtkIconTheme*) {
+      if (proxy_->image_type() && proxy_->image_visible())
+        Refresh();
+    });
+  }
 
   panel::Style::Instance().changed.connect(sigc::mem_fun(this, &PanelIndicatorEntryView::Refresh));
+  unity::Settings::Instance().dpi_changed.connect(sigc::mem_fun(this, &PanelIndicatorEntryView::Refresh));
 
   Refresh();
 }
@@ -100,11 +104,23 @@ void PanelIndicatorEntryView::ShowMenu(int button)
 {
   WindowManager& wm = WindowManager::Default();
 
-  if (!wm.IsExpoActive() && !wm.IsScaleActive())
+  if (wm.IsExpoActive())
   {
-    auto const& abs_geo = GetAbsoluteGeometry();
-    proxy_->ShowMenu(abs_geo.x, abs_geo.y + panel::Style::Instance().panel_height, button);
+    // Delay the activation until expo is closed
+    auto conn = std::make_shared<connection::Wrapper>();
+    *conn = wm.terminate_expo.connect([this, conn, button] {
+      ShowMenu(button);
+      (*conn)->disconnect();
+    });
+
+    wm.TerminateExpo();
   }
+
+  if (wm.IsScaleActive())
+    wm.TerminateScale();
+
+  auto const& abs_geo = GetAbsoluteGeometry();
+  proxy_->ShowMenu(abs_geo.x, abs_geo.y + abs_geo.height, button);
 }
 
 void PanelIndicatorEntryView::OnMouseDown(int x, int y, long button_flags, long key_flags)
@@ -118,12 +134,16 @@ void PanelIndicatorEntryView::OnMouseDown(int x, int y, long button_flags, long
     int button = nux::GetEventButton(button_flags);
 
     if (button == 2 && type_ == INDICATOR)
+    {
       SetOpacity(0.75f);
+      QueueDraw();
+    }
     else
+    {
       ShowMenu(button);
+      Refresh();
+    }
   }
-
-  Refresh();
 }
 
 void PanelIndicatorEntryView::OnMouseUp(int x, int y, long button_flags, long key_flags)
@@ -145,9 +165,8 @@ void PanelIndicatorEntryView::OnMouseUp(int x, int y, long button_flags, long ke
       proxy_->SecondaryActivate();
 
     SetOpacity(1.0f);
+    QueueDraw();
   }
-
-  Refresh();
 }
 
 void PanelIndicatorEntryView::OnMouseWheel(int x, int y, int delta,
@@ -180,102 +199,96 @@ void PanelIndicatorEntryView::SetActiveState(bool active, int button)
   }
 }
 
-glib::Object<GdkPixbuf> PanelIndicatorEntryView::MakePixbuf()
+glib::Object<GdkPixbuf> PanelIndicatorEntryView::MakePixbuf(int size)
 {
   glib::Object<GdkPixbuf> pixbuf;
-  GtkIconTheme* theme = gtk_icon_theme_get_default();
-  int image_type = proxy_->image_type();
+  auto image_type = proxy_->image_type();
 
-  if (image_type == GTK_IMAGE_PIXBUF)
+  switch (image_type)
   {
-    gsize len = 0;
-    guchar* decoded = g_base64_decode(proxy_->image_data().c_str(), &len);
+    case GTK_IMAGE_PIXBUF:
+    {
+      gsize len = 0;
+      auto* decoded = g_base64_decode(proxy_->image_data().c_str(), &len);
+      glib::Object<GInputStream> stream(g_memory_input_stream_new_from_data(decoded, len, nullptr));
+      pixbuf = gdk_pixbuf_new_from_stream(stream, nullptr, nullptr);
+      g_input_stream_close(stream, nullptr, nullptr);
+      g_free(decoded);
+      break;
+    }
 
-    glib::Object<GInputStream> stream(g_memory_input_stream_new_from_data(decoded,
-                                                                          len,
-                                                                          nullptr));
+    case GTK_IMAGE_ICON_NAME:
+    case GTK_IMAGE_STOCK:
+    case GTK_IMAGE_GICON:
+    {
+      GtkIconTheme* theme = gtk_icon_theme_get_default();
+      auto flags = static_cast<GtkIconLookupFlags>(0);
+      gtk::IconInfo info;
 
-    pixbuf = gdk_pixbuf_new_from_stream(stream, nullptr, nullptr);
+      if (image_type == GTK_IMAGE_GICON)
+      {
+        glib::Object<GIcon> icon(g_icon_new_for_string(proxy_->image_data().c_str(), nullptr));
+        info = gtk_icon_theme_lookup_by_gicon(theme, icon, size, flags);
+      }
+      else
+      {
+        info = gtk_icon_theme_lookup_icon(theme, proxy_->image_data().c_str(), size, flags);
+      }
 
-    g_free(decoded);
-    g_input_stream_close(stream, nullptr, nullptr);
-  }
-  else if (image_type == GTK_IMAGE_STOCK ||
-           image_type == GTK_IMAGE_ICON_NAME)
-  {
-    pixbuf = gtk_icon_theme_load_icon(theme, proxy_->image_data().c_str(), 22,
-                                      (GtkIconLookupFlags)0, nullptr);
-  }
-  else if (image_type == GTK_IMAGE_GICON)
-  {
-    glib::Object<GIcon> icon(g_icon_new_for_string(proxy_->image_data().c_str(), nullptr));
+      if (info)
+      {
+        auto* filename = gtk_icon_info_get_filename(info);
+        pixbuf = gdk_pixbuf_new_from_file_at_size(filename, -1, size, nullptr);
+      }
+      else if (image_type == GTK_IMAGE_ICON_NAME)
+      {
+        pixbuf = gdk_pixbuf_new_from_file_at_size(proxy_->image_data().c_str(), -1, size, nullptr);
+      }
 
-    gtk::IconInfo info(gtk_icon_theme_lookup_by_gicon(theme, icon, 22,
-                                                      (GtkIconLookupFlags)0));
-    if (info)
-      pixbuf = gtk_icon_info_load_icon(info, nullptr);
+      break;
+    }
   }
 
   return pixbuf;
 }
 
-void PanelIndicatorEntryView::DrawEntryPrelight(cairo_t* cr, unsigned int width, unsigned int height)
+void PanelIndicatorEntryView::DrawEntryContent(cairo_t *cr, unsigned int width, unsigned int height, glib::Object<GdkPixbuf> const& pixbuf, bool icon_scalable, glib::Object<PangoLayout> const& layout)
 {
-  GtkStyleContext* style_context = panel::Style::Instance().GetStyleContext();
+  int x = padding_;
 
+  auto panel_item = (type_ == MENU) ? panel::PanelItem::MENU : panel::PanelItem::INDICATOR;
+  GtkStyleContext* style_context = panel::Style::Instance().GetStyleContext(panel_item);
   gtk_style_context_save(style_context);
 
-  GtkWidgetPath* widget_path = gtk_widget_path_new();
-  gtk_widget_path_append_type(widget_path, GTK_TYPE_MENU_BAR);
-  gtk_widget_path_append_type(widget_path, GTK_TYPE_MENU_ITEM);
-  gtk_widget_path_iter_set_name(widget_path, -1 , "UnityPanelWidget");
-
-  gtk_style_context_set_path(style_context, widget_path);
   gtk_style_context_add_class(style_context, GTK_STYLE_CLASS_MENUBAR);
   gtk_style_context_add_class(style_context, GTK_STYLE_CLASS_MENUITEM);
-  gtk_style_context_set_state(style_context, GTK_STATE_FLAG_PRELIGHT);
-
-  gtk_render_background(style_context, cr, 0, 0, width, height);
-  gtk_render_frame(style_context, cr, 0, 0, width, height);
-
-  gtk_widget_path_free(widget_path);
-
-  gtk_style_context_restore(style_context);
-}
-
-void PanelIndicatorEntryView::DrawEntryContent(cairo_t *cr, unsigned int width, unsigned int height, glib::Object<GdkPixbuf> const& pixbuf, glib::Object<PangoLayout> const& layout)
-{
-  int x = left_padding_;
 
   if (IsActive())
-    DrawEntryPrelight(cr, width, height);
+  {
+    gtk_style_context_set_state(style_context, GTK_STATE_FLAG_PRELIGHT);
+    gtk_render_background(style_context, cr, 0, 0, width, height);
+    gtk_render_frame(style_context, cr, 0, 0, width, height);
+  }
+
+  if (!IsFocused())
+    gtk_style_context_set_state(style_context, GTK_STATE_FLAG_BACKDROP);
 
   if (pixbuf && IsIconVisible())
   {
-    GtkStyleContext* style_context = panel::Style::Instance().GetStyleContext();
     unsigned int icon_width = gdk_pixbuf_get_width(pixbuf);
 
-    gtk_style_context_save(style_context);
+    int y = (height - gdk_pixbuf_get_height(pixbuf)) / 2;
 
-    GtkWidgetPath* widget_path = gtk_widget_path_new();
-    gint pos = gtk_widget_path_append_type(widget_path, GTK_TYPE_MENU_BAR);
-    pos = gtk_widget_path_append_type(widget_path, GTK_TYPE_MENU_ITEM);
-    gtk_widget_path_iter_set_name(widget_path, pos, "UnityPanelWidget");
-
-    gtk_style_context_set_path(style_context, widget_path);
-    gtk_style_context_add_class(style_context, GTK_STYLE_CLASS_MENUBAR);
-    gtk_style_context_add_class(style_context, GTK_STYLE_CLASS_MENUITEM);
-
-    if (!IsFocused())
+    if (icon_scalable)
     {
-      gtk_style_context_set_state(style_context, GTK_STATE_FLAG_BACKDROP);
-    }
-    else if (IsActive())
-    {
-      gtk_style_context_set_state(style_context, GTK_STATE_FLAG_PRELIGHT);
+      double dpi_scale = cv_->DPIScale();
+      cairo_save(cr);
+      cairo_scale(cr, 1.0f/dpi_scale, 1.0f/dpi_scale);
+      x = padding_ * dpi_scale;
+      y = (std::ceil(height * dpi_scale) - gdk_pixbuf_get_height(pixbuf)) / 2;
+      icon_width /= dpi_scale;
     }
 
-    int y = (int)((height - gdk_pixbuf_get_height(pixbuf)) / 2);
     if (overlay_showing_ && !IsActive())
     {
       /* Most of the images we get are straight pixbufs (annoyingly), so when
@@ -283,20 +296,14 @@ void PanelIndicatorEntryView::DrawEntryContent(cairo_t *cr, unsigned int width,
        * a white square. It works surprisingly well for most symbolic-type
        * icon themes/icons.
        */
-      cairo_save(cr);
-
       cairo_push_group(cr);
       gdk_cairo_set_source_pixbuf(cr, pixbuf, x, y);
       cairo_paint_with_alpha(cr, (IsIconSensitive() && IsFocused()) ? 1.0 : 0.5);
-
-      cairo_pattern_t* pat = cairo_pop_group(cr);
+      std::shared_ptr<cairo_pattern_t> pat(cairo_pop_group(cr), cairo_pattern_destroy);
 
       cairo_set_source_rgba(cr, 1.0f, 1.0f, 1.0f, 1.0f);
       cairo_rectangle(cr, x, y, width, height);
-      cairo_mask(cr, pat);
-
-      cairo_pattern_destroy(pat);
-      cairo_restore(cr);
+      cairo_mask(cr, pat.get());
     }
     else
     {
@@ -306,94 +313,48 @@ void PanelIndicatorEntryView::DrawEntryContent(cairo_t *cr, unsigned int width,
       cairo_paint_with_alpha(cr, (IsIconSensitive() && IsFocused()) ? 1.0 : 0.5);
     }
 
-    gtk_widget_path_free(widget_path);
+    if (icon_scalable)
+    {
+      cairo_restore(cr);
+      x = padding_;
+    }
 
-    gtk_style_context_restore(style_context);
-
-    x += icon_width + spacing_;
+    x += icon_width + DEFAULT_SPACING;
   }
 
   if (layout)
   {
-    PangoRectangle log_rect;
-    pango_layout_get_extents(layout, nullptr, &log_rect);
-    unsigned int text_height = log_rect.height / PANGO_SCALE;
-    unsigned int text_width = log_rect.width / PANGO_SCALE;
+    nux::Size extents;
+    pango_layout_get_pixel_size(layout, &extents.width, &extents.height);
+    int y = (height - extents.height) / 2;
 
-    pango_cairo_update_layout(cr, layout);
-
-    GtkStyleContext* style_context = panel::Style::Instance().GetStyleContext();
-
-    gtk_style_context_save(style_context);
-
-    GtkWidgetPath* widget_path = gtk_widget_path_new();
-    gint pos = gtk_widget_path_append_type(widget_path, GTK_TYPE_MENU_BAR);
-    pos = gtk_widget_path_append_type(widget_path, GTK_TYPE_MENU_ITEM);
-    gtk_widget_path_iter_set_name(widget_path, pos, "UnityPanelWidget");
-
-    gtk_style_context_set_path(style_context, widget_path);
-    gtk_style_context_add_class(style_context, GTK_STYLE_CLASS_MENUBAR);
-    gtk_style_context_add_class(style_context, GTK_STYLE_CLASS_MENUITEM);
-
-    if (!IsFocused())
+    if (overlay_showing_ && !IsActive())
     {
-      gtk_style_context_set_state(style_context, GTK_STATE_FLAG_BACKDROP);
-    }
-    else if (IsActive())
-    {
-      gtk_style_context_set_state(style_context, GTK_STATE_FLAG_PRELIGHT);
-    }
-
-    int y = (height - text_height) / 2;
-
-
-    unsigned int text_space = GetMaximumWidth() - x - right_padding_;
-
-    if (text_width > text_space)
-    {
-      cairo_pattern_t* linpat;
-      int out_pixels = text_width - text_space;
-      const int fading_pixels = 15;
-
-      int fading_width = out_pixels < fading_pixels ? out_pixels : fading_pixels;
-
-      cairo_push_group(cr);
-      if (overlay_showing_)
-      {
-        cairo_move_to(cr, x, y);
-        cairo_set_source_rgb(cr, 1.0f, 1.0f, 1.0f);
-        pango_cairo_show_layout(cr, layout);
-      }
-      else
-      {
-        gtk_render_layout(style_context, cr, x, y, layout);
-      }
-      cairo_pop_group_to_source(cr);
-
-      int right_margin = width - right_padding_;
-      linpat = cairo_pattern_create_linear(right_margin - fading_width, y, right_margin, y);
-      cairo_pattern_add_color_stop_rgba(linpat, 0, 0, 0, 0, 1);
-      cairo_pattern_add_color_stop_rgba(linpat, 1, 0, 0, 0, 0);
-      cairo_mask(cr, linpat);
-      cairo_pattern_destroy(linpat);
+      cairo_move_to(cr, x, y);
+      cairo_set_source_rgb(cr, 1.0f, 1.0f, 1.0f);
+      pango_cairo_show_layout(cr, layout);
     }
     else
     {
-      if (overlay_showing_)
-      {
-        cairo_move_to(cr, x, y);
-        cairo_set_source_rgb(cr, 1.0f, 1.0f, 1.0f);
-        pango_cairo_show_layout(cr, layout);
-      }
-      else
+      if (!IsActive())
       {
+        // We need to render the background under the text glyphs, or the edge
+        // of the text won't be correctly anti-aliased. See bug #723167
+        cairo_push_group(cr);
         gtk_render_layout(style_context, cr, x, y, layout);
-      }
-    }
+        std::shared_ptr<cairo_pattern_t> pat(cairo_pop_group(cr), cairo_pattern_destroy);
 
-    gtk_widget_path_free(widget_path);
-    gtk_style_context_restore(style_context);
+        cairo_push_group(cr);
+        gtk_render_background(style_context, cr, 0, 0, width, height);
+        cairo_pop_group_to_source(cr);
+        cairo_mask(cr, pat.get());
+      }
+
+      gtk_render_layout(style_context, cr, x, y, layout);
+    }
   }
+
+  gtk_style_context_restore(style_context);
 }
 
 // We need to do a couple of things here:
@@ -419,88 +380,80 @@ void PanelIndicatorEntryView::Refresh()
   cairo_t* cr;
 
   std::string label = GetLabel();
-  glib::Object<GdkPixbuf> const& pixbuf = MakePixbuf();
+  auto& panel_style = panel::Style::Instance();
 
-  unsigned int width = 0;
-  unsigned int icon_width = 0;
-  unsigned int height = panel::Style::Instance().panel_height;
+  double dpi_scale = cv_->DPIScale();
+  int width = 0;
+  int height = panel_style.PanelHeight(monitor_) / dpi_scale;
+  int icon_width = 0;
+
+  int icon_size = RawPixel((type_ != DROP_DOWN) ? 22 : 10).CP(dpi_scale);
+  glib::Object<GdkPixbuf> const& pixbuf = MakePixbuf(icon_size);
+  bool icon_scalable = false;
 
   // First lets figure out our size
   if (pixbuf && IsIconVisible())
   {
     width = gdk_pixbuf_get_width(pixbuf);
+
+    if (gdk_pixbuf_get_height(pixbuf) == icon_size)
+    {
+      icon_scalable = true;
+      width /= dpi_scale;
+    }
+
     icon_width = width;
   }
 
   if (!label.empty() && IsLabelVisible())
   {
-    using namespace panel;
-    PangoContext* cxt;
     PangoAttrList* attrs = nullptr;
-    PangoRectangle log_rect;
-    GdkScreen* screen = gdk_screen_get_default();
-    PangoFontDescription* desc = nullptr;
-    PanelItem panel_item = (type_ == MENU) ? PanelItem::MENU : PanelItem::INDICATOR;
-
-    Style& panel_style = Style::Instance();
-    std::string const& font_description = panel_style.GetFontDescription(panel_item);
-    int dpi = panel_style.GetTextDPI();
+    auto panel_item = (type_ == MENU) ? panel::PanelItem::MENU : panel::PanelItem::INDICATOR;
+    std::string const& font = panel_style.GetFontDescription(panel_item);
 
     if (proxy_->show_now())
     {
       if (!pango_parse_markup(label.c_str(), -1, '_', &attrs, nullptr, nullptr, nullptr))
       {
-        g_debug("pango_parse_markup failed");
+        LOG_WARN(logger) << "Pango markup parsing failed";
       }
     }
 
-    desc = pango_font_description_from_string(font_description.c_str());
-    pango_font_description_set_weight(desc, PANGO_WEIGHT_NORMAL);
+    glib::Object<PangoContext> context(gdk_pango_context_get_for_screen(gdk_screen_get_default()));
+    std::shared_ptr<PangoFontDescription> desc(pango_font_description_from_string(font.c_str()), pango_font_description_free);
+    pango_context_set_font_description(context, desc.get());
+    pango_context_set_language(context, gtk_get_default_language());
+    pango_cairo_context_set_resolution(context, 96.0 * Settings::Instance().font_scaling());
 
-    nux::CairoGraphics cairo_graphics(CAIRO_FORMAT_ARGB32, 1, 1);
-    cr = cairo_graphics.GetInternalContext();
-
-    layout = pango_cairo_create_layout(cr);
-    if (attrs)
-    {
-      pango_layout_set_attributes(layout, attrs);
-      pango_attr_list_unref(attrs);
-    }
-
-    pango_layout_set_font_description(layout, desc);
-
-    boost::erase_all(label, "_");
+    label.erase(std::remove(label.begin(), label.end(), '_'), label.end());
+    layout = pango_layout_new(context);
+    pango_layout_set_height(layout, -1); //avoid wrap lines
     pango_layout_set_text(layout, label.c_str(), -1);
+    pango_layout_set_attributes(layout, attrs);
+    pango_attr_list_unref(attrs);
 
-    cxt = pango_layout_get_context(layout);
-    pango_cairo_context_set_font_options(cxt, gdk_screen_get_font_options(screen));
-    pango_cairo_context_set_resolution(cxt, dpi / static_cast<float>(PANGO_SCALE));
-    pango_layout_context_changed(layout);
-
-    pango_layout_get_extents(layout, nullptr, &log_rect);
-    unsigned int text_width = log_rect.width / PANGO_SCALE;
+    nux::Size extents;
+    pango_layout_get_pixel_size(layout, &extents.width, &extents.height);
 
     if (icon_width)
-      width += spacing_;
-    width += text_width;
+      width += DEFAULT_SPACING;
 
-    pango_font_description_free(desc);
+    width += extents.width;
   }
 
   if (width)
-    width += left_padding_ + right_padding_;
+    width += padding_ * 2;
 
-  width = std::min<int>(width, GetMaximumWidth());
-  SetMinimumWidth(width);
-
-  nux::CairoGraphics cg(CAIRO_FORMAT_ARGB32, width, height);
+  SetMinMaxSize(std::ceil(width * dpi_scale), std::ceil(height * dpi_scale));
+  nux::CairoGraphics cg(CAIRO_FORMAT_ARGB32, GetWidth(), GetHeight());
+  cairo_surface_set_device_scale(cg.GetSurface(), dpi_scale, dpi_scale);
   cr = cg.GetInternalContext();
   cairo_set_line_width(cr, 1);
   cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
   cairo_paint(cr);
 
   cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-  DrawEntryContent(cr, width, height, pixbuf, layout);
+  DrawEntryContent(cr, width, height, pixbuf, icon_scalable, layout);
 
   entry_texture_ = texture_ptr_from_cairo_graphics(cg);
   SetTexture(entry_texture_.GetPointer());
@@ -553,6 +506,16 @@ void PanelIndicatorEntryView::OverlayHidden()
   Refresh();
 }
 
+void PanelIndicatorEntryView::SetMonitor(int monitor)
+{
+  if (monitor_ == monitor)
+    return;
+
+  monitor_ = monitor;
+  cv_ = Settings::Instance().em(monitor);
+  Refresh();
+}
+
 void PanelIndicatorEntryView::SetOpacity(double opacity)
 {
   opacity = CLAMP(opacity, 0.0f, 1.0f);
@@ -630,7 +593,7 @@ std::string PanelIndicatorEntryView::GetName() const
   return "IndicatorEntry";
 }
 
-void PanelIndicatorEntryView::AddProperties(GVariantBuilder* builder)
+void PanelIndicatorEntryView::AddProperties(debug::IntrospectionData& introspection)
 {
   std::string type_name;
 
@@ -642,11 +605,14 @@ void PanelIndicatorEntryView::AddProperties(GVariantBuilder* builder)
     case MENU:
       type_name = "menu";
       break;
+    case DROP_DOWN:
+      type_name = "dropdown";
+      break;
     default:
       type_name = "other";
   }
 
-  variant::BuilderWrapper(builder)
+  introspection
   .add(GetAbsoluteGeometry())
   .add("entry_id", GetEntryID())
   .add("name_hint", proxy_->name_hint())
@@ -664,6 +630,7 @@ void PanelIndicatorEntryView::AddProperties(GVariantBuilder* builder)
   .add("menu_y", proxy_->geometry().y)
   .add("menu_width", proxy_->geometry().width)
   .add("menu_height", proxy_->geometry().height)
+  .add("menu_geo", proxy_->geometry())
   .add("focused", IsFocused());
 }
 
