@@ -25,14 +25,14 @@
 #include <NuxCore/Logger.h>
 
 #include "PanelIndicatorsView.h"
-
-#include <UnityCore/Variant.h>
-#include <glib.h>
+#include "PanelIndicatorEntryDropdownView.h"
 
 DECLARE_LOGGER(logger, "unity.indicators");
 
 namespace unity
 {
+namespace panel
+{
 using namespace indicator;
 
 NUX_IMPLEMENT_OBJECT_TYPE(PanelIndicatorsView);
@@ -41,6 +41,8 @@ PanelIndicatorsView::PanelIndicatorsView()
 : View(NUX_TRACKER_LOCATION)
 , opacity(1.0f, sigc::mem_fun(this, &PanelIndicatorsView::SetOpacity))
 , layout_(new nux::HLayout("", NUX_TRACKER_LOCATION))
+, monitor_(0)
+, overlay_showing_(false)
 {
   opacity.DisableNotifications();
   layout_->SetContentDistribution(nux::MAJOR_POSITION_END);
@@ -49,6 +51,20 @@ PanelIndicatorsView::PanelIndicatorsView()
   LOG_DEBUG(logger) << "Indicators View Added: ";
 }
 
+void PanelIndicatorsView::EnableDropdownMenu(bool enable, indicator::Indicators::Ptr const& indicators)
+{
+  if (enable && indicators)
+  {
+    dropdown_ = new PanelIndicatorEntryDropdownView(GetName(), indicators);
+    AddEntryView(dropdown_.GetPointer());
+  }
+  else
+  {
+    RemoveEntryView(dropdown_.GetPointer());
+    dropdown_.Release();
+  }
+}
+
 void PanelIndicatorsView::AddIndicator(Indicator::Ptr const& indicator)
 {
   LOG_DEBUG(logger) << "IndicatorAdded: " << indicator->name();
@@ -59,7 +75,7 @@ void PanelIndicatorsView::AddIndicator(Indicator::Ptr const& indicator)
 
   auto& conn_manager = indicators_connections_[indicator];
   conn_manager.Add(indicator->on_entry_added.connect(sigc::mem_fun(this, &PanelIndicatorsView::OnEntryAdded)));
-  conn_manager.Add(indicator->on_entry_removed.connect(sigc::mem_fun(this, &PanelIndicatorsView::OnEntryRemoved)));
+  conn_manager.Add(indicator->on_entry_removed.connect(sigc::mem_fun(this, &PanelIndicatorsView::RemoveEntry)));
 }
 
 void PanelIndicatorsView::RemoveIndicator(Indicator::Ptr const& indicator)
@@ -67,9 +83,9 @@ void PanelIndicatorsView::RemoveIndicator(Indicator::Ptr const& indicator)
   indicators_connections_.erase(indicator);
 
   for (auto const& entry : indicator->GetEntries())
-    OnEntryRemoved(entry->id());
+    RemoveEntry(entry->id());
 
-  for (auto i = indicators_.begin(); i != indicators_.end(); i++)
+  for (auto i = indicators_.begin(); i != indicators_.end(); ++i)
   {
     if (*i == indicator)
     {
@@ -81,7 +97,7 @@ void PanelIndicatorsView::RemoveIndicator(Indicator::Ptr const& indicator)
   LOG_DEBUG(logger) << "IndicatorRemoved: " << indicator->name();
 }
 
-PanelIndicatorsView::Indicators PanelIndicatorsView::GetIndicators()
+PanelIndicatorsView::Indicators const& PanelIndicatorsView::GetIndicators() const
 {
   return indicators_;
 }
@@ -97,26 +113,47 @@ void PanelIndicatorsView::Draw(nux::GraphicsEngine& GfxContext, bool force_draw)
 
 void PanelIndicatorsView::SetMaximumEntriesWidth(int max_width)
 {
-  unsigned int n_entries = 0;
+  if (!dropdown_)
+    return;
 
-  for (auto const& entry : entries_)
-    if (entry.second->IsVisible())
-      n_entries++;
+  int accumolated_width = dropdown_->GetBaseWidth();
+  std::vector<PanelIndicatorEntryView::Ptr> to_hide;
 
-  if (n_entries > 0)
+  for (auto* area : layout_->GetChildren())
   {
-    for (auto const& entry : entries_)
+    auto en = static_cast<PanelIndicatorEntryView*>(area);
+    if (en == dropdown_.GetPointer())
+      continue;
+
+    accumolated_width += en->GetBaseWidth();
+
+    if (accumolated_width > max_width)
+      to_hide.push_back(PanelIndicatorEntryView::Ptr(en));
+  }
+
+  // No need to hide an item if there's space that we considered for the dropdown
+  if (!dropdown_->IsVisible() && to_hide.size() == 1)
+  {
+    if (accumolated_width - dropdown_->GetBaseWidth() < max_width)
+      to_hide.clear();
+  }
+
+  // There's just one hidden entries, it might fit in the space we have
+  if (to_hide.empty() && dropdown_->Size() == 1)
+    accumolated_width -= dropdown_->GetBaseWidth();
+
+  if (accumolated_width < max_width)
+  {
+    while (!dropdown_->Empty() && dropdown_->Top()->GetBaseWidth() < (max_width - accumolated_width))
+      AddEntryView(dropdown_->Pop().GetPointer());
+  }
+  else
+  {
+    for (auto const& hidden : to_hide)
     {
-      if (entry.second->IsVisible() && n_entries > 0)
-      {
-        int max_entry_width = max_width / n_entries;
-
-        if (entry.second->GetBaseWidth() > max_entry_width)
-          entry.second->SetMaximumWidth(max_entry_width);
-
-        max_width -= entry.second->GetBaseWidth();
-        --n_entries;
-      }
+      layout_->RemoveChildObject(hidden.GetPointer());
+      RemoveChild(hidden.GetPointer());
+      dropdown_->Push(hidden);
     }
   }
 }
@@ -130,7 +167,13 @@ PanelIndicatorEntryView* PanelIndicatorsView::ActivateEntry(std::string const& e
     PanelIndicatorEntryView* view = entry->second;
 
     if (view->IsSensitive() && view->IsVisible())
+    {
       view->Activate(button);
+    }
+    else if (dropdown_)
+    {
+      dropdown_->ActivateChild(PanelIndicatorEntryView::Ptr(view));
+    }
 
     return view;
   }
@@ -140,14 +183,9 @@ PanelIndicatorEntryView* PanelIndicatorsView::ActivateEntry(std::string const& e
 
 bool PanelIndicatorsView::ActivateIfSensitive()
 {
-  std::map<int, PanelIndicatorEntryView*> sorted_entries;
-
-  for (auto const& entry : entries_)
-    sorted_entries[entry.second->GetEntryPriority()] = entry.second;
-
-  for (auto const& entry : sorted_entries)
+  for (auto* area : layout_->GetChildren())
   {
-    PanelIndicatorEntryView* view = entry.second;
+    auto* view = static_cast<PanelIndicatorEntryView*>(area);
 
     if (view->IsSensitive() && view->IsVisible() && view->IsFocused())
     {
@@ -162,8 +200,8 @@ bool PanelIndicatorsView::ActivateIfSensitive()
 
 void PanelIndicatorsView::GetGeometryForSync(EntryLocationMap& locations)
 {
-  for (auto const& entry : entries_)
-    entry.second->GetGeometryForSync(locations);
+  for (auto* area : layout_->GetChildren())
+    static_cast<PanelIndicatorEntryView*>(area)->GetGeometryForSync(locations);
 }
 
 PanelIndicatorEntryView* PanelIndicatorsView::ActivateEntryAt(int x, int y, int button)
@@ -177,11 +215,14 @@ PanelIndicatorEntryView* PanelIndicatorsView::ActivateEntryAt(int x, int y, int
   // which causes visible lag in many cases.
   //
 
-  for (auto const& entry : entries_)
+  for (auto* area : layout_->GetChildren())
   {
-    PanelIndicatorEntryView* view = entry.second;
+    auto view = static_cast<PanelIndicatorEntryView*>(area);
 
-    if (!target && view->IsVisible() && view->IsFocused() &&
+    if (!view->IsVisible())
+      continue;
+
+    if (!target && view->IsFocused() &&
         view->IsSensitive() &&
         view->GetAbsoluteGeometry().IsPointInside(x, y))
     {
@@ -199,11 +240,11 @@ PanelIndicatorEntryView* PanelIndicatorsView::ActivateEntryAt(int x, int y, int
 
   if (target && !found_old_active)
   {
-    for (auto const& entry : entries_)
+    for (auto* area : layout_->GetChildren())
     {
-      PanelIndicatorEntryView* view = entry.second;
+      auto view = static_cast<PanelIndicatorEntryView*>(area);
 
-      if (view != target && view->IsActive())
+      if (view != target && view->IsVisible() && view->IsActive())
       {
         view->Unactivate();
         break;
@@ -226,41 +267,60 @@ void PanelIndicatorsView::AddEntryView(PanelIndicatorEntryView* view, IndicatorE
   if (!view)
     return;
 
-  int entry_pos = pos;
-
+  auto const& entry_id = view->GetEntryID();
+  bool added_to_dropdown = false;
+  bool known_entry = (entries_.find(entry_id) != entries_.end());
   view->SetOpacity(opacity());
-  view->refreshed.connect(sigc::mem_fun(this, &PanelIndicatorsView::OnEntryRefreshed));
 
-  if (entry_pos == IndicatorEntryPosition::AUTO)
+  if (!known_entry && dropdown_ && !dropdown_->Empty())
   {
-    entry_pos = nux::NUX_LAYOUT_BEGIN;
-
-    if (view->GetEntryPriority() > -1)
+    if (pos == IndicatorEntryPosition::AUTO)
     {
-      for (auto area : layout_->GetChildren())
-      {
-        auto en = dynamic_cast<PanelIndicatorEntryView*>(area);
-
-        if (en)
-        {
-          if (en && view->GetEntryPriority() <= en->GetEntryPriority())
-            break;
-
-          entry_pos++;
-        }
-      }
+      dropdown_->Insert(PanelIndicatorEntryView::Ptr(view));
+      added_to_dropdown = true;
+    }
+    else if (view->GetEntryPriority() >= dropdown_->Top()->GetEntryPriority())
+    {
+      dropdown_->Push(PanelIndicatorEntryView::Ptr(view));
+      added_to_dropdown = true;
     }
   }
 
-  layout_->AddView(view, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FULL, 1.0, (nux::LayoutPosition) entry_pos);
+  if (!added_to_dropdown)
+  {
+    int entry_pos = pos;
+    if (entry_pos == IndicatorEntryPosition::AUTO)
+    {
+      entry_pos = nux::NUX_LAYOUT_BEGIN;
 
-  entries_[view->GetEntryID()] = view;
+      if (view->GetEntryPriority() > -1)
+      {
+        for (auto area : layout_->GetChildren())
+        {
+          auto en = static_cast<PanelIndicatorEntryView*>(area);
+          if (view->GetEntryPriority() <= en->GetEntryPriority())
+            break;
 
-  AddChild(view);
-  QueueRelayout();
-  QueueDraw();
+          ++entry_pos;
+        }
+      }
+    }
 
-  on_indicator_updated.emit(view);
+    layout_->AddView(view, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FULL, 1.0, (nux::LayoutPosition) entry_pos);
+    AddChild(view);
+
+    QueueRelayout();
+    QueueDraw();
+  }
+
+  if (!known_entry)
+  {
+    view->SetMonitor(monitor_);
+    view->refreshed.connect(sigc::mem_fun(this, &PanelIndicatorsView::OnEntryRefreshed));
+    entries_.insert({entry_id, view});
+    on_indicator_updated.emit();
+    entry_added.emit(view);
+  }
 }
 
 PanelIndicatorEntryView *PanelIndicatorsView::AddEntry(Entry::Ptr const& entry, int padding, IndicatorEntryPosition pos, IndicatorEntryType type)
@@ -268,6 +328,9 @@ PanelIndicatorEntryView *PanelIndicatorsView::AddEntry(Entry::Ptr const& entry,
   auto view = new PanelIndicatorEntryView(entry, padding, type);
   AddEntryView(view, pos);
 
+  if (overlay_showing_)
+    view->OverlayShown();
+
   return view;
 }
 
@@ -280,8 +343,7 @@ void PanelIndicatorsView::OnEntryRefreshed(PanelIndicatorEntryView* view)
 {
   QueueRelayout();
   QueueDraw();
-
-  on_indicator_updated.emit(view);
+  on_indicator_updated.emit();
 }
 
 void PanelIndicatorsView::RemoveEntryView(PanelIndicatorEntryView* view)
@@ -289,11 +351,15 @@ void PanelIndicatorsView::RemoveEntryView(PanelIndicatorEntryView* view)
   if (!view)
     return;
 
-  std::string const& entry_id = view->GetEntryID();
+  entry_removed.emit(view);
+
+  if (dropdown_)
+    dropdown_->Remove(PanelIndicatorEntryView::Ptr(view));
+
   RemoveChild(view);
-  on_indicator_updated.emit(view);
-  entries_.erase(entry_id);
+  entries_.erase(view->GetEntryID());
   layout_->RemoveChildObject(view);
+  on_indicator_updated.emit();
 
   QueueRelayout();
   QueueDraw();
@@ -304,23 +370,30 @@ void PanelIndicatorsView::RemoveEntry(std::string const& entry_id)
   RemoveEntryView(entries_[entry_id]);
 }
 
-void PanelIndicatorsView::OnEntryRemoved(std::string const& entry_id)
-{
-  RemoveEntry(entry_id);
-}
-
 void PanelIndicatorsView::OverlayShown()
 {
+  overlay_showing_ = true;
+
   for (auto const& entry: entries_)
     entry.second->OverlayShown();
 }
 
 void PanelIndicatorsView::OverlayHidden()
 {
+  overlay_showing_ = false;
+
   for (auto const& entry: entries_)
     entry.second->OverlayHidden();
 }
 
+void PanelIndicatorsView::SetMonitor(int monitor)
+{
+  monitor_ = monitor;
+
+  for (auto const& entry : entries_)
+    entry.second->SetMonitor(monitor_);
+}
+
 bool PanelIndicatorsView::SetOpacity(double& target, double const& new_value)
 {
   double opacity = CLAMP(new_value, 0.0f, 1.0f);
@@ -344,12 +417,13 @@ std::string PanelIndicatorsView::GetName() const
   return "Indicators";
 }
 
-void PanelIndicatorsView::AddProperties(GVariantBuilder* builder)
+void PanelIndicatorsView::AddProperties(debug::IntrospectionData& introspection)
 {
-  variant::BuilderWrapper(builder)
+  introspection
   .add(GetAbsoluteGeometry())
   .add("entries", entries_.size())
   .add("opacity", opacity);
 }
 
+} // namespace panel
 } // namespace unity
