@@ -443,14 +443,13 @@ check_parent_window_connected(NuxAreaAccessible* self)
 gboolean
 nux_area_accessible_parent_window_active(NuxAreaAccessible* self)
 {
-  gboolean active = FALSE;
   AtkStateSet* state_set = NULL;
 
   check_parent_window_connected(self);
 
   state_set = atk_object_ref_state_set(ATK_OBJECT(self->priv->parent_window));
 
-  active = atk_state_set_contains_state(state_set, ATK_STATE_ACTIVE);
+  gboolean active = atk_state_set_contains_state(state_set, ATK_STATE_ACTIVE);
 
   g_object_unref(state_set);
 
@@ -549,10 +548,8 @@ check_focus(NuxAreaAccessible* self)
 
   if (self->priv->focused != focus_in)
   {
-    gboolean is_parent_window_active = FALSE;
-
     self->priv->focused = focus_in;
-    is_parent_window_active = nux_area_accessible_parent_window_active(self);
+    gboolean is_parent_window_active = nux_area_accessible_parent_window_active(self);
 
     /* we don't emit focus_in=TRUE events until the top level window
        is active */
