@@ -27,13 +27,27 @@
 #include <Nux/WindowThread.h>
 #include <NuxCore/Property.h>
 #include <sigc++/sigc++.h>
+#include <unordered_set>
 
 #include <scale/scale.h>
 #include <core/core.h>
 #include <core/pluginclasshandler.h>
-#include <composite/composite.h>
 #include <opengl/opengl.h>
 
+// These fixes some definitions from the composite header
+#ifdef COLOR
+#define COMPIZ_COMPOSITE_COLOR 0xffff
+#undef COLOR
+#endif
+#ifdef OPAQUE
+#define COMPIZ_COMPOSITE_OPAQUE 0xffff
+#undef OPAQUE
+#endif
+#ifdef BRIGHT
+#define COMPIZ_COMPOSITE_BRIGHT 0xffff
+#undef BRIGHT
+#endif
+
 #include "unityshell_options.h"
 
 #include "Introspectable.h"
@@ -44,17 +58,23 @@
 #include "FontSettings.h"
 #include "ShortcutController.h"
 #include "LauncherController.h"
+#include "LockScreenController.h"
+#include "LockScreenSettings.h"
 #include "PanelController.h"
 #include "PanelStyle.h"
 #include "UScreen.h"
 #include "DebugDBusInterface.h"
 #include "ScreenIntrospection.h"
+#include "ScreenSaverDBusManager.h"
 #include "SwitcherController.h"
 #include "SessionController.h"
+#include "SessionDBusManager.h"
+#include "SpreadFilter.h"
 #include "UBusWrapper.h"
 #include "UnityshellPrivate.h"
 #include "UnityShowdesktopHandler.h"
 #include "ThumbnailGenerator.h"
+#include "MenuManager.h"
 
 #include "compizminimizedwindowhandler.h"
 #include "BGHash.h"
@@ -70,6 +90,19 @@ namespace unity
 {
 class UnityWindow;
 
+namespace decoration
+{
+class Manager;
+class Window;
+enum class WidgetState : unsigned;
+}
+
+namespace compiz_utils
+{
+struct CairoContext;
+struct PixmapTexture;
+}
+
 /* base screen class */
 class UnityScreen :
   public debug::Introspectable,
@@ -77,8 +110,10 @@ class UnityScreen :
   public ScreenInterface,
   public CompositeScreenInterface,
   public GLScreenInterface,
+  public ScaleScreenInterface,
   public BaseSwitchScreen,
   public PluginClassHandler <UnityScreen, CompScreen>,
+  public CompAction::Container,
   public UnityshellOptions
 {
 public:
@@ -89,6 +124,7 @@ public:
   CompScreen* screen;
   CompositeScreen* cScreen;
   GLScreen* gScreen;
+  ScaleScreen* sScreen;
 
   /* prepares nux for drawing */
   void nuxPrologue();
@@ -97,9 +133,11 @@ public:
 
   /* nux draw wrapper */
   void paintDisplay();
-  void paintPanelShadow(const CompRegion& clip);
+  void paintPanelShadow(CompRegion const& clip);
   void setPanelShadowMatrix(const GLMatrix& matrix);
 
+  void updateBlurDamage();
+  void damageCutoff();
   void preparePaint (int ms);
   void paintFboForOutput (CompOutput *output);
   void donePaint ();
@@ -130,11 +168,17 @@ public:
 
   /* handle X11 events */
   void handleEvent(XEvent*);
+  void addSupportedAtoms(std::vector<Atom>&);
 
   /* handle showdesktop */
   void enterShowDesktopMode ();
   void leaveShowDesktopMode (CompWindow *w);
 
+   /* window scaling */
+  bool layoutSlotsAndAssignWindows();
+
+  bool showMenuBarInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
+  bool showMenuBarTerminate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool showLauncherKeyInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool showLauncherKeyTerminate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool showPanelFirstMenuKeyInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
@@ -163,8 +207,8 @@ public:
   bool launcherSwitcherForwardInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool launcherSwitcherPrevInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool launcherSwitcherTerminate(CompAction* action, CompAction::State state, CompOption::Vector& options);
-  bool rightMaximizeKeyInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
-  bool leftMaximizeKeyInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
+
+  bool LockScreenInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
 
   /* handle option changes and change settings inside of the
    * panel and dock views */
@@ -188,16 +232,22 @@ public:
 
   void OnMinimizeDurationChanged();
 
+  void OnLockScreenRequested();
+  void SaveLockStamp(bool);
+
   switcher::Controller::Ptr switcher_controller();
   launcher::Controller::Ptr launcher_controller();
+  std::shared_ptr<lockscreen::Controller> lockscreen_controller();
 
   bool DoesPointIntersectUnityGeos(nux::Point const& pt);
 
   ui::LayoutWindow::Ptr GetSwitcherDetailLayoutWindow(Window window) const;
 
+  CompAction::Vector& getActions();
+
 protected:
   std::string GetName() const;
-  void AddProperties(GVariantBuilder* builder);
+  void AddProperties(debug::IntrospectionData&);
 
 private:
   enum CancelActionTarget
@@ -218,7 +268,7 @@ private:
   void initLauncher();
 
   void compizDamageNux(CompRegion const& region);
-  void nuxDamageCompiz();
+  void determineNuxDamage(CompRegion &nux_damage);
 
   void onRedrawRequested();
   void Relayout();
@@ -231,41 +281,60 @@ private:
 
   void OnDashRealized ();
 
+  void RaiseOSK();
+
   void OnLauncherStartKeyNav(GVariant* data);
   void OnLauncherEndKeyNav(GVariant* data);
-
-  void OnSwitcherStart(GVariant* data);
-  void OnSwitcherEnd(GVariant* data);
+  void OnSwitcherDetailChanged(bool detail);
 
   void OnInitiateSpread();
   void OnTerminateSpread();
 
+  void DamagePanelShadow();
+
+  void OnViewHidden(nux::BaseWindow *bw);
+
+  void RestoreWindow(GVariant* data);
+
   bool SaveInputThenFocus(const guint xid);
 
-  void OnPanelStyleChanged();
+  void OnDecorationStyleChanged();
 
   void InitGesturesSupport();
 
   void DrawPanelUnderDash();
 
+  void FillShadowRectForOutput(CompRect &shadowRect,
+                               CompOutput const &output);
   unsigned CompizModifiersToNux(unsigned input) const;
   unsigned XModifiersToNux(unsigned input) const;
 
   void UpdateCloseWindowKey(CompAction::KeyBinding const&);
+  void UpdateActivateIndicatorsKey();
+
+  bool getMipmap () override { return false; }
+
+  void DamageBlurUpdateRegion(nux::Geometry const&);
+
+  void ShowFirstRunHints();
 
   std::unique_ptr<na::TickSource> tick_source_;
   std::unique_ptr<na::AnimationController> animation_controller_;
 
-  Settings dash_settings_;
+  Settings unity_settings_;
   dash::Style    dash_style_;
   panel::Style   panel_style_;
   FontSettings   font_settings_;
   internal::FavoriteStoreGSettings favorite_store_;
   ThumbnailGenerator thumbnail_generator_;
+  lockscreen::Settings lockscreen_settings_;
 
   /* The window thread should be the last thing removed, as c++ does it in reverse order */
   std::unique_ptr<nux::WindowThread> wt;
 
+  menu::Manager::Ptr menus_;
+  std::shared_ptr<decoration::Manager> deco_manager_;
+
   /* These must stay below the window thread, please keep the order */
   launcher::Controller::Ptr launcher_controller_;
   dash::Controller::Ptr     dash_controller_;
@@ -274,9 +343,13 @@ private:
   switcher::Controller::Ptr switcher_controller_;
   hud::Controller::Ptr      hud_controller_;
   shortcut::Controller::Ptr shortcut_controller_;
+  session::DBusManager::Ptr session_dbus_manager_;
   session::Controller::Ptr  session_controller_;
+  lockscreen::DBusManager::Ptr screensaver_dbus_manager_;
+  std::shared_ptr<lockscreen::Controller> lockscreen_controller_;
   debug::DebugDBusInterface debugger_;
   std::unique_ptr<BGHash>   bghash_;
+  spread::Filter::Ptr       spread_filter_;
 
   /* Subscription for gestures that manipulate Unity launcher */
   std::unique_ptr<nux::GesturesSubscription> gestures_sub_launcher_;
@@ -308,6 +381,12 @@ private:
   bool    _key_nav_mode_requested;
   CompOutput* _last_output;
 
+  /* a small count-down work-a-around
+   * to force full redraws of the shell
+   * a certain number of frames after a
+   * suspend / resume cycle */
+  unsigned int force_draw_countdown_;
+
   CompRegion panelShadowPainted;
   CompRegion nuxRegion;
   CompRegion fullscreenRegion;
@@ -329,7 +408,7 @@ private:
   GLMatrix panel_shadow_matrix_;
 
   bool paint_panel_under_dash_;
-  std::set<UnityWindow*> fake_decorated_windows_;
+  std::unordered_set<UnityWindow*> fake_decorated_windows_;
 
   bool scale_just_activated_;
   WindowMinimizeSpeedController minimize_speed_controller_;
@@ -341,9 +420,18 @@ private:
   UBusManager ubus_manager_;
   glib::SourceManager sources_;
 
+  CompRegion buffered_compiz_damage_this_frame_;
+  CompRegion buffered_compiz_damage_last_frame_;
+  bool       ignore_redraw_request_;
+  bool       dirty_helpers_on_this_frame_;
+
+  unsigned int back_buffer_age_;
+
   bool is_desktop_active_;
 
   friend class UnityWindow;
+  friend class debug::ScreenIntrospection;
+  friend class decoration::Manager;
 };
 
 class UnityWindow :
@@ -374,6 +462,7 @@ public:
   void activate();
 
   void updateFrameRegion(CompRegion &region);
+  void getOutputExtents(CompWindowExtents& output);
 
   /* occlusion detection
    * and window hiding */
@@ -382,6 +471,8 @@ public:
   /* basic window draw function */
   bool glDraw(GLMatrix const&, GLWindowPaintAttrib const&, CompRegion const&, unsigned mask);
 
+  bool damageRect(bool initial, CompRect const&);
+
   void updateIconPos (int &wx, int &wy, int x, int y, float width, float height);
   void windowNotify(CompWindowNotify n);
   void moveNotify(int x, int y, bool immediate);
@@ -408,14 +499,11 @@ public:
 
 protected:
   std::string GetName() const;
-  void AddProperties(GVariantBuilder* builder);
+  void AddProperties(debug::IntrospectionData&);
 
 private:
-  typedef compiz::CompizMinimizedWindowHandler<UnityScreen, UnityWindow>
-          UnityMinimizedHandler;
-  struct PixmapTexture;
-  typedef std::shared_ptr<PixmapTexture> PixmapTexturePtr;
-  struct CairoContext;
+  typedef compiz::CompizMinimizedWindowHandler<UnityScreen, UnityWindow> UnityMinimizedHandler;
+  typedef std::shared_ptr<compiz_utils::PixmapTexture> PixmapTexturePtr;
 
   void DoEnableFocus ();
   void DoDisableFocus ();
@@ -450,8 +538,8 @@ private:
 
   compiz::WindowInputRemoverLock::Ptr GetInputRemover ();
 
-  void RenderDecoration(CairoContext const&, double aspect = 1.0f);
-  void RenderText(CairoContext const&, int x, int y, int width, int height);
+  void RenderDecoration(compiz_utils::CairoContext const&, double aspect = 1.0f);
+  void RenderTitle(compiz_utils::CairoContext const&, int x, int y, int width, int height, double aspect = 1.0f);
   void DrawTexture(GLTexture::List const& textures, GLWindowPaintAttrib const&,
                    GLMatrix const&, unsigned mask, int x, int y, double aspect = 1.0f);
 
@@ -463,26 +551,21 @@ private:
 
   void BuildDecorationTexture();
   void CleanupCachedTextures();
-  static void SetupSharedTextures();
-  static void CleanupSharedTextures();
-  static void LoadCloseIcon(panel::WindowState state, GLTexture::List& texture);
 
 public:
   std::unique_ptr <UnityMinimizedHandler> mMinimizeHandler;
 
 private:
   std::unique_ptr <ShowdesktopHandler> mShowdesktopHandler;
-  static GLTexture::List close_normal_tex_;
-  static GLTexture::List close_prelight_tex_;
-  static GLTexture::List close_pressed_tex_;
-  static GLTexture::List glow_texture_;
   PixmapTexturePtr decoration_tex_;
   PixmapTexturePtr decoration_selected_tex_;
   std::string decoration_title_;
   compiz::WindowInputRemoverLock::Weak input_remover_;
-  panel::WindowState close_icon_state_;
+  decoration::WidgetState close_icon_state_;
   nux::Geometry close_button_geo_;
+  std::shared_ptr<decoration::Window> deco_win_;
   bool middle_clicked_;
+  bool need_fake_deco_redraw_;
   bool is_nux_window_;
   glib::Source::UniquePtr focus_desktop_timeout_;
 
