@@ -35,6 +35,7 @@
 
 #include <upstart.h>
 #include <nih/alloc.h>
+#include <nih/error.h>
 
 G_DEFINE_TYPE (PanelService, panel_service, G_TYPE_OBJECT);
 
@@ -52,18 +53,20 @@ G_DEFINE_TYPE (PanelService, panel_service, G_TYPE_OBJECT);
 #define SHOW_HUD_KEY "show-hud"
 
 static PanelService *static_service = NULL;
+static gboolean lockscreen_mode = FALSE;
 
 struct _PanelServicePrivate
 {
   GSList     *indicators;
+  GSList     *dropdown_entries;
   GHashTable *id2entry_hash;
   GHashTable *panel2entries_hash;
 
   gint32 timeouts[N_TIMEOUT_SLOTS];
 
   IndicatorObjectEntry *last_entry;
-  GtkWidget *menubar;
-  GtkWidget *offscreen_window;
+  IndicatorObjectEntry *last_dropdown_entry;
+  const gchar *last_panel;
   GtkMenu *last_menu;
   gint32   last_x;
   gint32   last_y;
@@ -112,7 +115,6 @@ static const gchar * indicator_order[][2] = {
   {"libapplication.so", NULL},                /* indicator-application" */
   {"floating-indicators", NULL},              /* position-less NG indicators */
   {"libprintersmenu.so", NULL},               /* indicator-printers */
-  {"libsyncindicator.so", NULL},              /* indicator-sync */
   {"libapplication.so", "gsd-keyboard-xkb"},  /* keyboard layout selector */
   {"libmessaging.so", NULL},                  /* indicator-messages */
   {"libpower.so", NULL},                      /* indicator-power */
@@ -148,32 +150,22 @@ panel_service_class_dispose (GObject *self)
   g_idle_remove_by_data (self);
   gdk_window_remove_filter (NULL, (GdkFilterFunc)event_filter, self);
 
-  if (priv->upstart != NULL)
+  if (priv->upstart != NULL && !lockscreen_mode)
     {
       int event_sent = 0;
       event_sent = upstart_emit_event_sync (NULL, priv->upstart,
                                             "indicator-services-end", NULL, 0);
       if (event_sent != 0)
-         g_warning("Unable to signal for indicator services to start");
+        {
+          NihError * err = nih_error_get();
+          g_warning("Unable to signal for indicator services to stop: %s", err->message);
+          nih_free(err);
+        }
 
       nih_unref (priv->upstart, NULL);
       priv->upstart = NULL;
     }
 
-  if (GTK_IS_WIDGET (priv->menubar) &&
-      gtk_widget_get_realized (GTK_WIDGET (priv->menubar)))
-    {
-      g_object_unref (priv->menubar);
-      priv->menubar = NULL;
-    }
-
-  if (GTK_IS_WIDGET (priv->offscreen_window) &&
-      gtk_widget_get_realized (GTK_WIDGET (priv->offscreen_window)))
-    {
-      g_object_unref (priv->offscreen_window);
-      priv->offscreen_window = NULL;
-    }
-
   if (GTK_IS_WIDGET (priv->last_menu) &&
       gtk_widget_get_realized (GTK_WIDGET (priv->last_menu)))
     {
@@ -199,6 +191,12 @@ panel_service_class_dispose (GObject *self)
       priv->gsettings = NULL;
     }
 
+  if (priv->dropdown_entries)
+    {
+      g_slist_free_full (priv->dropdown_entries, g_free);
+      priv->dropdown_entries = NULL;
+    }
+
   G_OBJECT_CLASS (panel_service_parent_class)->dispose (self);
 }
 
@@ -230,8 +228,8 @@ panel_service_class_init (PanelServiceClass *klass)
                   G_SIGNAL_RUN_LAST,
                   0,
                   NULL, NULL, NULL,
-                  G_TYPE_NONE, 5, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT,
-                  G_TYPE_UINT, G_TYPE_UINT);
+                  G_TYPE_NONE, 6, G_TYPE_STRING, G_TYPE_STRING,
+                  G_TYPE_INT, G_TYPE_INT, G_TYPE_UINT, G_TYPE_UINT);
 
   _service_signals[RE_SYNC] =
     g_signal_new ("re-sync",
@@ -278,7 +276,7 @@ panel_service_class_init (PanelServiceClass *klass)
   g_type_class_add_private (obj_class, sizeof (PanelServicePrivate));
 }
 
-static IndicatorObjectEntry *
+IndicatorObjectEntry *
 get_entry_at (PanelService *self, gint x, gint y)
 {
   GHashTableIter panel_iter, entries_iter;
@@ -306,6 +304,65 @@ get_entry_at (PanelService *self, gint x, gint y)
   return NULL;
 }
 
+static IndicatorObjectEntry *
+get_entry_at_panel (PanelService *self, const gchar *panel, gint x, gint y)
+{
+  GHashTable *entry2geometry_hash;
+  GHashTableIter entries_iter;
+  gpointer key, value;
+
+  if (!panel)
+    return NULL;
+
+  entry2geometry_hash = g_hash_table_lookup (self->priv->panel2entries_hash, panel);
+
+  if (!entry2geometry_hash)
+    return NULL;
+
+  g_hash_table_iter_init (&entries_iter, entry2geometry_hash);
+  while (g_hash_table_iter_next (&entries_iter, &key, &value))
+    {
+      IndicatorObjectEntry *entry = key;
+      GdkRectangle *geo = value;
+
+      if (x >= geo->x && x <= (geo->x + geo->width) &&
+          y >= geo->y && y <= (geo->y + geo->height))
+        {
+          return entry;
+        }
+    }
+
+  return NULL;
+}
+
+static const gchar*
+get_panel_at (PanelService *self, gint x, gint y)
+{
+  GHashTableIter panel_iter, entries_iter;
+  gpointer key, value, k, v;
+
+  g_hash_table_iter_init (&panel_iter, self->priv->panel2entries_hash);
+  while (g_hash_table_iter_next (&panel_iter, &key, &value))
+    {
+      const gchar *panel_id = key;
+      GHashTable *entry2geometry_hash = value;
+      g_hash_table_iter_init (&entries_iter, entry2geometry_hash);
+
+      while (g_hash_table_iter_next (&entries_iter, &k, &v))
+        {
+          GdkRectangle *geo = v;
+
+          if (x >= geo->x && x <= (geo->x + geo->width) &&
+              y >= geo->y && y <= (geo->y + geo->height))
+            {
+              return panel_id;
+            }
+        }
+    }
+
+  return NULL;
+}
+
 static IndicatorObject *
 get_entry_parent_indicator (IndicatorObjectEntry *entry)
 {
@@ -320,6 +377,11 @@ get_indicator_entry_id_by_entry (IndicatorObjectEntry *entry)
 {
   gchar *entry_id = NULL;
 
+  if (g_slist_find (static_service->priv->dropdown_entries, entry))
+    {
+      return g_strdup (entry->name_hint);
+    }
+
   if (entry)
     entry_id = g_strdup_printf ("%p", entry);
 
@@ -351,6 +413,17 @@ get_indicator_entry_by_id (PanelService *self, const gchar *entry_id)
         }
     }
 
+  if (!entry && g_str_has_suffix (entry_id, "-dropdown"))
+    {
+      /* Unity might register some "fake" dropdown entries that it might use to
+       * to present long menu bars (right now only for appmenu indicator) */
+      entry = g_new0 (IndicatorObjectEntry, 1);
+      entry->parent_object = panel_service_get_indicator (self, "libappmenu.so");
+      entry->name_hint = g_strdup (entry_id);
+      self->priv->dropdown_entries = g_slist_append (self->priv->dropdown_entries, entry);
+      g_hash_table_insert (self->priv->id2entry_hash, (gpointer)entry->name_hint, entry);
+    }
+
   return entry;
 }
 
@@ -419,6 +492,9 @@ event_filter (GdkXEvent *ev, GdkEvent *gev, PanelService *self)
     {
       case XI_KeyPress:
         {
+          if (lockscreen_mode)
+            break;
+
           KeySym keysym = XkbKeycodeToKeysym (event->display, event->detail, 0, 0);
 
           if (event_matches_keybinding (event->mods.base, keysym, &priv->menu_toggle) ||
@@ -447,7 +523,7 @@ event_filter (GdkXEvent *ev, GdkEvent *gev, PanelService *self)
 
       case XI_ButtonPress:
         {
-          priv->pressed_entry = get_entry_at (self, event->root_x, event->root_y);
+          priv->pressed_entry = get_entry_at_panel (self, priv->last_panel, event->root_x, event->root_y);
           priv->use_event = (priv->pressed_entry == NULL);
 
           if (priv->pressed_entry)
@@ -458,9 +534,9 @@ event_filter (GdkXEvent *ev, GdkEvent *gev, PanelService *self)
 
       case XI_ButtonRelease:
         {
-          IndicatorObjectEntry *entry;
+          IndicatorObjectEntry *entry = NULL;
           gboolean event_is_a_click = FALSE;
-          entry = get_entry_at (self, event->root_x, event->root_y);
+          entry = get_entry_at_panel (self, priv->last_panel, event->root_x, event->root_y);
 
           if (event->detail == 1 || event->detail == 3)
             {
@@ -544,12 +620,16 @@ initial_resync (PanelService *self)
 static gboolean
 ready_signal (PanelService *self)
 {
-  if (PANEL_IS_SERVICE (self) && self->priv->upstart != NULL)
+  if (PANEL_IS_SERVICE (self) && self->priv->upstart != NULL && !lockscreen_mode)
     {
       int event_sent = 0;
       event_sent = upstart_emit_event_sync (NULL, self->priv->upstart, "indicator-services-start", NULL, 0);
       if (event_sent != 0)
-         g_warning ("Unable to signal for indicator services to start");
+        {
+          NihError * err = nih_error_get();
+          g_warning("Unable to signal for indicator services to start: %s", err->message);
+          nih_free(err);
+        }
     }
 
   return FALSE;
@@ -648,10 +728,6 @@ panel_service_init (PanelService *self)
   PanelServicePrivate *priv;
   priv = self->priv = GET_PRIVATE (self);
 
-  priv->offscreen_window = gtk_offscreen_window_new ();
-  priv->menubar = gtk_menu_bar_new ();
-  gtk_container_add (GTK_CONTAINER (priv->offscreen_window), priv->menubar);
-
   gdk_window_add_filter (NULL, (GdkFilterFunc)event_filter, self);
 
   priv->id2entry_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
@@ -672,7 +748,7 @@ panel_service_init (PanelService *self)
   update_keybinding (priv->gsettings, SHOW_HUD_KEY, &priv->show_hud);
 
   const gchar *upstartsession = g_getenv ("UPSTART_SESSION");
-  if (upstartsession != NULL)
+  if (upstartsession != NULL && !lockscreen_mode)
     {
       DBusConnection *conn = dbus_connection_open (upstartsession, NULL);
       if (conn != NULL)
@@ -681,6 +757,12 @@ panel_service_init (PanelService *self)
                                               NULL,
                                               DBUS_PATH_UPSTART,
                                               NULL, NULL);
+          if (priv->upstart == NULL)
+            {
+              NihError * err = nih_error_get();
+              g_warning("Unable to get Upstart proxy: %s", err->message);
+              nih_free(err);
+            }
           dbus_connection_unref (conn);
         }
     }
@@ -811,7 +893,11 @@ initial_load_default_or_custom_indicators (PanelService *self, GList *indicators
 
   if (!indicators)
     {
-      load_indicators (self);
+      if (!lockscreen_mode)
+        {
+          load_indicators (self);
+        }
+
       load_indicators_from_indicator_files (self);
       sort_indicators (self);
     }
@@ -860,6 +946,12 @@ panel_service_get_default_with_indicators (GList *indicators)
   return self;
 }
 
+void
+panel_service_set_lockscreen_mode (gboolean enable)
+{
+  lockscreen_mode = enable;
+}
+
 guint
 panel_service_get_n_indicators (PanelService *self)
 {
@@ -1265,7 +1357,7 @@ load_indicators_from_indicator_files (PanelService *self)
       IndicatorNg *indicator;
 
       filename = g_build_filename (INDICATOR_SERVICE_DIR, name, NULL);
-      indicator = indicator_ng_new_for_profile (filename, "desktop", &error);
+      indicator = indicator_ng_new_for_profile (filename, !lockscreen_mode ? "desktop" : "desktop_lockscreen", &error);
       if (indicator)
         {
           load_indicator (self, INDICATOR_OBJECT (indicator), name);
@@ -1350,55 +1442,81 @@ sort_indicators (PanelService *self)
 }
 
 static gchar *
-gtk_image_to_data (GtkImage *image)
+gtk_image_to_data (GtkImage *image, guint32 *storage_type)
 {
-  GtkImageType type = gtk_image_get_storage_type (image);
+  if (!GTK_IS_IMAGE (image))
+    return NULL;
+
+  *storage_type = gtk_image_get_storage_type (image);
   gchar *ret = NULL;
 
-  if (type == GTK_IMAGE_PIXBUF)
+  switch (*storage_type)
     {
-      GdkPixbuf  *pixbuf;
-      gchar      *buffer = NULL;
-      gsize       buffer_size = 0;
-      GError     *error = NULL;
+      case GTK_IMAGE_PIXBUF:
+      {
+        gchar *file;
+        g_object_get (image, "file", &file, NULL);
 
-      pixbuf = gtk_image_get_pixbuf (image);
+        if (file)
+          {
+            if (file[0] != '\0')
+              {
+                *storage_type = GTK_IMAGE_ICON_NAME;
+                ret = file;
+                break;
+              }
 
-      if (gdk_pixbuf_save_to_buffer (pixbuf, &buffer, &buffer_size, "png", &error, NULL))
-        {
-          ret = g_base64_encode ((const guchar *)buffer, buffer_size);
-          g_free (buffer);
-        }
-      else
-        {
-          g_warning ("Unable to convert pixbuf to png data: '%s'", error ? error->message : "unknown");
-          if (error)
-            g_error_free (error);
+            g_free (file);
+          }
 
-          ret = g_strdup ("");
-        }
-    }
-  else if (type == GTK_IMAGE_STOCK)
-    {
-      g_object_get (G_OBJECT (image), "stock", &ret, NULL);
-    }
-  else if (type == GTK_IMAGE_ICON_NAME)
-    {
-      g_object_get (G_OBJECT (image), "icon-name", &ret, NULL);
-    }
-  else if (type == GTK_IMAGE_GICON)
-    {
-      GIcon *icon = NULL;
-      gtk_image_get_gicon (image, &icon, NULL);
-      if (G_IS_ICON (icon))
-        {
+        GdkPixbuf  *pixbuf;
+        gchar      *buffer = NULL;
+        gsize       buffer_size = 0;
+        GError     *error = NULL;
+
+        pixbuf = gtk_image_get_pixbuf (image);
+
+        if (gdk_pixbuf_save_to_buffer (pixbuf, &buffer, &buffer_size, "png", &error, NULL))
+          {
+            ret = g_base64_encode ((const guchar *)buffer, buffer_size);
+            g_free (buffer);
+          }
+        else
+          {
+            g_warning ("Unable to convert pixbuf to png data: '%s'", error ? error->message : "unknown");
+            if (error)
+              g_error_free (error);
+          }
+
+        break;
+      }
+      case GTK_IMAGE_STOCK:
+      {
+        g_object_get (G_OBJECT (image), "stock", &ret, NULL);
+        break;
+      }
+      case GTK_IMAGE_ICON_NAME:
+      {
+        g_object_get (G_OBJECT (image), "icon-name", &ret, NULL);
+        break;
+      }
+      case GTK_IMAGE_GICON:
+      {
+        GIcon *icon = NULL;
+        gtk_image_get_gicon (image, &icon, NULL);
+        if (G_IS_ICON (icon))
           ret = g_icon_to_string (icon);
-        }
-    }
-  else
-    {
-      ret = g_strdup ("");
-      g_warning ("Unable to support GtkImageType: %d", type);
+
+        break;
+      }
+      case GTK_IMAGE_EMPTY:
+      {
+        break;
+      }
+    default:
+      {
+        g_warning ("Unable to support GtkImageType: %u", *storage_type);
+      }
     }
 
   return ret;
@@ -1413,7 +1531,8 @@ indicator_entry_to_variant (IndicatorObjectEntry *entry,
 {
   gboolean is_label = GTK_IS_LABEL (entry->label);
   gboolean is_image = GTK_IS_IMAGE (entry->image);
-  gchar *image_data = NULL;
+  guint32 image_type = 0;
+  gchar *image_data = gtk_image_to_data (entry->image, &image_type);
 
   g_variant_builder_add (b, "(ssssbbusbbi)",
                          indicator_id,
@@ -1422,8 +1541,8 @@ indicator_entry_to_variant (IndicatorObjectEntry *entry,
                          is_label ? gtk_label_get_label (entry->label) : "",
                          is_label ? gtk_widget_get_sensitive (GTK_WIDGET (entry->label)) : FALSE,
                          is_label ? gtk_widget_get_visible (GTK_WIDGET (entry->label)) : FALSE,
-                         is_image ? (guint32)gtk_image_get_storage_type (entry->image) : (guint32) 0,
-                         is_image ? (image_data = gtk_image_to_data (entry->image)) : "",
+                         is_image ? image_type : 0,
+                         image_data ? image_data : "",
                          is_image ? gtk_widget_get_sensitive (GTK_WIDGET (entry->image)) : FALSE,
                          is_image ? gtk_widget_get_visible (GTK_WIDGET (entry->image)) : FALSE,
                          prio);
@@ -1504,6 +1623,14 @@ indicator_object_to_variant (IndicatorObject *object, const gchar *indicator_id,
     }
 }
 
+static int
+get_monitor_scale_at (gint x, gint y)
+{
+  GdkScreen *screen = gdk_screen_get_default ();
+  int monitor = gdk_screen_get_monitor_at_point (screen, x, y);
+  return gdk_screen_get_monitor_scale_factor (screen, monitor);
+}
+
 static void
 positon_menu (GtkMenu  *menu,
               gint     *x,
@@ -1514,8 +1641,13 @@ positon_menu (GtkMenu  *menu,
   PanelService *self = PANEL_SERVICE (user_data);
   PanelServicePrivate *priv = self->priv;
 
-  *x = priv->last_x;
-  *y = priv->last_y;
+  GdkScreen *screen = gdk_screen_get_default ();
+  gint monitor = gdk_screen_get_monitor_at_point (screen, priv->last_x, priv->last_y);
+  gtk_menu_set_monitor (menu, monitor);
+
+  gint scale = gdk_screen_get_monitor_scale_factor (screen, monitor);
+  *x = priv->last_x / scale;
+  *y = priv->last_y / scale;
   *push = TRUE;
 }
 
@@ -1523,11 +1655,13 @@ static void
 on_active_menu_hidden (GtkMenu *menu, PanelService *self)
 {
   PanelServicePrivate *priv = self->priv;
+  g_signal_handlers_disconnect_by_data (priv->last_menu, self);
 
   priv->last_x = 0;
   priv->last_y = 0;
   priv->last_menu_button = 0;
 
+  priv->last_panel = NULL;
   priv->last_menu = NULL;
   priv->last_entry = NULL;
   priv->last_left = 0;
@@ -1538,7 +1672,7 @@ on_active_menu_hidden (GtkMenu *menu, PanelService *self)
   priv->use_event = FALSE;
   priv->pressed_entry = NULL;
 
-  g_signal_emit (self, _service_signals[ENTRY_ACTIVATED], 0, "", 0, 0, 0, 0);
+  g_signal_emit (self, _service_signals[ENTRY_ACTIVATED], 0, "", "", 0, 0, 0, 0);
 }
 
 
@@ -1637,6 +1771,13 @@ panel_service_sync_geometry (PanelService *self,
 
       if (width < 0 || height < 0 || !valid_entry)
         {
+          /* If the entry has been removed let's make sure that its menu is closed */
+          if (valid_entry && GTK_IS_MENU (priv->last_menu) && priv->last_menu == entry->menu)
+            {
+              if (!priv->last_panel || g_strcmp0 (priv->last_panel, panel_id) == 0)
+                gtk_menu_popdown (entry->menu);
+            }
+
           if (entry2geometry_hash)
             {
               if (g_hash_table_size (entry2geometry_hash) > 1)
@@ -1648,12 +1789,6 @@ panel_service_sync_geometry (PanelService *self,
                   g_hash_table_remove (priv->panel2entries_hash, panel_id);
                 }
             }
-
-          /* If the entry has been removed let's make sure that its menu is closed */
-          if (valid_entry && GTK_IS_MENU (priv->last_menu) && priv->last_menu == entry->menu)
-            {
-              gtk_menu_popdown (entry->menu);
-            }
         }
       else
         {
@@ -1679,7 +1814,8 @@ panel_service_sync_geometry (PanelService *self,
 
           /* If the current entry geometry has changed, we need to move the menu
            * accordingly to the change we recorded! */
-          if (GTK_IS_MENU (priv->last_menu) && priv->last_menu == entry->menu)
+          if (GTK_IS_MENU (priv->last_menu) && priv->last_menu == entry->menu &&
+              g_strcmp0 (priv->last_panel, panel_id) == 0)
             {
               GtkWidget *top_widget = gtk_widget_get_toplevel (GTK_WIDGET (priv->last_menu));
 
@@ -1688,8 +1824,9 @@ panel_service_sync_geometry (PanelService *self,
                   GtkWindow *top_win = GTK_WINDOW (top_widget);
                   gint old_x, old_y;
 
+                  gint scale = get_monitor_scale_at (x, y);
                   gtk_window_get_position (top_win, &old_x, &old_y);
-                  gtk_window_move (top_win, old_x - (geo->x - x), old_y - (geo->y - y));
+                  gtk_window_move (top_win, old_x - (geo->x - x) / scale, old_y - (geo->y - y) / scale);
                 }
             }
 
@@ -1710,29 +1847,9 @@ panel_service_sync_geometry (PanelService *self,
 static gboolean
 panel_service_entry_is_visible (PanelService *self, IndicatorObjectEntry *entry)
 {
-  GHashTableIter panel_iter;
-  gpointer key, value;
-  gboolean found_geo;
-
   g_return_val_if_fail (PANEL_IS_SERVICE (self), FALSE);
   g_return_val_if_fail (entry != NULL, FALSE);
 
-  found_geo = FALSE;
-  g_hash_table_iter_init (&panel_iter, self->priv->panel2entries_hash);
-
-  while (g_hash_table_iter_next (&panel_iter, &key, &value) && !found_geo)
-    {
-      GHashTable *entry2geometry_hash = value;
-
-      if (g_hash_table_lookup (entry2geometry_hash, entry))
-        {
-          found_geo = TRUE;
-        }
-    }
-
-  if (!found_geo)
-    return FALSE;
-
   if (GTK_IS_LABEL (entry->label))
     {
       if (gtk_widget_get_visible (GTK_WIDGET (entry->label)) &&
@@ -1751,7 +1868,55 @@ panel_service_entry_is_visible (PanelService *self, IndicatorObjectEntry *entry)
         }
     }
 
-  return TRUE;
+  if (g_slist_find (self->priv->dropdown_entries, entry))
+    return TRUE;
+
+  return FALSE;
+}
+
+static gboolean
+panel_service_entry_is_visible_on_panel (PanelService *self,
+                                         IndicatorObjectEntry *entry,
+                                         const gchar *panel_id)
+{
+  GHashTable *entry2geometry_hash;
+  GHashTableIter panel_iter;
+  gpointer key, value;
+  gboolean found_geo;
+
+  g_return_val_if_fail (PANEL_IS_SERVICE (self), FALSE);
+  g_return_val_if_fail (entry != NULL, FALSE);
+
+  found_geo = FALSE;
+
+  if (panel_id)
+    {
+      entry2geometry_hash = g_hash_table_lookup (self->priv->panel2entries_hash, panel_id);
+
+      if (entry2geometry_hash && g_hash_table_lookup (entry2geometry_hash, entry))
+        {
+          found_geo = TRUE;
+        }
+    }
+  else
+    {
+      g_hash_table_iter_init (&panel_iter, self->priv->panel2entries_hash);
+
+      while (g_hash_table_iter_next (&panel_iter, &key, &value) && !found_geo)
+        {
+          entry2geometry_hash = value;
+
+          if (g_hash_table_lookup (entry2geometry_hash, entry))
+            {
+              found_geo = TRUE;
+            }
+        }
+    }
+
+  if (!found_geo)
+    return FALSE;
+
+  return panel_service_entry_is_visible (self, entry);
 }
 
 static int
@@ -1762,7 +1927,6 @@ indicator_entry_compare_func (gpointer* v1, gpointer* v2)
 
 static void
 activate_next_prev_menu (PanelService         *self,
-                         IndicatorObject      *object,
                          IndicatorObjectEntry *entry,
                          GtkMenuDirectionType  direction)
 {
@@ -1772,25 +1936,41 @@ activate_next_prev_menu (PanelService         *self,
   GList  *ordered_entries = NULL;
   GList  *entries;
   gchar  *id;
-  GSList *l;
+  GSList *l, *sl;
   GList *ll;
 
   for (l = indicators; l; l = l->next)
     {
-      gint parent_priority = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (l->data), "priority"));
-      entries = indicator_object_get_entries (l->data);
-      const gchar *indicator_id = g_object_get_data (G_OBJECT (l->data), "id");
+      IndicatorObject *object = l->data;
+      gint parent_priority = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (object), "priority"));
+      entries = indicator_object_get_entries (object);
+      const gchar *indicator_id = g_object_get_data (G_OBJECT (object), "id");
+
       if (entries)
         {
           int index = 0;
 
+          if (!priv->last_dropdown_entry)
+            {
+              for (sl = priv->dropdown_entries; sl; sl = sl->next)
+                {
+                  IndicatorObjectEntry *dropdown = sl->data;
+
+                  if (dropdown->parent_object == object)
+                    entries = g_list_append (entries, dropdown);
+                }
+            }
+
           for (ll = entries; ll; ll = ll->next)
             {
               gint prio = -1;
               new_entry = ll->data;
 
-              if (!panel_service_entry_is_visible (self, new_entry))
-                continue;
+              if (!priv->last_dropdown_entry || new_entry != entry)
+                {
+                  if (!panel_service_entry_is_visible_on_panel (self, new_entry, priv->last_panel))
+                    continue;
+                }
 
               if (new_entry->name_hint)
                 {
@@ -1850,7 +2030,7 @@ on_active_menu_move_current (GtkMenu              *menu,
                              PanelService         *self)
 {
   PanelServicePrivate *priv;
-  IndicatorObject     *object;
+  IndicatorObjectEntry *entry;
 
   g_return_if_fail (PANEL_IS_SERVICE (self));
   priv = self->priv;
@@ -1869,37 +2049,26 @@ on_active_menu_move_current (GtkMenu              *menu,
           GtkWidget *item = (GtkWidget *)c->data;
 
           if (GTK_IS_MENU_ITEM (item)
-              && gtk_widget_get_state (item) == GTK_STATE_PRELIGHT
+              && gtk_widget_get_state_flags (item) & GTK_STATE_FLAG_PRELIGHT
               && gtk_menu_item_get_submenu (GTK_MENU_ITEM (item)))
             {
               /* Skip direction due to there being a submenu,
                * and we don't want to inhibit going into that */
+              g_list_free (children);
               return;
             }
         }
       g_list_free (children);
+
+      entry = (priv->last_dropdown_entry) ? priv->last_dropdown_entry : priv->last_entry;
+    }
+  else
+    {
+      entry = priv->last_entry;
     }
 
   /* Find the next/prev indicator */
-  object = get_entry_parent_indicator(priv->last_entry);
-  if (object == NULL)
-    {
-      g_warning ("Unable to find IndicatorObject for entry");
-      return;
-    }
-
-  activate_next_prev_menu (self, object, priv->last_entry, direction);
-}
-
-static void
-menu_deactivated (GtkWidget *menu)
-{
-  g_signal_handlers_disconnect_by_func (menu, menu_deactivated, NULL);
-
-  if (g_object_is_floating (menu))
-    g_object_ref_sink (menu);
-
-  g_object_unref (menu);
+  activate_next_prev_menu (self, entry, direction);
 }
 
 static void
@@ -1913,6 +2082,7 @@ static void
 panel_service_show_entry_common (PanelService *self,
                                  IndicatorObject *object,
                                  IndicatorObjectEntry *entry,
+                                 GtkMenu      *fake_menu,
                                  guint32       xid,
                                  gint32        x,
                                  gint32        y,
@@ -1939,6 +2109,7 @@ panel_service_show_entry_common (PanelService *self,
 
       g_signal_handlers_disconnect_by_data (priv->last_menu, self);
 
+      priv->last_panel = NULL;
       priv->last_entry = NULL;
       priv->last_menu = NULL;
       priv->last_menu_button = 0;
@@ -1946,7 +2117,7 @@ panel_service_show_entry_common (PanelService *self,
 
   if (entry != NULL)
     {
-      if (GTK_IS_MENU (entry->menu))
+      if (GTK_IS_MENU (entry->menu) || fake_menu)
         {
           if (xid > 0)
             {
@@ -1957,7 +2128,7 @@ panel_service_show_entry_common (PanelService *self,
               indicator_object_entry_activate (object, entry, CurrentTime);
             }
 
-          priv->last_menu = entry->menu;
+          priv->last_menu = (fake_menu) ? fake_menu : entry->menu;
         }
       else
         {
@@ -1971,7 +2142,7 @@ panel_service_show_entry_common (PanelService *self,
           gtk_widget_show (menu_item);
 
           g_signal_connect (priv->last_menu, "deactivate",
-                            G_CALLBACK (menu_deactivated), NULL);
+                            G_CALLBACK (gtk_widget_destroy), NULL);
           g_signal_connect (priv->last_menu, "destroy",
                             G_CALLBACK (gtk_widget_destroyed), &priv->last_menu);
           g_signal_connect (menu_item, "activate",
@@ -1982,6 +2153,8 @@ panel_service_show_entry_common (PanelService *self,
       priv->last_x = x;
       priv->last_y = y;
       priv->last_menu_button = button;
+      priv->last_panel = get_panel_at (self, x, y);
+
       g_signal_connect (priv->last_menu, "hide", G_CALLBACK (on_active_menu_hidden), self);
       g_signal_connect_after (priv->last_menu, "move-current",
                               G_CALLBACK (on_active_menu_move_current), self);
@@ -1998,8 +2171,8 @@ panel_service_show_entry_common (PanelService *self,
           gdk_window_get_origin (gdkwin, &left, &top);
 
           gchar *entry_id = get_indicator_entry_id_by_entry (entry);
-          g_signal_emit (self, _service_signals[ENTRY_ACTIVATED], 0, entry_id,
-                         left, top, width, height);
+          g_signal_emit (self, _service_signals[ENTRY_ACTIVATED], 0,
+                         priv->last_panel, entry_id, left, top, width, height);
           g_free (entry_id);
 
           priv->last_left = left;
@@ -2040,7 +2213,133 @@ panel_service_show_entry (PanelService *self,
   entry = get_indicator_entry_by_id (self, entry_id);
   object = get_entry_parent_indicator (entry);
 
-  panel_service_show_entry_common (self, object, entry, xid, x, y, button);
+  panel_service_show_entry_common (self, object, entry, NULL, xid, x, y, button);
+}
+
+static void
+on_drop_down_menu_hidden (GtkWidget *menu)
+{
+  GList *l, *children;
+
+  children = gtk_container_get_children (GTK_CONTAINER (menu));
+
+  for (l = children; l; l = l->next)
+  {
+    if (!GTK_IS_MENU_ITEM (l->data))
+      continue;
+
+    GtkMenuItem *menu_item = GTK_MENU_ITEM (l->data);
+    GtkWidget *entry_menu = gtk_menu_item_get_submenu (menu_item);
+    GtkWidget *old_attached = g_object_get_data (G_OBJECT (entry_menu), "ups-attached-widget");
+
+    gtk_menu_item_set_submenu (menu_item, NULL);
+
+    if (old_attached)
+      gtk_menu_attach_to_widget (GTK_MENU (entry_menu), old_attached, NULL);
+  }
+
+  g_list_free (children);
+  gtk_widget_destroy (menu);
+}
+
+void
+panel_service_show_entries (PanelService *self,
+                            gchar       **entries,
+                            const gchar  *selected,
+                            guint32       xid,
+                            gint32        x,
+                            gint32        y)
+{
+  gint                 i;
+  IndicatorObject      *object;
+  IndicatorObjectEntry *entry, *selected_entry, *first_entry, *last_entry;
+  GtkWidget            *menu;
+
+  g_return_if_fail (PANEL_IS_SERVICE (self));
+  g_return_if_fail (entries && entries[0]);
+
+  first_entry = get_indicator_entry_by_id (self, entries[0]);
+
+  if (first_entry == self->priv->last_entry)
+    return;
+
+  if (!first_entry)
+    {
+      g_warning ("Impossible to show entries for an invalid entry or object");
+      return;
+    }
+
+  last_entry = NULL;
+  menu = gtk_menu_new ();
+  selected_entry = get_indicator_entry_by_id (self, selected);
+  object = get_entry_parent_indicator (first_entry);
+
+  for (i = 0; entries[i]; ++i)
+    {
+      entry = get_indicator_entry_by_id (self, entries[i]);
+
+      if (entry != first_entry && !last_entry)
+        continue;
+
+      GtkWidget *menu_item;
+      const char *label = NULL;
+
+      if (GTK_IS_LABEL (entry->label))
+        label = gtk_label_get_label (entry->label);
+
+      if (GTK_IS_IMAGE (entry->image))
+        {
+          G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+          if (label)
+            {
+              menu_item = gtk_image_menu_item_new_with_mnemonic (label);
+            }
+          else
+            {
+              menu_item = gtk_image_menu_item_new ();
+            }
+
+          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), GTK_WIDGET (entry->image));
+          gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (menu_item), TRUE);
+          G_GNUC_END_IGNORE_DEPRECATIONS
+        }
+      else if (label)
+        {
+          menu_item = gtk_menu_item_new_with_mnemonic (label);
+        }
+      else
+        {
+          continue;
+        }
+
+      GtkWidget *old_attached = gtk_menu_get_attach_widget (entry->menu);
+      if (old_attached)
+        {
+          g_object_set_data (G_OBJECT (entry->menu), "ups-attached-widget", old_attached);
+          gtk_menu_detach (entry->menu);
+        }
+
+      gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+      gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), GTK_WIDGET (entry->menu));
+      gtk_widget_show (menu_item);
+      last_entry = entry;
+
+      if (entry == selected_entry)
+        gtk_menu_shell_select_item (GTK_MENU_SHELL (menu), menu_item);
+    }
+
+  if (!last_entry)
+    {
+      g_warning ("No valid entries to show");
+      gtk_widget_destroy (menu);
+      return;
+    }
+
+  g_signal_connect_after (menu, "hide", G_CALLBACK (on_drop_down_menu_hidden), NULL);
+  g_signal_connect (menu, "destroy", G_CALLBACK (gtk_widget_destroyed), &self->priv->last_dropdown_entry);
+
+  panel_service_show_entry_common (self, object, first_entry, GTK_MENU (menu), xid, x, y, 1);
+  self->priv->last_dropdown_entry = last_entry;
 }
 
 void
@@ -2062,7 +2361,7 @@ panel_service_show_app_menu (PanelService *self, guint32 xid, gint32 x, gint32 y
       entry = entries->data;
       g_list_free (entries);
 
-      panel_service_show_entry_common (self, object, entry, xid, x, y, 1);
+      panel_service_show_entry_common (self, object, entry, NULL, xid, x, y, 1);
     }
 }
 
