@@ -32,6 +32,7 @@ namespace
   // Compiz' plug-in names
   const std::string CORE_PLUGIN_NAME = "core";
   const std::string EXPO_PLUGIN_NAME = "expo";
+  const std::string GRID_PLUGIN_NAME = "grid";
   const std::string MOVE_PLUGIN_NAME = "move";
   const std::string RESIZE_PLUGIN_NAME = "resize";
   const std::string SCALE_PLUGIN_NAME = "scale";
@@ -48,6 +49,9 @@ namespace
   // Compiz Expo Options
   const std::string EXPO_OPTION_EXPO_KEY = "expo_key";
 
+  // Compiz Grid Options
+  const std::string GRID_OPTION_LEFT_MAXIMIZE = "left_maximize";
+
   // Compiz Move Options
   const std::string MOVE_OPTION_INITIATE_BUTTON = "initiate_button";
 
@@ -55,6 +59,7 @@ namespace
   const std::string RESIZE_OPTION_INITIATE_BUTTON = "initiate_button";
 
   // Compiz Scale Options
+  const std::string SCALE_OPTION_INITIATE_KEY = "initiate_key";
   const std::string SCALE_OPTION_INITIATE_ALL_KEY = "initiate_all_key";
 
   // Compiz Unityshell Options
@@ -64,11 +69,8 @@ namespace
   const std::string UNITYSHELL_OPTION_SHOW_HUD = "show_hud";
   const std::string UNITYSHELL_OPTION_PANEL_FIRST_MENU = "panel_first_menu";
   const std::string UNITYSHELL_OPTION_ALT_TAB_FORWARD = "alt_tab_forward";
+  const std::string UNITYSHELL_OPTION_ALT_TAB_FORWARD_ALL = "alt_tab_forward_all";
   const std::string UNITYSHELL_OPTION_ALT_TAB_NEXT_WINDOW = "alt_tab_next_window";
-  const std::string UNITYSHELL_OPTION_MAXIMIZE = "window_maximize";
-  const std::string UNITYSHELL_OPTION_LEFT_MAXIMIZE = "window_left_maximize";
-  const std::string UNITYSHELL_OPTION_RIGHT_MAXIMIZE = "window_right_maximize";
-  const std::string UNITYSHELL_OPTION_RESTORE_MINIMIZE = "window_restore_minimize";
 
   // Compiz Wall Options
   const std::string WALL_OPTION_LEFT_KEY = "left_key";
@@ -90,27 +92,26 @@ Model::Ptr CompizModeller::GetCurrentModel() const
 void CompizModeller::BuildModel(int hsize, int vsize)
 {
   std::list<shortcut::AbstractHint::Ptr> hints;
+  bool workspace_enabled = (hsize * vsize > 1);
 
-  if (hsize * vsize > 1)
+  if (workspace_enabled)
   {
-    // Workspaces enabled
     AddLauncherHints(hints);
     AddDashHints(hints);
     AddMenuHints(hints);
-    AddSwitcherHints(hints);
+    AddSwitcherHints(hints, workspace_enabled);
     AddWorkspaceHints(hints);
-    AddWindowsHints(hints);
   }
   else
   {
-    // Workspaces disabled
     AddLauncherHints(hints);
     AddMenuHints(hints);
-    AddSwitcherHints(hints);
+    AddSwitcherHints(hints, workspace_enabled);
     AddDashHints(hints);
-    AddWindowsHints(hints);
   }
 
+  AddWindowsHints(hints, workspace_enabled);
+
   model_ = std::make_shared<shortcut::Model>(hints);
   model_changed.emit(model_);
 }
@@ -239,7 +240,7 @@ void CompizModeller::AddMenuHints(std::list<shortcut::AbstractHint::Ptr> &hints)
                                                    _("Cursor Left or Right")));
 }
 
-void CompizModeller::AddSwitcherHints(std::list<shortcut::AbstractHint::Ptr> &hints)
+void CompizModeller::AddSwitcherHints(std::list<shortcut::AbstractHint::Ptr> &hints, bool ws_enabled)
 {
   static const std::string switching(_("Switching"));
 
@@ -249,6 +250,15 @@ void CompizModeller::AddSwitcherHints(std::list<shortcut::AbstractHint::Ptr> &hi
                                                    UNITYSHELL_PLUGIN_NAME,
                                                    UNITYSHELL_OPTION_ALT_TAB_FORWARD));
 
+  if (ws_enabled)
+  {
+    hints.push_back(std::make_shared<shortcut::Hint>(switching, "", "",
+                                                     _("Switches between applications from all workspaces."),
+                                                     shortcut::OptionType::COMPIZ_KEY,
+                                                     UNITYSHELL_PLUGIN_NAME,
+                                                     UNITYSHELL_OPTION_ALT_TAB_FORWARD_ALL));
+  }
+
   hints.push_back(std::make_shared<shortcut::Hint>(switching, "", "",
                                                    _("Switches windows of current applications."),
                                                    shortcut::OptionType::COMPIZ_KEY,
@@ -284,15 +294,26 @@ void CompizModeller::AddWorkspaceHints(std::list<shortcut::AbstractHint::Ptr> &h
                                                    WALL_OPTION_LEFT_WINDOW_KEY));
 }
 
-void CompizModeller::AddWindowsHints(std::list<shortcut::AbstractHint::Ptr> &hints)
+void CompizModeller::AddWindowsHints(std::list<shortcut::AbstractHint::Ptr> &hints, bool ws_enabled)
 {
   static const std::string windows(_("Windows"));
 
   hints.push_back(std::make_shared<shortcut::Hint>(windows, "", "",
-                                                   _("Spreads all windows in the current workspace."),
+                                                   (ws_enabled ?
+                                                    _("Spreads all windows in the current workspace.") :
+                                                    _("Spreads all windows.")),
                                                    shortcut::OptionType::COMPIZ_KEY,
                                                    SCALE_PLUGIN_NAME,
-                                                   SCALE_OPTION_INITIATE_ALL_KEY));
+                                                   SCALE_OPTION_INITIATE_KEY));
+
+  if (ws_enabled)
+  {
+    hints.push_back(std::make_shared<shortcut::Hint>(windows, "", "",
+                                                     _("Spreads all windows in all the workspaces."),
+                                                     shortcut::OptionType::COMPIZ_KEY,
+                                                     SCALE_PLUGIN_NAME,
+                                                     SCALE_OPTION_INITIATE_ALL_KEY));
+  }
 
   hints.push_back(std::make_shared<shortcut::Hint>(windows, "", "",
                                                    _("Minimises all windows."),
@@ -315,8 +336,8 @@ void CompizModeller::AddWindowsHints(std::list<shortcut::AbstractHint::Ptr> &hin
   hints.push_back(std::make_shared<shortcut::Hint>(windows, "", _(" or Right"),
                                                    _("Semi-maximise the current window."),
                                                    shortcut::OptionType::COMPIZ_KEY,
-                                                   UNITYSHELL_PLUGIN_NAME,
-                                                   UNITYSHELL_OPTION_LEFT_MAXIMIZE));
+                                                   GRID_PLUGIN_NAME,
+                                                   GRID_OPTION_LEFT_MAXIMIZE));
 
   hints.push_back(std::make_shared<shortcut::Hint>(windows, "", "",
                                                    _("Closes the current window."),
