@@ -41,8 +41,9 @@ public:
 
   Model(std::list<AbstractHint::Ptr> const& hints);
 
-  std::vector<std::string>& categories() { return categories_; }
-  std::map<std::string, std::list<AbstractHint::Ptr>>& hints() { return hints_; }
+  nux::Property<int> categories_per_column;
+  std::vector<std::string> const& categories() const { return categories_; }
+  std::map<std::string, std::list<AbstractHint::Ptr>> const& hints() const { return hints_; }
 
   void Fill();
 
