@@ -29,6 +29,7 @@ class LauncherIconsTests(LauncherTestCase):
 
     def setUp(self):
         super(LauncherIconsTests, self).setUp()
+
         self.set_unity_option('launcher_hide_mode', 0)
 
     def ensure_expo_launcher_icon(self):
@@ -62,14 +63,14 @@ class LauncherIconsTests(LauncherTestCase):
     def test_bfb_tooltip_disappear_when_dash_is_opened(self):
         """Tests that the bfb tooltip disappear when the dash is opened."""
         bfb = self.unity.launcher.model.get_bfb_icon()
-        self.mouse.move(bfb.center_x, bfb.center_y)
+        self.mouse.move(bfb.center.x, bfb.center.y)
 
         self.assertThat(lambda: bfb.get_tooltip(), Eventually(NotEquals(None)))
         self.assertThat(bfb.get_tooltip().active, Eventually(Equals(True)))
         self.unity.dash.ensure_visible()
         self.addCleanup(self.unity.dash.ensure_hidden)
 
-        self.assertThat(bfb.get_tooltip().active, Eventually(Equals(False)))
+        self.assertThat(bfb.get_tooltip(), Equals(None))
 
     def test_bfb_tooltip_is_disabled_when_dash_is_open(self):
         """Tests the that bfb tooltip is disabled when the dash is open."""
@@ -77,7 +78,7 @@ class LauncherIconsTests(LauncherTestCase):
         self.addCleanup(self.unity.dash.ensure_hidden)
 
         bfb = self.unity.launcher.model.get_bfb_icon()
-        self.mouse.move(bfb.center_x, bfb.center_y)
+        self.mouse.move(bfb.center.x, bfb.center.y)
 
         # Tooltips are lazy-created  in Unity, so if the BFB tooltip has never
         # been shown before, get_tooltip will return None. If that happens, then
@@ -240,21 +241,6 @@ class LauncherIconsTests(LauncherTestCase):
         self.assertThat(self.unity.dash.visible, Eventually(Equals(True)))
         self.assertThat(self.unity.window_manager.scale_active, Eventually(Equals(False)))
 
-    def test_icon_shows_on_quick_application_reopen(self):
-        """Icons must stay on launcher when an application is quickly closed/reopened."""
-        calc = self.process_manager.start_app("Calculator")
-        desktop_file = calc.desktop_file
-        calc_icon = self.unity.launcher.model.get_icon(desktop_id=desktop_file)
-        self.assertThat(calc_icon.visible, Eventually(Equals(True)))
-
-        self.process_manager.close_all_app("Calculator")
-        calc = self.process_manager.start_app("Calculator")
-        sleep(2)
-
-        calc_icon = self.unity.launcher.model.get_icon(desktop_id=desktop_file)
-        self.assertThat(calc_icon, NotEquals(None))
-        self.assertThat(calc_icon.visible, Eventually(Equals(True)))
-
     def test_right_click_on_icon_ends_expo(self):
         """Right click on a launcher icon in expo mode must end the expo
         and show the quicklist.
@@ -268,7 +254,7 @@ class LauncherIconsTests(LauncherTestCase):
         self.addCleanup(self.keybinding, "expo/cancel")
 
         bfb = self.unity.launcher.model.get_bfb_icon()
-        self.mouse.move(bfb.center_x, bfb.center_y)
+        self.mouse.move(bfb.center.x, bfb.center.y)
         self.mouse.click(button=3)
 
         self.assertThat(self.launcher_instance.quicklist_open, Eventually(Equals(True)))
@@ -367,6 +353,7 @@ class LauncherDragIconsBehavior(LauncherTestCase):
 
     def setUp(self):
         super(LauncherDragIconsBehavior, self).setUp()
+
         self.set_unity_option('launcher_hide_mode', 0)
 
     def ensure_calc_icon_not_in_launcher(self):
