@@ -21,17 +21,17 @@
 
 #include <config.h>
 #include <gmock/gmock.h>
+#include <zeitgeist.h>
 
 #include <UnityCore/GLibWrapper.h>
 
 #include "ApplicationLauncherIcon.h"
 #include "FavoriteStore.h"
-#include "StandaloneWindowManager.h"
 #include "UBusWrapper.h"
 #include "UBusMessages.h"
-#include "ZeitgeistUtils.h"
 #include "mock-application.h"
 #include "test_utils.h"
+#include "test_standalone_wm.h"
 
 using namespace testing;
 using namespace testmocks;
@@ -85,10 +85,10 @@ MATCHER_P(AreArgsEqual, a, "")
 
 struct TestApplicationLauncherIcon : testmocks::TestUnityAppBase
 {
+  virtual ~TestApplicationLauncherIcon() {}
+
   virtual void SetUp() override
   {
-    WM = dynamic_cast<StandaloneWindowManager*>(&WindowManager::Default());
-
     usc_app = std::make_shared<MockApplication::Nice>(USC_DESKTOP, "softwarecenter");
     usc_icon = new NiceMock<MockApplicationLauncherIcon>(usc_app);
     ASSERT_EQ(usc_icon->DesktopFile(), USC_DESKTOP);
@@ -102,12 +102,6 @@ struct TestApplicationLauncherIcon : testmocks::TestUnityAppBase
     ASSERT_TRUE(mock_icon->DesktopFile().empty());
   }
 
-  virtual void TearDown() override
-  {
-    for (auto const& win : WM->GetStandaloneWindows())
-      WM->Close(win->Xid());
-  }
-
   void AddMockWindow(Window xid, int monitor, int desktop)
   {
     auto app_window = std::make_shared<MockApplicationWindow::Nice>(xid);
@@ -166,7 +160,7 @@ struct TestApplicationLauncherIcon : testmocks::TestUnityAppBase
     EXPECT_TRUE(app->icon.changed.empty());
   }
 
-  StandaloneWindowManager* WM;
+  testwrapper::StandaloneWM WM;
   MockApplication::Ptr usc_app;
   MockApplication::Ptr empty_app;
   MockApplication::Ptr mock_app;
@@ -674,8 +668,14 @@ TEST_F(TestApplicationLauncherIcon, PerformScrollSwitchDirection)
   mock_icon->PerformScroll(AbstractLauncherIcon::ScrollDirection::DOWN, 200);
   EXPECT_THAT(WM->GetWindowsInStackingOrder(), testing::ElementsAre(7, 6, 5, 4, 3, 1, 2));
 
-  mock_icon->PerformScroll(AbstractLauncherIcon::ScrollDirection::UP, 400);
-  EXPECT_THAT(WM->GetWindowsInStackingOrder(), testing::ElementsAre(7, 6, 4, 3, 1, 2, 5));
+  mock_icon->PerformScroll(AbstractLauncherIcon::ScrollDirection::DOWN, 400);
+  EXPECT_THAT(WM->GetWindowsInStackingOrder(), testing::ElementsAre(7, 6, 5, 4, 2, 1, 3));
+
+  mock_icon->PerformScroll(AbstractLauncherIcon::ScrollDirection::UP, 600);
+  EXPECT_THAT(WM->GetWindowsInStackingOrder(), testing::ElementsAre(7, 6, 5, 4, 3, 1, 2));
+
+  mock_icon->PerformScroll(AbstractLauncherIcon::ScrollDirection::UP, 800);
+  EXPECT_THAT(WM->GetWindowsInStackingOrder(), testing::ElementsAre(7, 6, 5, 4, 3, 2, 1));
 }
 
 TEST_F(TestApplicationLauncherIcon, PerformScrollNoWindows)
@@ -728,6 +728,23 @@ TEST_F(TestApplicationLauncherIcon, PerformScrollInitiallyUnfocusedWindow)
   ASSERT_EQ(WM->GetActiveWindow(), 1);
 }
 
+TEST_F(TestApplicationLauncherIcon, PerformScrollSingleUnfocusedWindow)
+{
+  AddMockWindow(1, 0, 0);
+
+  auto external_window = std::make_shared<unity::StandaloneWindow>(2);
+  WM->AddStandaloneWindow(external_window);
+  mock_icon->SetQuirk(AbstractLauncherIcon::Quirk::ACTIVE, false);
+
+  EXPECT_THAT(WM->GetWindowsInStackingOrder(), testing::ElementsAre(1, 2));
+  ASSERT_EQ(WM->GetActiveWindow(), 2);
+
+  mock_icon->PerformScroll(AbstractLauncherIcon::ScrollDirection::DOWN, 200);
+
+  EXPECT_THAT(WM->GetWindowsInStackingOrder(), testing::ElementsAre(2, 1));
+  ASSERT_EQ(WM->GetActiveWindow(), 1);
+}
+
 TEST_F(TestApplicationLauncherIcon, ActiveQuirkWMCrossCheck)
 {
   auto win = std::make_shared<MockApplicationWindow::Nice>(g_random_int());
@@ -990,7 +1007,11 @@ TEST_F(TestApplicationLauncherIcon, QuicklistMenuItemRemoteIgnoresInvisible)
   EXPECT_FALSE(HasMenuItemWithLabel(mock_icon, "InvisibleLabel"));
 }
 
-struct QuitLabel : TestApplicationLauncherIcon, testing::WithParamInterface<std::string> {};
+struct QuitLabel : TestApplicationLauncherIcon, testing::WithParamInterface<std::string>
+{
+  virtual ~QuitLabel() {}
+};
+
 INSTANTIATE_TEST_CASE_P(TestApplicationLauncherIcon, QuitLabel, testing::Values("Quit", "Exit", "Close"));
 
 TEST_P(/*TestApplicationLauncherIcon*/QuitLabel, QuicklistMenuItemRemoteOverridesQuitByLabelNotRunning)
