@@ -7,16 +7,13 @@
 
 #include "logger_helper.h"
 #include "test_utils.h"
-
-
-const gchar* LOCAL_DATA_DIR = BUILDDIR"/tests/data:/usr/share";
-const gchar* SCHEMA_DIRECTORY = BUILDDIR"/settings";
+#include "UnitySettings.h"
 
 int main(int argc, char** argv)
 {
   ::testing::InitGoogleTest(&argc, argv);
 
-  // init XDG_DATA_DIRS before GTK to point to the local test-dir as 
+  // init XDG_DATA_DIRS before GTK to point to the local test-dir as
   // the environment is only read once by glib and then cached
   const std::string LOCAL_DATA_DIR = BUILDDIR"/tests/data:/usr/share";
   g_setenv("XDG_DATA_DIRS", LOCAL_DATA_DIR.c_str(), TRUE);
@@ -25,6 +22,7 @@ int main(int argc, char** argv)
 
   gtk_init(&argc, &argv);
   setlocale(LC_ALL, "C");
+  unity::Settings settings;
 
   nux::NuxInitialize(0);
   std::unique_ptr<nux::WindowThread> win_thread(nux::CreateNuxWindow("Tests",
