@@ -20,7 +20,6 @@
 #include <gtest/gtest.h>
 
 #include "SingleMonitorLauncherIcon.h"
-#include "MultiMonitor.h"
 
 using namespace unity;
 using namespace launcher;
@@ -28,7 +27,11 @@ using namespace launcher;
 namespace
 {
 
-TEST(TestSingleMonitorLauncherIcon, Construction)
+struct TestSingleMonitorLauncherIconMock : testing::Test
+{
+};
+
+TEST_F(TestSingleMonitorLauncherIconMock, Construction)
 {
   SingleMonitorLauncherIcon icon(AbstractLauncherIcon::IconType::NONE, 1);
 
@@ -37,7 +40,7 @@ TEST(TestSingleMonitorLauncherIcon, Construction)
   EXPECT_FALSE(icon.IsVisibleOnMonitor(0));
 }
 
-TEST(TestSingleMonitorLauncherIcon, MonitorVisibility)
+TEST_F(TestSingleMonitorLauncherIconMock, MonitorVisibility)
 {
   SingleMonitorLauncherIcon icon(AbstractLauncherIcon::IconType::NONE, 2);
 
