@@ -17,71 +17,153 @@
  * Authored by: Jay Taoko <jay.taoko@canonical.com>
  */
 
+#include <NuxCore/Logger.h>
 #include "BackgroundEffectHelper.h"
-#include "TimeUtil.h"
 
-#include <time.h>
-#ifdef TRUE
-#undef TRUE
-#endif
-
-#ifdef FALSE
-#undef FALSE
-#endif
-
-#include <X11/Xregion.h>
-#include <boost/utility.hpp>
+#include "TextureCache.h"
 #include "UnitySettings.h"
 
+namespace
+{
+DECLARE_LOGGER(logger, "unity.background_effect_helper");
+
+const int BLUR_RADIUS = 3;
+const float sigma_high = 5.0f;
+const float sigma_med = 3.0f;
+const float sigma_low = 1.0f;
+}
 
 using namespace unity;
 
-std::list<BackgroundEffectHelper*> BackgroundEffectHelper::registered_list_;
-
+nux::Property<BlurType> BackgroundEffectHelper::blur_type(BLUR_ACTIVE);
 nux::Geometry BackgroundEffectHelper::monitor_rect_;
+std::list<BackgroundEffectHelper*> BackgroundEffectHelper::registered_list_;
+std::vector<nux::Geometry> BackgroundEffectHelper::blur_geometries_;
+sigc::signal<void, nux::Geometry const&> BackgroundEffectHelper::blur_region_needs_update_;
 
-nux::Property<BlurType> BackgroundEffectHelper::blur_type (BLUR_ACTIVE);
-nux::Property<float> BackgroundEffectHelper::sigma_high (5.0f);
-nux::Property<float> BackgroundEffectHelper::sigma_med (3.0f);
-nux::Property<float> BackgroundEffectHelper::sigma_low (1.0f);
-nux::Property<bool> BackgroundEffectHelper::updates_enabled (true);
-nux::Property<bool> BackgroundEffectHelper::detecting_occlusions (false);
+BackgroundEffectHelper::BackgroundEffectHelper(nux::View* view)
+  : owner(view)
+  , enabled(false)
+  , cache_dirty(true)
+{
+  enabled.changed.connect(sigc::mem_fun(this, &BackgroundEffectHelper::OnEnabledChanged));
+  owner.changed.connect(sigc::mem_fun(this, &BackgroundEffectHelper::OnOwnerChanged));
+  noise_texture_ = TextureCache::GetDefault().FindTexture("dash_noise.png");
+
+  if (Settings::Instance().GetLowGfxMode())
+    blur_type = BLUR_NONE;
+}
 
 BackgroundEffectHelper::BackgroundEffectHelper()
-{
-  enabled = false;
-  cache_dirty = true;
-  enabled.changed.connect (sigc::mem_fun(this, &BackgroundEffectHelper::OnEnabledChanged));
-  noise_texture_ = nux::CreateTextureFromFile(PKGDATADIR"/dash_noise.png");
-  
-  if (Settings::Instance().GetLowGfxMode())
-  {
-    blur_type(BLUR_NONE);
-  }
-
-  Register(this);
-}
+  : BackgroundEffectHelper(nullptr)
+{}
 
 BackgroundEffectHelper::~BackgroundEffectHelper()
 {
-  noise_texture_->UnReference();
   Unregister(this);
 }
 
-void BackgroundEffectHelper::OnEnabledChanged(bool value)
+void BackgroundEffectHelper::OnEnabledChanged(bool enabled)
 {
-  if (value)
-    DirtyCache();
+  if (enabled)
+  {
+    Register(this);
+    SetupOwner(owner());
+  }
+  else
+  {
+    connections_.Clear();
+    Unregister(this);
+  }
 }
 
-void BackgroundEffectHelper::ProcessDamage(nux::Geometry geo)
+void BackgroundEffectHelper::OnOwnerChanged(nux::View* view)
 {
-  for (BackgroundEffectHelper * bg_effect_helper : registered_list_)
+  geo_getter_func_ = nullptr;
+
+  if (!view)
   {
-    if (bg_effect_helper->cache_dirty || !bg_effect_helper->owner || !bg_effect_helper->enabled)
+    connections_.Clear();
+    Unregister(this);
+  }
+}
+
+void BackgroundEffectHelper::SetupOwner(nux::View* view)
+{
+  if (!view)
+    return;
+
+  auto cb = [this] (nux::Area*, nux::Geometry&) { UpdateOwnerGeometry(); };
+  connections_.Add(view->geometry_changed.connect(cb));
+
+  auto* parent = view->GetTopLevelViewWindow();
+  if (!parent)
+  {
+    LOG_ERROR(logger) << "The parent window for the owner must be set!";
+  }
+  else
+  {
+    connections_.Add(parent->geometry_changed.connect(cb));
+  }
+
+  if (!UpdateOwnerGeometry())
+  {
+    DirtyCache();
+    UpdateBlurGeometries();
+  }
+}
+
+void BackgroundEffectHelper::SetGeometryGetter(GeometryGetterFunc const& func)
+{
+  geo_getter_func_ = func;
+}
+
+bool BackgroundEffectHelper::UpdateOwnerGeometry()
+{
+  auto const& geo = geo_getter_func_ ? geo_getter_func_() : owner()->GetAbsoluteGeometry();
+
+  if (requested_blur_geometry_ != geo)
+  {
+    // For some reason a view might have this size at show time, let's just ignore them.
+    if (geo.width != 1 && geo.height != 1)
+    {
+      requested_blur_geometry_ = geo;
+
+      DirtyCache();
+      UpdateBlurGeometries();
+
+      return true;
+    }
+  }
+
+  return false;
+}
+
+void BackgroundEffectHelper::UpdateBlurGeometries()
+{
+  int radius = GetBlurRadius();
+  blur_geometries_.clear();
+  blur_geometries_.reserve(registered_list_.size());
+
+  for (BackgroundEffectHelper* bg_effect_helper : registered_list_)
+  {
+    /* Use the last requested region. The real region is clipped to the
+     * monitor geometry, but that is done at paint time */
+    auto const& blur_geo = bg_effect_helper->requested_blur_geometry_;
+
+    if (!blur_geo.IsNull())
+      blur_geometries_.push_back(blur_geo.GetExpand(radius, radius));
+  }
+}
+
+void BackgroundEffectHelper::ProcessDamage(nux::Geometry const& geo)
+{
+  for (BackgroundEffectHelper* bg_effect_helper : registered_list_)
+  {
+    if (bg_effect_helper->cache_dirty)
       continue;
 
-    if (!geo.Intersect (bg_effect_helper->blur_geometry_).IsNull())
+    if (geo.IsIntersecting(bg_effect_helper->requested_blur_geometry_))
     {
       bg_effect_helper->DirtyCache();
     }
@@ -90,21 +172,32 @@ void BackgroundEffectHelper::ProcessDamage(nux::Geometry geo)
 
 bool BackgroundEffectHelper::HasEnabledHelpers()
 {
-  for (BackgroundEffectHelper * bg_effect_helper : registered_list_)
-  {
-    if (bg_effect_helper->enabled)
-    {
-      return true;
-    }
-  }
+  return !registered_list_.empty();
+}
 
-  return false;
+float BackgroundEffectHelper::GetBlurSigma()
+{
+  nux::GpuDevice* gpu_device = nux::GetGraphicsDisplay()->GetGpuDevice();
+  int opengl_version = gpu_device->GetOpenGLMajorVersion();
+  return (opengl_version >= 3) ? sigma_high : sigma_med;
+}
+
+int BackgroundEffectHelper::GetBlurRadius()
+{
+  return (blur_type != BLUR_NONE) ? GetBlurSigma() * BLUR_RADIUS : 0;
+}
+
+std::vector<nux::Geometry> const& BackgroundEffectHelper::GetBlurGeometries()
+{
+  /* TODO: to reduce useless glCopyTexSubImage2D during paint we should return
+   * here only damaged geometries */
+  return blur_geometries_;
 }
 
 bool BackgroundEffectHelper::HasDirtyHelpers()
 {
-  for (BackgroundEffectHelper * bg_effect_helper : registered_list_)
-    if (bg_effect_helper->enabled && bg_effect_helper->cache_dirty)
+  for (BackgroundEffectHelper* bg_effect_helper : registered_list_)
+    if (bg_effect_helper->cache_dirty)
       return true;
 
   return false;
@@ -112,44 +205,55 @@ bool BackgroundEffectHelper::HasDirtyHelpers()
 
 void BackgroundEffectHelper::Register(BackgroundEffectHelper* self)
 {
+  auto* view = self->owner();
+
+  if (!view)
+  {
+    LOG_ERROR(logger) << "Registering an invalid helper, must set an owner!";
+    return;
+  }
+
   registered_list_.push_back(self);
 }
 
 void BackgroundEffectHelper::Unregister(BackgroundEffectHelper* self)
 {
   registered_list_.remove(self);
+  UpdateBlurGeometries();
 }
 
-void BackgroundEffectHelper::DirtyCache ()
+void BackgroundEffectHelper::DirtyCache()
 {
-  if (cache_dirty)
+  if ((cache_dirty && blur_geometry_ == requested_blur_geometry_) || !owner())
     return;
 
   cache_dirty = true;
-  if (owner)
-    owner()->QueueDraw();
+  owner()->QueueDraw();
+
+  int radius = GetBlurRadius();
+  blur_region_needs_update_.emit(requested_blur_geometry_.GetExpand(radius, radius));
 }
 
-nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetBlurRegion(nux::Geometry geo, bool force_update)
+nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetBlurRegion(bool force_update)
 {
   bool should_update = force_update || cache_dirty;
 
   /* Static blur: only update when the size changed */
   if ((blur_type != BLUR_ACTIVE || !should_update)
       && blur_texture_.IsValid()
-      && (geo == blur_geometry_))
+      && (requested_blur_geometry_ == blur_geometry_))
   {
     return blur_texture_;
   }
 
   nux::GraphicsEngine* graphics_engine = nux::GetGraphicsDisplay()->GetGraphicsEngine();
-  
+
   int monitor_width = BackgroundEffectHelper::monitor_rect_.width;
   int monitor_height = BackgroundEffectHelper::monitor_rect_.height;
 
-  nux::Geometry temp = geo;
+  nux::Geometry temp = requested_blur_geometry_;
   temp.OffsetPosition(-monitor_rect_.x, -monitor_rect_.y);
-  blur_geometry_ =  nux::Geometry(0, 0, monitor_width, monitor_height).Intersect(temp);
+  blur_geometry_ = nux::Geometry(0, 0, monitor_width, monitor_height).Intersect(temp);
 
   nux::GpuDevice* gpu_device = nux::GetGraphicsDisplay()->GetGpuDevice();
   if (blur_geometry_.IsNull() || blur_type == BLUR_NONE || !gpu_device->backup_texture0_.IsValid())
@@ -157,15 +261,13 @@ nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetBlurRegion(nu
     return nux::ObjectPtr<nux::IOpenGLBaseTexture>();
   }
 
-  int opengl_version = gpu_device->GetOpenGLMajorVersion();
-  int sigma = opengl_version >= 3 ? sigma_high : sigma_med;
-  int radius = 3 * sigma;
+  const int radius = GetBlurRadius();
 
   // Define a larger region of that account for the blur radius
   nux::Geometry larger_blur_geometry;
   larger_blur_geometry.x = std::max(blur_geometry_.x - radius, 0);
   larger_blur_geometry.y = std::max(blur_geometry_.y - radius, 0);
-  
+
   int xx = std::min(blur_geometry_.x + blur_geometry_.width + radius, monitor_width);
 
   larger_blur_geometry.width = xx - larger_blur_geometry.x;
@@ -188,7 +290,7 @@ nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetBlurRegion(nu
   graphics_engine->SetViewport(0, 0, larger_blur_geometry.width, larger_blur_geometry.height);
   graphics_engine->SetScissor(0, 0, larger_blur_geometry.width, larger_blur_geometry.height);
   // Disable nux scissoring
-  graphics_engine->GetRenderStates ().EnableScissor (false);
+  graphics_engine->GetRenderStates().EnableScissor(false);
 
   // The background texture is the same size as the monitor where we are rendering.
   nux::TexCoordXForm texxform__bg;
@@ -200,22 +302,21 @@ nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetBlurRegion(nu
   bool support_frag = gpu_device->GetGpuInfo().Support_ARB_Fragment_Shader();
   bool support_vert = gpu_device->GetGpuInfo().Support_ARB_Vertex_Shader();
 
-  if (support_vert && support_frag && opengl_version >= 2)
+  if (support_vert && support_frag && gpu_device->GetOpenGLMajorVersion() >= 2)
   {
     float noise_factor = 1.1f;
-    float gaussian_sigma = opengl_version >= 3 ? sigma_high : sigma_med;
-    int blur_passes = 1;
+    float gaussian_sigma = GetBlurSigma();
 
     nux::ObjectPtr<nux::IOpenGLBaseTexture> device_texture = gpu_device->backup_texture0_;
-    nux::ObjectPtr<nux::CachedBaseTexture> noise_device_texture = graphics_engine->CacheResource(noise_texture_);
+    nux::ObjectPtr<nux::CachedBaseTexture> noise_device_texture = graphics_engine->CacheResource(noise_texture_.GetPointer());
 
     unsigned int buffer_width = larger_blur_geometry.width;
     unsigned int buffer_height = larger_blur_geometry.height;
 
     blur_fx_struct_.src_texture = device_texture;
-    graphics_engine->QRP_GLSL_GetHQBlurFx(0, 0, buffer_width, buffer_height,
-                                                  &blur_fx_struct_, texxform__bg, nux::color::White,
-                                                 gaussian_sigma, blur_passes);
+    graphics_engine->QRP_GLSL_GetLSBlurFx(0, 0, buffer_width, buffer_height,
+                                          &blur_fx_struct_, texxform__bg, nux::color::White,
+                                          gaussian_sigma);
 
     nux::TexCoordXForm texxform;
     nux::TexCoordXForm noise_texxform;
@@ -240,7 +341,7 @@ nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetBlurRegion(nu
       0, 0, blur_geometry_.width, blur_geometry_.height,
       noise_device_texture->m_Texture, noise_texxform, noise_color,
       &noise_fx_struct_, texxform, nux::color::White);
-    
+
     blur_texture_ = noise_fx_struct_.dst_texture;
   }
   else
@@ -251,7 +352,7 @@ nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetBlurRegion(nu
     int blur_passes = 1;
 
     nux::ObjectPtr<nux::IOpenGLBaseTexture> device_texture = gpu_device->backup_texture0_;
-    nux::ObjectPtr<nux::CachedBaseTexture> noise_device_texture = graphics_engine->CacheResource(noise_texture_);
+    nux::ObjectPtr<nux::CachedBaseTexture> noise_device_texture = graphics_engine->CacheResource(noise_texture_.GetPointer());
 
     unsigned int offset = 0;
     int quad_width = larger_blur_geometry.width;
@@ -328,28 +429,30 @@ nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetBlurRegion(nu
   return blur_texture_;
 }
 
-nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetRegion(nux::Geometry geo, bool force_update)
+nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetRegion(bool force_update)
 {
   bool should_update = force_update || cache_dirty;
 
   /* Static blur: only update when the size changed */
   if ((!should_update)
       && blur_texture_.IsValid()
-      && (geo == blur_geometry_))
+      && (requested_blur_geometry_ == blur_geometry_))
   {
     return blur_texture_;
   }
 
   nux::GraphicsEngine* graphics_engine = nux::GetGraphicsDisplay()->GetGraphicsEngine();
-  
+
   int monitor_width = BackgroundEffectHelper::monitor_rect_.width;
   int monitor_height = BackgroundEffectHelper::monitor_rect_.height;
 
-  nux::Geometry temp = geo;
+  nux::Geometry temp = requested_blur_geometry_;
   temp.OffsetPosition(-monitor_rect_.x, -monitor_rect_.y);
+
   blur_geometry_ =  nux::Geometry(0, 0, monitor_width, monitor_height).Intersect(temp);
 
   nux::GpuDevice* gpu_device = nux::GetGraphicsDisplay()->GetGpuDevice();
+
   if (blur_geometry_.IsNull() || !gpu_device->backup_texture0_.IsValid())
   {
     return nux::ObjectPtr<nux::IOpenGLBaseTexture>();
@@ -366,18 +469,16 @@ nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetRegion(nux::G
   graphics_engine->SetViewport(0, 0, blur_geometry_.width, blur_geometry_.height);
   graphics_engine->SetScissor(0, 0, blur_geometry_.width, blur_geometry_.height);
   // Disable nux scissoring
-  graphics_engine->GetRenderStates ().EnableScissor (false);
+  graphics_engine->GetRenderStates().EnableScissor(false);
 
   // The background texture is the same size as the monitor where we are rendering.
   nux::TexCoordXForm texxform__bg;
-  texxform__bg.flip_v_coord = false;
   texxform__bg.SetTexCoordType(nux::TexCoordXForm::OFFSET_COORD);
   texxform__bg.uoffset = ((float) blur_geometry_.x) / monitor_width;
   texxform__bg.voffset = ((float) blur_geometry_.y) / monitor_height;
 
   {
     nux::ObjectPtr<nux::IOpenGLBaseTexture> device_texture = gpu_device->backup_texture0_;
-    nux::ObjectPtr<nux::CachedBaseTexture> noise_device_texture = graphics_engine->CacheResource(noise_texture_);
 
     unsigned int offset = 0;
     int quad_width = blur_geometry_.width;
@@ -399,7 +500,7 @@ nux::ObjectPtr<nux::IOpenGLBaseTexture> BackgroundEffectHelper::GetRegion(nux::G
   {
     current_fbo->Activate(true);
     graphics_engine->Push2DWindow(current_fbo->GetWidth(), current_fbo->GetHeight());
-    graphics_engine->GetRenderStates ().EnableScissor (true);
+    graphics_engine->GetRenderStates().EnableScissor(true);
   }
   else
   {
