@@ -1,6 +1,6 @@
 // -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright (C) 2010 Canonical Ltd
+ * Copyright (C) 2010-2013 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -15,21 +15,16 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Authored by: Alex Launi <alex.launi@canonical.com>
+ *              Marco Trevisan <marco.trevisan@canonical.com>
  */
 
-#include <gio/gio.h>
+#ifndef UNITY_DEBUG_DBUS_INTERFACE_H
+#define UNITY_DEBUG_DBUS_INTERFACE_H
 
-#ifndef _DEBUG_DBUS_INTERFACE_H
-#define _DEBUG_DBUS_INTERFACE_H
-
-#include <UnityCore/GLibDBusServer.h>
-
-class CompScreen;
+#include <memory>
 
 namespace unity
 {
-extern const std::string DBUS_BUS_NAME;
-
 namespace debug
 {
 class Introspectable;
@@ -37,14 +32,12 @@ class Introspectable;
 class DebugDBusInterface
 {
 public:
-  DebugDBusInterface(Introspectable* introspectable);
+  DebugDBusInterface(Introspectable* root);
   ~DebugDBusInterface();
 
 private:
-  static GVariant* HandleDBusMethodCall(std::string const&, GVariant*);
-  static GVariant* BuildFakeReturn();
-
-  glib::DBusServer server_;
+  struct Impl;
+  std::unique_ptr<Impl> impl_;
 };
 }
 }
