@@ -18,12 +18,16 @@
  */
 
 #include <glib.h>
-#include <sstream>
-#include "PluginAdapter.h"
+#include <gdk/gdkx.h>
 #include "UScreen.h"
+#include "UnitySettings.h"
+#include "DecorationStyle.h"
+#include "PluginAdapter.h"
+#include "CompizUtils.h"
+#include "MultiMonitor.h"
 
+#include <NuxGraphics/XInputWindow.h>
 #include <NuxCore/Logger.h>
-#include <UnityCore/Variant.h>
 
 namespace unity
 {
@@ -32,18 +36,12 @@ namespace
 {
 const int THRESHOLD_HEIGHT = 600;
 const int THRESHOLD_WIDTH = 1024;
-const char* _UNITY_FRAME_EXTENTS = "_UNITY_FRAME_EXTENTS";
 
 std::shared_ptr<PluginAdapter> global_instance;
 }
 
 #define MAXIMIZABLE (CompWindowActionMaximizeHorzMask & CompWindowActionMaximizeVertMask & CompWindowActionResizeMask)
 
-#define MWM_HINTS_FUNCTIONS     (1L << 0)
-#define MWM_HINTS_DECORATIONS   (1L << 1)
-#define MWM_HINTS_UNDECORATED_UNITY 0x80
-
-
 WindowManagerPtr create_window_manager()
 {
   return global_instance;
@@ -62,26 +60,23 @@ void PluginAdapter::Initialize(CompScreen* screen)
   global_instance.reset(new PluginAdapter(screen));
 }
 
-PluginAdapter::PluginAdapter(CompScreen* screen) :
-  m_Screen(screen),
-  _in_show_desktop (false),
-  _last_focused_window(nullptr)
-{
-  _spread_state = false;
-  _spread_windows_state = false;
-  _expo_state = false;
-  _vp_switch_started = false;
-
-  _grab_show_action = 0;
-  _grab_hide_action = 0;
-  _grab_toggle_action = 0;
-  _coverage_area_before_automaximize = 0.75;
-  bias_active_to_viewport = false;
-}
+PluginAdapter::PluginAdapter(CompScreen* screen)
+  : bias_active_to_viewport(false)
+  , m_Screen(screen)
+  , _coverage_area_before_automaximize(0.75)
+  , _spread_state(false)
+  , _spread_windows_state(false)
+  , _expo_state(false)
+  , _vp_switch_started(false)
+  , _in_show_desktop(false)
+  , _grab_show_action(nullptr)
+  , _grab_hide_action(nullptr)
+  , _grab_toggle_action(nullptr)
+  , _last_focused_window(nullptr)
+{}
 
 PluginAdapter::~PluginAdapter()
-{
-}
+{}
 
 /* A No-op for now, but could be useful later */
 void PluginAdapter::OnScreenGrabbed()
@@ -143,33 +138,6 @@ void PluginAdapter::NotifyStateChange(CompWindow* window, unsigned int state, un
   }
 }
 
-void PluginAdapter::NotifyNewDecorationState(Window xid)
-{
-  auto deco_state_it = _window_decoration_state.find(xid);
-  bool wasTracked = (deco_state_it != _window_decoration_state.end());
-  bool wasDecorated = false;
-
-  if (wasTracked)
-  {
-    wasDecorated = deco_state_it->second;
-    _window_decoration_state.erase(deco_state_it);
-  }
-
-  bool decorated = HasWindowDecorations(xid);
-
-  if (decorated == wasDecorated)
-    return;
-
-  if (decorated && (!wasDecorated || !wasTracked))
-  {
-    window_decorated.emit(xid);
-  }
-  else if (wasDecorated || !wasTracked)
-  {
-    window_undecorated.emit(xid);
-  }
-}
-
 void PluginAdapter::Notify(CompWindow* window, CompWindowNotify notify)
 {
   switch (notify)
@@ -323,7 +291,7 @@ void MultiActionList::TerminateAll(CompOption::Vector const& extra_args) const
 
   if (primary_action_)
   {
-    primary_action_->terminate()(primary_action_, 0, argument);
+    primary_action_->terminate()(primary_action_, CompAction::StateCancel, argument);
     return;
   }
 
@@ -372,13 +340,8 @@ std::string PluginAdapter::MatchStringForXids(std::vector<Window> const& windows
 {
   std::ostringstream sout;
 
-  sout << "any & (";
-
   for (auto const& window : windows)
-  {
-    sout << "| xid=" << window << " ";
-  }
-  sout << ")";
+    sout << "xid=" << window << " | ";
 
   return sout.str();
 }
@@ -399,17 +362,17 @@ void PluginAdapter::TerminateScale()
 
 bool PluginAdapter::IsScaleActive() const
 {
-  return m_Screen->grabExist("scale");
+  return _spread_state;
 }
 
 bool PluginAdapter::IsScaleActiveForGroup() const
 {
-  return _spread_windows_state && m_Screen->grabExist("scale");
+  return _spread_windows_state && _spread_state;
 }
 
 bool PluginAdapter::IsExpoActive() const
 {
-  return m_Screen->grabExist("expo");
+  return _expo_state;
 }
 
 bool PluginAdapter::IsWallActive() const
@@ -450,6 +413,63 @@ std::vector<Window> PluginAdapter::GetWindowsInStackingOrder() const
   return ret;
 }
 
+int PluginAdapter::MonitorGeometryIn(nux::Geometry const& geo) const
+{
+  std::vector<nux::Geometry> const& monitors = unity::UScreen::GetDefault()->GetMonitors();
+  for (unsigned i = 0; i < monitors.size(); ++i)
+  {
+    nux::Geometry const& i_g = geo.Intersect(monitors[i]);
+
+    if (i_g.width > geo.width/2 && i_g.height > geo.height/2)
+      return i;
+  }
+
+  return 0;
+}
+
+bool PluginAdapter::IsTopWindowFullscreenOnMonitorWithMouse() const
+{
+  int monitor = unity::UScreen::GetDefault()->GetMonitorWithMouse();
+  Window top_win = GetTopMostWindowInMonitor(monitor);
+  CompWindow* window = m_Screen->findWindow(top_win);
+
+  if (window)
+    return (window->state() & CompWindowStateFullscreenMask);
+
+  return false;
+}
+
+Window PluginAdapter::GetTopMostWindowInMonitor(int monitor) const
+{
+  CompWindow* window = nullptr;
+  nux::Geometry const& m_geo = unity::UScreen::GetDefault()->GetMonitorGeometry(monitor);
+
+  CompPoint screen_vp = m_Screen->vp();
+
+  auto const& windows = m_Screen->windows();
+  for (auto it = windows.rbegin(); it != windows.rend(); ++it)
+  {
+    window = *it;
+    nux::Geometry const& win_geo = GetWindowGeometry(window->id());
+    nux::Geometry const& intersect_geo = win_geo.Intersect(m_geo);
+
+    if (intersect_geo.width > 0 &&
+        intersect_geo.height > 0 &&
+        window->defaultViewport() == screen_vp &&
+        window->isViewable() && window->isMapped() &&
+        !window->minimized() && !window->inShowDesktopMode() &&
+        !(window->state() & CompWindowStateAboveMask) &&
+        !(window->type() & CompWindowTypeSplashMask) &&
+        !(window->type() & CompWindowTypeDockMask) &&
+        !window->overrideRedirect())
+    {
+      return window->id();
+    }
+  }
+
+  return 0;
+}
+
 bool PluginAdapter::IsWindowMaximized(Window window_id) const
 {
   CompWindow* window = m_Screen->findWindow(window_id);
@@ -477,69 +497,23 @@ bool PluginAdapter::IsWindowHorizontallyMaximized(Window window_id) const
   return false;
 }
 
-unsigned long PluginAdapter::GetMwnDecorations(Window window_id) const
-{
-  Display* display = m_Screen->dpy();
-  MotifWmHints* hints = NULL;
-  Atom type = None;
-  gint format;
-  gulong nitems;
-  gulong bytes_after;
-  unsigned long decorations = 0;
-
-  if (XGetWindowProperty(display, window_id, Atoms::mwmHints, 0,
-                         sizeof(MotifWmHints) / sizeof(long), False,
-                         Atoms::mwmHints, &type, &format, &nitems, &bytes_after,
-                         (guchar**)&hints) != Success)
-  {
-    return decorations;
-  }
-
-  decorations |= (MwmDecorAll | MwmDecorTitle);
-
-  if (!hints)
-    return decorations;
-
-  /* Check for the presence of the high bit
-   * if present, it means that we undecorated
-   * this window, so don't mark it as undecorated */
-  if (type == Atoms::mwmHints && format != 0 && hints->flags & MWM_HINTS_DECORATIONS)
-  {
-    decorations = hints->decorations;
-  }
-
-  XFree(hints);
-  return decorations;
-}
-
 bool PluginAdapter::HasWindowDecorations(Window window_id) const
 {
-  auto deco_state_it = _window_decoration_state.find(window_id);
-
-  if (deco_state_it != _window_decoration_state.end())
-    return deco_state_it->second;
-
-  unsigned long decorations = GetMwnDecorations(window_id);
-
-  /* Must have both bits set */
-  bool decorated = (decorations & (MwmDecorAll | MwmDecorTitle)) ||
-                   (decorations & MWM_HINTS_UNDECORATED_UNITY);
-
-  _window_decoration_state[window_id] = decorated;
-
-  return decorated;
+  return compiz_utils::IsWindowFullyDecorable(m_Screen->findWindow(window_id));
 }
 
 bool PluginAdapter::IsWindowDecorated(Window window_id) const
 {
-  bool decorated = GetMwnDecorations(window_id) & (MwmDecorAll | MwmDecorTitle);
-
-  if (decorated && GetCardinalProperty(window_id, Atoms::frameExtents).empty())
+  if (CompWindow* win = m_Screen->findWindow(window_id))
   {
-    decorated = false;
+    if ((win->state() & MAXIMIZE_STATE) != MAXIMIZE_STATE &&
+        compiz_utils::IsWindowFullyDecorable(win))
+    {
+      return true;
+    }
   }
 
-  return decorated;
+  return false;
 }
 
 bool PluginAdapter::IsWindowOnCurrentDesktop(Window window_id) const
@@ -564,7 +538,6 @@ bool PluginAdapter::IsWindowObscured(Window window_id) const
       return true;
 
     CompPoint window_vp = window->defaultViewport();
-    nux::Geometry const& win_geo = GetWindowGeometry(window->id());
     // Check if any windows above this one are blocking it
     for (CompWindow* sibling = window->next; sibling != NULL; sibling = sibling->next)
     {
@@ -573,7 +546,7 @@ bool PluginAdapter::IsWindowObscured(Window window_id) const
           && sibling->isMapped()
           && sibling->isViewable()
           && (sibling->state() & MAXIMIZE_STATE) == MAXIMIZE_STATE
-          && !GetWindowGeometry(sibling->id()).Intersect(win_geo).IsNull())
+          && sibling->borderRect().intersects(window->borderRect()))
       {
         return true;
       }
@@ -600,6 +573,14 @@ bool PluginAdapter::IsWindowVisible(Window window_id) const
   return false;
 }
 
+bool PluginAdapter::IsWindowShaded(Window window_id) const
+{
+  if (CompWindow* window = m_Screen->findWindow(window_id))
+    return (window->state() & CompWindowStateShadedMask);
+
+  return false;
+}
+
 bool PluginAdapter::IsWindowOnTop(Window window_id) const
 {
   if (window_id == GetTopMostValidWindowInViewport())
@@ -710,60 +691,19 @@ bool PluginAdapter::IsWindowMaximizable(Window window_id) const
 void PluginAdapter::Maximize(Window window_id)
 {
   if (CompWindow* window = m_Screen->findWindow(window_id))
-    window->maximize(MAXIMIZE_STATE);
+    window->maximize(CompWindowStateMaximizedVertMask);
 }
 
-void PluginAdapter::VerticallyMaximizeWindowAt(CompWindow* window, nux::Geometry const& geo)
+void PluginAdapter::VerticallyMaximize(Window window_id)
 {
-  if (window && ((window->type() & CompWindowTypeNormalMask) ||
-      ((window->actions() & CompWindowActionMaximizeVertMask) &&
-        window->actions() & CompWindowActionResizeMask)))
-  {
-    /* First we unmaximize the Window */
-    if (window->state() & MAXIMIZE_STATE)
-      window->maximize(0);
-
-    /* Then we vertically maximize the it so it can be unminimized correctly */
-    if (!(window->state() & CompWindowStateMaximizedVertMask))
-      window->maximize(CompWindowStateMaximizedVertMask);
-
-    /* Then we resize and move it on the requested place */
-    MoveResizeWindow(window->id(), geo);
-  }
+  if (CompWindow* window = m_Screen->findWindow(window_id))
+    window->maximize(CompWindowStateMaximizedVertMask);
 }
 
-void PluginAdapter::LeftMaximize(Window window_id)
+void PluginAdapter::HorizontallyMaximize(Window window_id)
 {
-  CompWindow* window = m_Screen->findWindow(window_id);
-
-  if (!window)
-    return;
-
-  /* Let's compute the area where the window should stay */
-  CompRect workarea = m_Screen->getWorkareaForOutput(window->outputDevice());
-  nux::Geometry win_geo(workarea.x() + window->border().left,
-                        workarea.y() + window->border().top,
-                        workarea.width() / 2 - (window->border().left + window->border().right),
-                        workarea.height() - (window->border().top + window->border().bottom));
-
-  VerticallyMaximizeWindowAt(window, win_geo);
-}
-
-void PluginAdapter::RightMaximize(Window window_id)
-{
-  CompWindow* window = m_Screen->findWindow(window_id);
-
-  if (!window)
-    return;
-
-  /* Let's compute the area where the window should stay */
-  CompRect workarea = m_Screen->getWorkareaForOutput(window->outputDevice());
-  nux::Geometry win_geo(workarea.x() + workarea.width() / 2 + window->border().left,
-                        workarea.y() + window->border().top,
-                        workarea.width() / 2 - (window->border().left + window->border().right),
-                        workarea.height() - (window->border().top + window->border().bottom));
-
-  VerticallyMaximizeWindowAt(window, win_geo);
+  if (CompWindow* window = m_Screen->findWindow(window_id))
+    window->maximize(CompWindowStateMaximizedHorzMask);
 }
 
 void PluginAdapter::Restore(Window window_id)
@@ -779,13 +719,23 @@ void PluginAdapter::RestoreAt(Window window_id, int x, int y)
   if (window && (window->state() & MAXIMIZE_STATE))
   {
     nux::Geometry new_geo(GetWindowSavedGeometry(window_id));
-    new_geo.x = x;
-    new_geo.y = y;
     window->maximize(0);
+
+    auto const& border = window->border();
+    new_geo.x = x;
+    new_geo.y = y + border.top;
+    new_geo.width -= (border.left + border.right);
+    new_geo.height -= (border.top + border.bottom);
+
     MoveResizeWindow(window_id, new_geo);
   }
 }
 
+void PluginAdapter::ShowActionMenu(Time timestamp, Window window_id, unsigned button, nux::Point const& p)
+{
+  m_Screen->toolkitAction(Atoms::toolkitActionWindowMenu, timestamp, window_id, button, p.x, p.y);
+}
+
 void PluginAdapter::Minimize(Window window_id)
 {
   CompWindow* window = m_Screen->findWindow(window_id);
@@ -800,6 +750,26 @@ void PluginAdapter::UnMinimize(Window window_id)
     window->unminimize();
 }
 
+void PluginAdapter::Shade(Window window_id)
+{
+  CompWindow* window = m_Screen->findWindow(window_id);
+  if (window && (window->actions() & CompWindowActionShadeMask))
+  {
+    window->changeState(window->state() | CompWindowStateShadedMask);
+    window->updateAttributes(CompStackingUpdateModeNone);
+  }
+}
+
+void PluginAdapter::UnShade(Window window_id)
+{
+  CompWindow* window = m_Screen->findWindow(window_id);
+  if (window && (window->actions() & CompWindowActionShadeMask))
+  {
+    window->changeState(window->state() & ~CompWindowStateShadedMask);
+    window->updateAttributes(CompStackingUpdateModeNone);
+  }
+}
+
 void PluginAdapter::Close(Window window_id)
 {
   CompWindow* window = m_Screen->findWindow(window_id);
@@ -1086,49 +1056,44 @@ nux::Geometry PluginAdapter::GetScreenGeometry() const
   return nux::Geometry(0, 0, m_Screen->width(), m_Screen->height());
 }
 
+template<typename T>
+nux::Size get_edge_size(WindowManager::Edge edge, CompRect const& win_rect, T const& extents)
+{
+  switch (edge)
+  {
+    case WindowManager::Edge::LEFT:
+      return nux::Size(extents.left, win_rect.height());
+    case WindowManager::Edge::TOP:
+      return nux::Size(win_rect.width(), extents.top);
+    case WindowManager::Edge::RIGHT:
+      return nux::Size(extents.right, win_rect.height());
+    case WindowManager::Edge::BOTTOM:
+      return nux::Size(win_rect.width(), extents.bottom);
+  }
+
+  return nux::Size();
+}
+
 nux::Size PluginAdapter::GetWindowDecorationSize(Window window_id, WindowManager::Edge edge) const
 {
   if (CompWindow* window = m_Screen->findWindow(window_id))
   {
-    if (HasWindowDecorations(window_id))
+    if (compiz_utils::IsWindowFullyDecorable(window))
     {
       auto const& win_rect = window->borderRect();
 
-      if (IsWindowDecorated(window_id))
+      if ((window->state() & MAXIMIZE_STATE) == MAXIMIZE_STATE)
       {
-        auto const& extents = window->border();
-
-        switch (edge)
-        {
-          case Edge::LEFT:
-            return nux::Size(extents.left, win_rect.height());
-          case Edge::TOP:
-            return nux::Size(win_rect.width(), extents.top);
-          case Edge::RIGHT:
-            return nux::Size(extents.right, win_rect.height());
-          case Edge::BOTTOM:
-            return nux::Size(win_rect.width(), extents.bottom);
-        }
+        auto const& extents = decoration::Style::Get()->Border();
+        nux::Geometry win_geo(win_rect.x(), win_rect.y(), win_rect.width(), win_rect.height());
+        auto deco_size = get_edge_size(edge, win_rect, extents);
+        double scale = Settings::Instance().em(MonitorGeometryIn(win_geo))->DPIScale();
+        return nux::Size(deco_size.width * scale, deco_size.height * scale);
       }
       else
       {
-        Atom unity_extents = gdk_x11_get_xatom_by_name(_UNITY_FRAME_EXTENTS);
-        auto const& extents = GetCardinalProperty(window_id, unity_extents);
-
-        if (extents.size() == 4)
-        {
-          switch (edge)
-          {
-            case Edge::LEFT:
-              return nux::Size(extents[unsigned(Edge::LEFT)], win_rect.height());
-            case Edge::TOP:
-              return nux::Size(win_rect.width(), extents[unsigned(Edge::TOP)]);
-            case Edge::RIGHT:
-              return nux::Size(extents[unsigned(Edge::RIGHT)], win_rect.height());
-            case Edge::BOTTOM:
-              return nux::Size(win_rect.width(), extents[unsigned(Edge::BOTTOM)]);
-          }
-        }
+        auto const& extents = window->border();
+        return get_edge_size(edge, win_rect, extents);
       }
     }
   }
@@ -1235,12 +1200,10 @@ void PluginAdapter::SetViewportSize(int horizontal, int vertical)
     return;
   }
 
-  CompOption::Value hsize;
-  hsize.set<int>(horizontal);
+  CompOption::Value hsize(horizontal);
   m_Screen->setOptionForPlugin("core", "hsize", hsize);
 
   CompOption::Value vsize(vertical);
-  vsize.set<int>(vertical);
   m_Screen->setOptionForPlugin("core", "vsize", vsize);
 
   LOG_INFO(logger) << "Setting viewport size to " << hsize.i() << "x" << vsize.i();
@@ -1256,100 +1219,6 @@ int PluginAdapter::GetViewportVSize() const
   return m_Screen->vpSize().height();
 }
 
-void PluginAdapter::SetMwmWindowHints(Window xid, MotifWmHints* new_hints) const
-{
-  Display* display = m_Screen->dpy();
-  MotifWmHints* data = NULL;
-  MotifWmHints* hints = NULL;
-  Atom type = None;
-  gint format;
-  gulong nitems;
-  gulong bytes_after;
-
-  if (XGetWindowProperty(display,
-                         xid,
-                         Atoms::mwmHints, 0, sizeof(MotifWmHints) / sizeof(long),
-                         False, AnyPropertyType, &type, &format, &nitems,
-                         &bytes_after, (guchar**)&data) != Success)
-  {
-    return;
-  }
-
-  if (type != Atoms::mwmHints || !data)
-  {
-    hints = new_hints;
-  }
-  else
-  {
-    hints = data;
-
-    if (new_hints->flags & MWM_HINTS_FUNCTIONS)
-    {
-      hints->flags |= MWM_HINTS_FUNCTIONS;
-      hints->functions = new_hints->functions;
-    }
-    if (new_hints->flags & MWM_HINTS_DECORATIONS)
-    {
-      hints->flags |= MWM_HINTS_DECORATIONS;
-      hints->decorations = new_hints->decorations;
-    }
-  }
-
-  XChangeProperty(display, xid, Atoms::mwmHints, Atoms::mwmHints, 32, PropModeReplace,
-                  (guchar*)hints, sizeof(MotifWmHints) / sizeof(long));
-
-  if (data)
-    XFree(data);
-}
-
-void PluginAdapter::Decorate(Window window_id) const
-{
-  if (!HasWindowDecorations(window_id))
-    return;
-
-  MotifWmHints hints = { 0 };
-
-  hints.flags = MWM_HINTS_DECORATIONS;
-  hints.decorations = GDK_DECOR_ALL & ~(MWM_HINTS_UNDECORATED_UNITY);
-
-  SetMwmWindowHints(window_id, &hints);
-
-  // Removing the saved windows extents
-  Atom atom = gdk_x11_get_xatom_by_name(_UNITY_FRAME_EXTENTS);
-  XDeleteProperty(m_Screen->dpy(), window_id, atom);
-}
-
-void PluginAdapter::Undecorate(Window window_id) const
-{
-  if (!IsWindowDecorated(window_id))
-    return;
-
-  if (CompWindow* window = m_Screen->findWindow(window_id))
-  {
-    // Saving the previous window extents values
-    long extents[4];
-    auto const& border = window->border();
-    extents[unsigned(Edge::LEFT)] = border.left;
-    extents[unsigned(Edge::RIGHT)] = border.right;
-    extents[unsigned(Edge::TOP)] = border.top;
-    extents[unsigned(Edge::BOTTOM)] = border.bottom;
-
-    Atom atom = gdk_x11_get_xatom_by_name(_UNITY_FRAME_EXTENTS);
-    XChangeProperty(m_Screen->dpy(), window_id, atom, XA_CARDINAL, 32,
-                    PropModeReplace, (unsigned char*) extents, 4);
-  }
-
-  MotifWmHints hints = { 0 };
-
-  /* Set the high bit to indicate that we undecorated this
-   * window, when an application attempts to "undecorate"
-   * the window again, this bit will be cleared */
-  hints.flags = MWM_HINTS_DECORATIONS;
-  hints.decorations = MWM_HINTS_UNDECORATED_UNITY;
-
-  SetMwmWindowHints(window_id, &hints);
-}
-
 bool PluginAdapter::IsScreenGrabbed() const
 {
   return m_Screen->grabbed();
@@ -1544,12 +1413,11 @@ void PluginAdapter::OnWindowClosed(CompWindow *w)
     _last_focused_window = NULL;
 }
 
-void PluginAdapter::AddProperties(GVariantBuilder* builder)
+void PluginAdapter::AddProperties(debug::IntrospectionData& wrapper)
 {
-  unity::variant::BuilderWrapper wrapper(builder);
   wrapper.add(GetScreenGeometry())
          .add("workspace_count", WorkspaceCount())
-         .add("active_window", (uint64_t)GetActiveWindow())
+         .add("active_window", GetActiveWindow())
          .add("screen_grabbed", IsScreenGrabbed())
          .add("scale_active", IsScaleActive())
          .add("scale_active_for_group", IsScaleActiveForGroup())
@@ -1558,90 +1426,4 @@ void PluginAdapter::AddProperties(GVariantBuilder* builder)
          .add("showdesktop_active", _in_show_desktop);
 }
 
-std::string PluginAdapter::GetWindowName(Window window_id) const
-{
-  std::string name;
-  Atom visibleNameAtom;
-
-  visibleNameAtom = gdk_x11_get_xatom_by_name("_NET_WM_VISIBLE_NAME");
-  name = GetUtf8Property(window_id, visibleNameAtom);
-
-  if (name.empty())
-    name = GetUtf8Property(window_id, Atoms::wmName);
-
-  if (name.empty())
-    name = GetTextProperty(window_id, XA_WM_NAME);
-
-  return name;
-}
-
-std::string PluginAdapter::GetUtf8Property(Window window_id, Atom atom) const
-{
-  Atom type;
-  int result, format;
-  unsigned long n_items, bytes_after;
-  char *val = nullptr;
-  std::string retval;
-
-  result = XGetWindowProperty(m_Screen->dpy(), window_id, atom, 0L, 65536, False,
-                              Atoms::utf8String, &type, &format, &n_items,
-                              &bytes_after, reinterpret_cast<unsigned char **>(&val));
-
-  if (result != Success)
-    return retval;
-
-  if (type == Atoms::utf8String && format == 8 && val && n_items > 0)
-  {
-    retval = std::string(val, n_items);
-  }
-
-  XFree(val);
-
-  return retval;
-}
-
-std::string PluginAdapter::GetTextProperty(Window window_id, Atom atom) const
-{
-  XTextProperty text;
-  std::string retval;
-  text.nitems = 0;
-
-  if (XGetTextProperty(m_Screen->dpy(), window_id, &text, atom))
-  {
-    if (text.value)
-    {
-      retval = std::string(reinterpret_cast<char*>(text.value), text.nitems);
-      XFree(text.value);
-    }
-  }
-
-  return retval;
-}
-
-std::vector<long> PluginAdapter::GetCardinalProperty(Window window_id, Atom atom) const
-{
-  Atom type;
-  int result, format;
-  unsigned long n_items, bytes_after;
-  long *buf = nullptr;
-
-  result = XGetWindowProperty(m_Screen->dpy(), window_id, atom, 0L, 65536, False,
-                              XA_CARDINAL, &type, &format, &n_items, &bytes_after,
-                              reinterpret_cast<unsigned char **>(&buf));
-
-  std::unique_ptr<long[], int(*)(void*)> buffer(buf, XFree);
-
-  if (result == Success && type == XA_CARDINAL && format == 32 && buffer && n_items > 0)
-  {
-    std::vector<long> values(n_items);
-
-    for (unsigned i = 0; i < n_items; ++i)
-      values[i] = buffer[i];
-
-    return values;
-  }
-
-  return std::vector<long>();
-}
-
 } // namespace unity
