@@ -25,7 +25,6 @@
 #include "UScreen.h"
 
 #include <NuxCore/Logger.h>
-#include <UnityCore/Variant.h>
 
 // Entirely stubs for now, unless we need this functionality at some point
 
@@ -46,6 +45,7 @@ StandaloneWindow::StandaloneWindow(Window xid)
   , v_maximized(false)
   , h_maximized(false)
   , minimized(false)
+  , shaded(false)
   , decorated(true)
   , has_decorations(true)
   , on_top(false)
@@ -82,6 +82,7 @@ StandaloneWindow::StandaloneWindow(Window xid)
 
   v_maximized.changed.connect([this] (bool value) { maximized.changed.emit(maximized()); });
   h_maximized.changed.connect([this] (bool value) { maximized.changed.emit(maximized()); });
+  shaded.changed.connect([this] (bool value) { minimized = value; });
 }
 
 WindowManagerPtr create_window_manager()
@@ -130,6 +131,18 @@ std::vector<Window> StandaloneWindowManager::GetWindowsInStackingOrder() const
 
   return ret;
 }
+  
+int StandaloneWindowManager::MonitorGeometryIn(nux::Geometry const& geo) const
+{
+  // TODO
+  return 0;
+}
+
+bool StandaloneWindowManager::IsTopWindowFullscreenOnMonitorWithMouse() const
+{
+  // TODO
+  return false;
+}
 
 bool StandaloneWindowManager::IsWindowMaximized(Window window_id) const
 {
@@ -264,69 +277,46 @@ bool StandaloneWindowManager::InShowDesktop() const
   return in_show_desktop_;
 }
 
-void StandaloneWindowManager::Decorate(Window window_id) const
-{
-  auto window = GetWindowByXid(window_id);
-  if (window)
-    window->decorated = window->has_decorations();
-}
-
-void StandaloneWindowManager::Undecorate(Window window_id) const
-{
-  auto window = GetWindowByXid(window_id);
-  if (window)
-    window->decorated = false;
-}
-
 void StandaloneWindowManager::Maximize(Window window_id)
 {
   auto window = GetWindowByXid(window_id);
   if (window)
+  {
     window->maximized = true;
+    window->decorated = false;
+  }
 }
 
-void StandaloneWindowManager::LeftMaximize(Window window_id)
+void StandaloneWindowManager::VerticallyMaximize(Window window_id)
 {
   auto window = GetWindowByXid(window_id);
-
-  if (!window)
-    return;
-
-  /* Let's compute the area where the window should be put */
-  auto workarea = GetWorkAreaGeometry(window_id);
-  workarea.width /= 2;
-
-  if (window->maximized)
-    window->maximized = false;
-
-  window->v_maximized = true;
-  MoveResizeWindow(window_id, workarea);
+  if (window)
+  {
+    window->h_maximized = false;
+    window->v_maximized = true;
+    window->decorated = true;
+  }
 }
 
-void StandaloneWindowManager::RightMaximize(Window window_id)
+void StandaloneWindowManager::HorizontallyMaximize(Window window_id)
 {
   auto window = GetWindowByXid(window_id);
-
-  if (!window)
-    return;
-
-  /* Let's compute the area where the window should be put */
-  auto workarea = GetWorkAreaGeometry(window_id);
-  workarea.width /= 2;
-  workarea.x += workarea.width;
-
-  if (window->maximized)
-    window->maximized = false;
-
-  window->v_maximized = true;
-  MoveResizeWindow(window_id, workarea);
+  if (window)
+  {
+    window->v_maximized = false;
+    window->h_maximized = true;
+    window->decorated = true;
+  }
 }
 
 void StandaloneWindowManager::Restore(Window window_id)
 {
   auto window = GetWindowByXid(window_id);
   if (window)
+  {
     window->maximized = false;
+    window->decorated = true;
+  }
 }
 
 void StandaloneWindowManager::RestoreAt(Window window_id, int x, int y)
@@ -345,7 +335,7 @@ void StandaloneWindowManager::UnMinimize(Window window_id)
     window->minimized = false;
 
     if (window->maximized)
-      Undecorate(window_id);
+      window->decorated = false;
   }
 }
 
@@ -357,7 +347,7 @@ void StandaloneWindowManager::Minimize(Window window_id)
     window->minimized = true;
 
     if (window->maximized)
-      Decorate(window_id);
+      window->decorated = true;
   }
 }
 
@@ -416,11 +406,9 @@ void StandaloneWindowManager::RestackBelow(Window window_id, Window sibiling_id)
 }
 
 void StandaloneWindowManager::TerminateScale()
-{}
-
-void StandaloneWindowManager::SetScaleActive(bool scale_active)
 {
-  scale_active_ = scale_active;
+  scale_active_ = false;
+  scale_active_for_group_ = false;
 }
 
 bool StandaloneWindowManager::IsScaleActive() const
@@ -428,11 +416,6 @@ bool StandaloneWindowManager::IsScaleActive() const
   return scale_active_;
 }
 
-void StandaloneWindowManager::SetScaleActiveForGroup(bool scale_active_for_group)
-{
-  scale_active_for_group_ = scale_active_for_group;
-}
-
 bool StandaloneWindowManager::IsScaleActiveForGroup() const
 {
   return scale_active_for_group_;
@@ -440,7 +423,7 @@ bool StandaloneWindowManager::IsScaleActiveForGroup() const
 
 void StandaloneWindowManager::InitiateExpo()
 {
-  expo_state_ = !expo_state_;
+  expo_state_ = true;
 }
 
 void StandaloneWindowManager::TerminateExpo()
@@ -458,11 +441,6 @@ bool StandaloneWindowManager::IsWallActive() const
   return false;
 }
 
-void StandaloneWindowManager::SetIsAnyWindowMoving(bool is_any_window_moving)
-{
-  is_any_window_moving_ = is_any_window_moving;
-}
-
 bool StandaloneWindowManager::IsAnyWindowMoving() const
 {
   return is_any_window_moving_;
@@ -625,8 +603,91 @@ std::string StandaloneWindowManager::GetWindowName(Window window_id) const
   return "";
 }
 
+std::string StandaloneWindowManager::GetStringProperty(Window, Atom) const
+{
+  return std::string();
+}
+
+std::vector<long> StandaloneWindowManager::GetCardinalProperty(Window, Atom) const
+{
+  return std::vector<long>();
+}
+
+bool StandaloneWindowManager::IsWindowShaded(Window window_id) const
+{
+  auto window = GetWindowByXid(window_id);
+  if (window)
+    return window->shaded;
+
+  return false;
+}
+
+void StandaloneWindowManager::Shade(Window window_id)
+{
+  auto window = GetWindowByXid(window_id);
+  if (window)
+    window->shaded = true;
+}
+
+void StandaloneWindowManager::UnShade(Window window_id)
+{
+  auto window = GetWindowByXid(window_id);
+  if (window)
+    window->shaded = false;
+}
+
+
+
 // Mock functions
 
+void StandaloneWindowManager::ResetStatus()
+{
+  for (auto const& win : GetStandaloneWindows())
+    Close(win->Xid());
+
+  expo_state_ = false;
+  in_show_desktop_ = false;
+  scale_active_ = false;
+  scale_active_for_group_ = false;
+  is_any_window_moving_ = false;
+  current_desktop_ = 0;
+  viewport_size_ = {2, 2};
+  current_vp_ = {0, 0};
+}
+
+void StandaloneWindowManager::SetScaleActiveForGroup(bool scale_active_for_group)
+{
+  if (scale_active_for_group_ == scale_active_for_group)
+    return;
+
+  scale_active_ = scale_active_for_group;
+  scale_active_for_group_ = scale_active_for_group;
+  scale_active_ ? initiate_spread.emit() : terminate_spread.emit();
+}
+
+void StandaloneWindowManager::SetScaleActive(bool scale_active)
+{
+  if (scale_active_ == scale_active)
+    return;
+
+  scale_active_ = scale_active;
+  scale_active_ ? initiate_spread.emit() : terminate_spread.emit();
+}
+
+void StandaloneWindowManager::SetExpoActive(bool expo_active)
+{
+  if (expo_state_ == expo_active)
+    return;
+
+  expo_state_ = expo_active;
+  expo_state_ ? initiate_expo.emit() : terminate_expo.emit();
+}
+
+void StandaloneWindowManager::SetIsAnyWindowMoving(bool is_any_window_moving)
+{
+  is_any_window_moving_ = is_any_window_moving;
+}
+
 void StandaloneWindowManager::SetCurrentDesktop(unsigned desktop_id)
 {
   current_desktop_ = desktop_id;
@@ -645,8 +706,6 @@ void StandaloneWindowManager::AddStandaloneWindow(StandaloneWindow::Ptr const& w
   window->visible.changed.connect([this, xid] (bool v) {v ? window_shown(xid) : window_hidden(xid);});
   window->maximized.changed.connect([this, xid] (bool v) {v ? window_maximized(xid) : window_restored(xid);});
   window->minimized.changed.connect([this, xid] (bool v) {v ? window_minimized(xid) : window_unminimized(xid);});
-  window->decorated.changed.connect([this, xid] (bool v) {v ? window_decorated(xid) : window_undecorated(xid);});
-  window->has_decorations.changed.connect([this, xid] (bool v) {v ? window_decorated(xid) : window_undecorated(xid);});
   window->resized.connect([this, xid] { window_resized(xid); });
   window->moved.connect([this, xid] { window_moved(xid); });
 
@@ -675,12 +734,11 @@ void StandaloneWindowManager::SetCurrentViewport(nux::Point const& vp)
   current_vp_ = vp;
 }
 
-void StandaloneWindowManager::AddProperties(GVariantBuilder* builder)
+void StandaloneWindowManager::AddProperties(debug::IntrospectionData& wrapper)
 {
-  unity::variant::BuilderWrapper wrapper(builder);
   wrapper.add(GetScreenGeometry())
          .add("workspace_count", WorkspaceCount())
-         .add("active_window", (uint64_t)GetActiveWindow())
+         .add("active_window", GetActiveWindow())
          .add("screen_grabbed", IsScreenGrabbed())
          .add("scale_active", IsScaleActive())
          .add("scale_active_for_group", IsScaleActiveForGroup())
