@@ -18,18 +18,14 @@
  *              Marco Trevisan (Treviño) <3v1n0@ubuntu.com>
  */
 
-#include "config.h"
-
 #include <Nux/Nux.h>
 #include <array>
 
 #include <UnityCore/GLibWrapper.h>
-#include <UnityCore/Variant.h>
 
 #include "WindowButtons.h"
 #include "WindowButtonPriv.h"
 
-#include "unity-shared/TextureCache.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/WindowManager.h"
 
@@ -44,6 +40,7 @@ WindowButton::WindowButton(panel::WindowButtonType type)
             sigc::mem_fun(this, &WindowButton::EnabledSetter))
   , overlay_mode(false)
   , type_(type)
+  , monitor_(0)
 {
   overlay_mode.changed.connect([this] (bool) { UpdateSize(); QueueDraw(); });
   SetAcceptKeyNavFocusOnMouseDown(false);
@@ -51,6 +48,17 @@ WindowButton::WindowButton(panel::WindowButtonType type)
   LoadImages();
 }
 
+void WindowButton::UpdateDPIChanged()
+{
+  LoadImages();
+}
+
+void WindowButton::OnMonitorChanged(int monitor)
+{
+  monitor_ = monitor;
+  LoadImages();
+}
+
 void WindowButton::SetVisualState(nux::ButtonVisualState new_state)
 {
   if (new_state != visual_state_)
@@ -134,7 +142,7 @@ void WindowButton::Draw(nux::GraphicsEngine& GfxContext, bool force_draw)
 
 void WindowButton::UpdateSize()
 {
-  int panel_height = panel::Style::Instance().panel_height;
+  int panel_height = panel::Style::Instance().PanelHeight(monitor_);
   nux::BaseTexture* tex;
   tex = (overlay_mode()) ? normal_dash_tex_.GetPointer() : normal_tex_.GetPointer();
   int width = 0;
@@ -142,7 +150,7 @@ void WindowButton::UpdateSize()
 
   if (tex)
   {
-    width = std::min(panel_height, tex->GetWidth());
+    width  = std::min(panel_height, tex->GetWidth());
     height = std::min(panel_height, tex->GetHeight());
   }
 
@@ -153,39 +161,22 @@ void WindowButton::LoadImages()
 {
   panel::Style& style = panel::Style::Instance();
 
-  normal_tex_ = style.GetWindowButton(type_, panel::WindowState::NORMAL);
-  prelight_tex_ = style.GetWindowButton(type_, panel::WindowState::PRELIGHT);
-  pressed_tex_ = style.GetWindowButton(type_, panel::WindowState::PRESSED);
-  unfocused_tex_ = style.GetWindowButton(type_, panel::WindowState::UNFOCUSED);
-  disabled_tex_ = style.GetWindowButton(type_, panel::WindowState::DISABLED);
-  unfocused_prelight_tex_ = style.GetWindowButton(type_, panel::WindowState::UNFOCUSED_PRELIGHT);
-  unfocused_pressed_tex_ = style.GetWindowButton(type_, panel::WindowState::UNFOCUSED_PRESSED);
-  normal_dash_tex_ = GetDashWindowButton(type_, panel::WindowState::NORMAL);
-  prelight_dash_tex_ = GetDashWindowButton(type_, panel::WindowState::PRELIGHT);
-  pressed_dash_tex_ = GetDashWindowButton(type_, panel::WindowState::PRESSED);
-  disabled_dash_tex_ = GetDashWindowButton(type_, panel::WindowState::DISABLED);
+  normal_tex_ = style.GetWindowButton(type_, panel::WindowState::NORMAL, monitor_);
+  prelight_tex_ = style.GetWindowButton(type_, panel::WindowState::PRELIGHT, monitor_);
+  pressed_tex_ = style.GetWindowButton(type_, panel::WindowState::PRESSED, monitor_);
+  unfocused_tex_ = style.GetWindowButton(type_, panel::WindowState::BACKDROP, monitor_);
+  disabled_tex_ = style.GetWindowButton(type_, panel::WindowState::DISABLED, monitor_);
+  unfocused_prelight_tex_ = style.GetWindowButton(type_, panel::WindowState::BACKDROP_PRELIGHT, monitor_);
+  unfocused_pressed_tex_ = style.GetWindowButton(type_, panel::WindowState::BACKDROP_PRESSED, monitor_);
+  normal_dash_tex_ = style.GetDashWindowButton(type_, panel::WindowState::NORMAL, monitor_);
+  prelight_dash_tex_ = style.GetDashWindowButton(type_, panel::WindowState::PRELIGHT, monitor_);
+  pressed_dash_tex_ = style.GetDashWindowButton(type_, panel::WindowState::PRESSED, monitor_);
+  disabled_dash_tex_ = style.GetDashWindowButton(type_, panel::WindowState::DISABLED, monitor_);
 
   UpdateSize();
   QueueDraw();
 }
 
-nux::ObjectPtr<nux::BaseTexture> WindowButton::GetDashWindowButton(panel::WindowButtonType type, panel::WindowState state)
-{
-  nux::ObjectPtr<nux::BaseTexture> texture;
-  static const std::array<std::string, 4> names = {{ "close_dash", "minimize_dash", "unmaximize_dash", "maximize_dash" }};
-  static const std::array<std::string, 4> states = {{ "", "_prelight", "_pressed", "_disabled" }};
-
-  std::string subpath = names[static_cast<int>(type)] + states[static_cast<int>(state)] + ".png";
-
-  auto& cache = TextureCache::GetDefault();
-  texture = cache.FindTexture(subpath);
-
-  if (!texture)
-    texture = panel::Style::Instance().GetFallbackWindowButton(type, state);
-
-  return texture;
-}
-
 panel::WindowButtonType WindowButton::GetType() const
 {
   return type_;
@@ -206,7 +197,7 @@ std::string WindowButton::GetName() const
   return "WindowButton";
 }
 
-void WindowButton::AddProperties(GVariantBuilder* builder)
+void WindowButton::AddProperties(debug::IntrospectionData& introspection)
 {
   std::string type_name;
   std::string state_name;
@@ -225,6 +216,8 @@ void WindowButton::AddProperties(GVariantBuilder* builder)
     case panel::WindowButtonType::UNMAXIMIZE:
       type_name = "Unmaximize";
       break;
+    default:
+      break;
   }
 
   switch (visual_state_)
@@ -239,7 +232,7 @@ void WindowButton::AddProperties(GVariantBuilder* builder)
       state_name = "normal";
   }
 
-  variant::BuilderWrapper(builder).add(GetAbsoluteGeometry())
+  introspection.add(GetAbsoluteGeometry())
                                   .add("type", type_name)
                                   .add("visible", IsVisible() && Parent()->opacity() != 0.0f)
                                   .add("sensitive", Parent()->GetInputEventSensitivity())
@@ -261,18 +254,19 @@ WindowButtons::WindowButtons()
 {
   controlled_window.changed.connect(sigc::mem_fun(this, &WindowButtons::OnControlledWindowChanged));
   focused.changed.connect(sigc::hide(sigc::mem_fun(this, &WindowButtons::QueueDraw)));
+  monitor.changed.connect(sigc::mem_fun(this, &WindowButtons::OnMonitorChanged));
 
-  auto lambda_enter = [&](int x, int y, unsigned long button_flags, unsigned long key_flags)
+  auto lambda_enter = [this](int x, int y, unsigned long button_flags, unsigned long key_flags)
   {
     mouse_enter.emit(x, y, button_flags, key_flags);
   };
 
-  auto lambda_leave = [&](int x, int y, unsigned long button_flags, unsigned long key_flags)
+  auto lambda_leave = [this](int x, int y, unsigned long button_flags, unsigned long key_flags)
   {
     mouse_leave.emit(x, y, button_flags, key_flags);
   };
 
-  auto lambda_moved = [&](int x, int y, int dx, int dy, unsigned long button_flags, unsigned long key_flags)
+  auto lambda_moved = [this](int x, int y, int dx, int dy, unsigned long button_flags, unsigned long key_flags)
   {
     mouse_move.emit(x, y, dx, dy, button_flags, key_flags);
   };
@@ -316,6 +310,26 @@ WindowButtons::WindowButtons()
   ubus_manager_.RegisterInterest(UBUS_OVERLAY_SHOWN, sigc::mem_fun(this, &WindowButtons::OnOverlayShown));
   ubus_manager_.RegisterInterest(UBUS_OVERLAY_HIDDEN, sigc::mem_fun(this, &WindowButtons::OnOverlayHidden));
   Settings::Instance().form_factor.changed.connect(sigc::mem_fun(this, &WindowButtons::OnDashSettingsUpdated));
+  WindowManager::Default().initiate_spread.connect(sigc::mem_fun(this, &WindowButtons::OnSpreadInitiate));
+  WindowManager::Default().terminate_spread.connect(sigc::mem_fun(this, &WindowButtons::OnSpreadTerminate));
+}
+
+void WindowButtons::UpdateDPIChanged()
+{
+  for (auto area : GetChildren())
+    static_cast<internal::WindowButton*>(area)->UpdateDPIChanged();
+
+  ComputeContentSize();
+  int width = GetContentWidth();
+  SetMaximumWidth(width);
+  SetMinimumWidth(width);
+}
+
+void WindowButtons::OnMonitorChanged(int monitor)
+{
+  // Need to update the EMConverter in each window button if the monitor changes
+  for (auto area : GetChildren())
+    static_cast<internal::WindowButton*>(area)->OnMonitorChanged(monitor);
 }
 
 nux::Area* WindowButtons::FindAreaUnderMouse(const nux::Point& mouse, nux::NuxEventType event_type)
@@ -354,14 +368,19 @@ nux::Area* WindowButtons::FindAreaUnderMouse(const nux::Point& mouse, nux::NuxEv
 
 void WindowButtons::OnCloseClicked(nux::Button *button)
 {
-  auto win_button = dynamic_cast<internal::WindowButton*>(button);
+  auto win_button = static_cast<internal::WindowButton*>(button);
 
-  if (!win_button || !win_button->enabled())
+  if (!win_button->enabled())
     return;
 
   if (win_button->overlay_mode())
   {
-    ubus_manager_.SendMessage(UBUS_OVERLAY_CLOSE_REQUEST);
+    auto& wm = WindowManager::Default();
+
+    if (wm.IsScaleActive())
+      wm.TerminateScale();
+    else
+      ubus_manager_.SendMessage(UBUS_OVERLAY_CLOSE_REQUEST);
   }
   else
   {
@@ -373,9 +392,9 @@ void WindowButtons::OnCloseClicked(nux::Button *button)
 
 void WindowButtons::OnMinimizeClicked(nux::Button *button)
 {
-  auto win_button = dynamic_cast<internal::WindowButton*>(button);
+  auto win_button = static_cast<internal::WindowButton*>(button);
 
-  if (!win_button || !win_button->enabled())
+  if (!win_button->enabled())
     return;
 
   if (!win_button->overlay_mode())
@@ -386,9 +405,9 @@ void WindowButtons::OnMinimizeClicked(nux::Button *button)
 
 void WindowButtons::OnRestoreClicked(nux::Button *button)
 {
-  auto win_button = dynamic_cast<internal::WindowButton*>(button);
+  auto win_button = static_cast<internal::WindowButton*>(button);
 
-  if (!win_button || !win_button->enabled())
+  if (!win_button->enabled())
     return;
 
   if (win_button->overlay_mode())
@@ -410,9 +429,9 @@ void WindowButtons::OnRestoreClicked(nux::Button *button)
 
 void WindowButtons::OnMaximizeClicked(nux::Button *button)
 {
-  auto win_button = dynamic_cast<internal::WindowButton*>(button);
+  auto win_button = static_cast<internal::WindowButton*>(button);
 
-  if (!win_button || !win_button->enabled())
+  if (!win_button->enabled())
     return;
 
   if (win_button->overlay_mode())
@@ -437,12 +456,7 @@ void WindowButtons::OnOverlayShown(GVariant* data)
   if (overlay_monitor != monitor())
   {
     for (auto area : GetChildren())
-    {
-      auto button = dynamic_cast<internal::WindowButton*>(area);
-
-      if (button)
-        button->enabled = false;
-    }
+      static_cast<internal::WindowButton*>(area)->enabled = false;
 
     return;
   }
@@ -451,24 +465,20 @@ void WindowButtons::OnOverlayShown(GVariant* data)
 
   for (auto area : GetChildren())
   {
-    auto button = dynamic_cast<internal::WindowButton*>(area);
+    auto button = static_cast<internal::WindowButton*>(area);
+    if (button->GetType() == panel::WindowButtonType::CLOSE)
+      button->enabled = true;
 
-    if (button)
-    {
-      if (button->GetType() == panel::WindowButtonType::CLOSE)
-        button->enabled = true;
+    if (button->GetType() == panel::WindowButtonType::UNMAXIMIZE)
+      restore_button = button;
 
-      if (button->GetType() == panel::WindowButtonType::UNMAXIMIZE)
-        restore_button = button;
+    if (button->GetType() == panel::WindowButtonType::MAXIMIZE)
+      maximize_button = button;
 
-      if (button->GetType() == panel::WindowButtonType::MAXIMIZE)
-        maximize_button = button;
+    if (button->GetType() == panel::WindowButtonType::MINIMIZE)
+      button->enabled = false;
 
-      if (button->GetType() == panel::WindowButtonType::MINIMIZE)
-        button->enabled = false;
-
-      button->overlay_mode = true;
-    }
+    button->overlay_mode = true;
   }
 
   if (restore_button && maximize_button)
@@ -496,9 +506,6 @@ void WindowButtons::OnOverlayShown(GVariant* data)
 
 void WindowButtons::OnOverlayHidden(GVariant* data)
 {
-  internal::WindowButton* maximize_button = nullptr;
-  internal::WindowButton* restore_button = nullptr;
-
   glib::String overlay_identity;
   gboolean can_maximise = FALSE;
   gint32 overlay_monitor = 0;
@@ -509,12 +516,7 @@ void WindowButtons::OnOverlayHidden(GVariant* data)
   if (overlay_monitor != monitor())
   {
     for (auto area : GetChildren())
-    {
-      auto button = dynamic_cast<internal::WindowButton*>(area);
-
-      if (button)
-        button->enabled = true;
-    }
+      static_cast<internal::WindowButton*>(area)->enabled = true;
   }
 
   if (active_overlay_ != overlay_identity.Str())
@@ -522,30 +524,54 @@ void WindowButtons::OnOverlayHidden(GVariant* data)
 
   active_overlay_ = "";
 
-  WindowManager& wm = WindowManager::Default();
+  if (!WindowManager::Default().IsScaleActive())
+    ResetNormalButtonState();
+}
+
+void WindowButtons::OnSpreadInitiate()
+{
   for (auto area : GetChildren())
   {
-    auto button = dynamic_cast<internal::WindowButton*>(area);
+    auto button = static_cast<internal::WindowButton*>(area);
+    button->enabled = (button->GetType() == panel::WindowButtonType::CLOSE);
+    button->overlay_mode = true;
+  }
+}
 
-    if (button)
+void WindowButtons::OnSpreadTerminate()
+{
+  if (!active_overlay_.empty())
+      return;
+
+  ResetNormalButtonState();
+}
+
+void WindowButtons::ResetNormalButtonState()
+{
+  WindowManager& wm = WindowManager::Default();
+  internal::WindowButton* maximize_button = nullptr;
+  internal::WindowButton* restore_button = nullptr;
+
+  for (auto* area : GetChildren())
+  {
+    auto button = static_cast<internal::WindowButton*>(area);
+
+    if (controlled_window())
     {
-      if (controlled_window())
-      {
-        if (button->GetType() == panel::WindowButtonType::CLOSE)
-          button->enabled = wm.IsWindowClosable(controlled_window());
+      if (button->GetType() == panel::WindowButtonType::CLOSE)
+        button->enabled = wm.IsWindowClosable(controlled_window());
 
-        if (button->GetType() == panel::WindowButtonType::MINIMIZE)
-          button->enabled = wm.IsWindowMinimizable(controlled_window());
-      }
-
-      if (button->GetType() == panel::WindowButtonType::UNMAXIMIZE)
-        restore_button = button;
-
-      if (button->GetType() == panel::WindowButtonType::MAXIMIZE)
-        maximize_button = button;
-
-      button->overlay_mode = false;
+      if (button->GetType() == panel::WindowButtonType::MINIMIZE)
+        button->enabled = wm.IsWindowMinimizable(controlled_window());
     }
+
+    if (button->GetType() == panel::WindowButtonType::UNMAXIMIZE)
+      restore_button = button;
+
+    if (button->GetType() == panel::WindowButtonType::MAXIMIZE)
+      maximize_button = button;
+
+    button->overlay_mode = false;
   }
 
   if (restore_button && maximize_button)
@@ -572,22 +598,19 @@ void WindowButtons::OnDashSettingsUpdated(FormFactor form_factor)
 
   for (auto area : GetChildren())
   {
-    auto button = dynamic_cast<internal::WindowButton*>(area);
+    auto button = static_cast<internal::WindowButton*>(area);
 
-    if (button)
-    {
-      if (!button->overlay_mode())
-        break;
+    if (!button->overlay_mode())
+      break;
 
-      if (button->GetType() == panel::WindowButtonType::UNMAXIMIZE)
-        restore_button = button;
+    if (button->GetType() == panel::WindowButtonType::UNMAXIMIZE)
+      restore_button = button;
 
-      if (button->GetType() == panel::WindowButtonType::MAXIMIZE)
-        maximize_button = button;
+    if (button->GetType() == panel::WindowButtonType::MAXIMIZE)
+      maximize_button = button;
 
-      if (restore_button && maximize_button)
-        break;
-    }
+    if (restore_button && maximize_button)
+      break;
   }
 
   if (restore_button && restore_button->overlay_mode() && maximize_button)
@@ -632,10 +655,7 @@ void WindowButtons::OnControlledWindowChanged(Window xid)
     WindowManager& wm = WindowManager::Default();
     for (auto area : GetChildren())
     {
-      auto button = dynamic_cast<internal::WindowButton*>(area);
-
-      if (!button)
-        continue;
+      auto button = static_cast<internal::WindowButton*>(area);
 
       if (button->GetType() == panel::WindowButtonType::CLOSE)
         button->enabled = wm.IsWindowClosable(xid);
@@ -646,20 +666,31 @@ void WindowButtons::OnControlledWindowChanged(Window xid)
   }
 }
 
+bool WindowButtons::IsMouseOwner()
+{
+  for (auto* area : GetChildren())
+  {
+    if (static_cast<internal::WindowButton*>(area)->IsMouseOwner())
+      return true;
+  }
+
+  return false;
+}
+
 std::string WindowButtons::GetName() const
 {
   return "WindowButtons";
 }
 
-void WindowButtons::AddProperties(GVariantBuilder* builder)
+void WindowButtons::AddProperties(debug::IntrospectionData& introspection)
 {
-  variant::BuilderWrapper(builder).add(GetAbsoluteGeometry())
-                                  .add("monitor", monitor())
-                                  .add("opacity", opacity())
-                                  .add("visible", opacity() != 0.0f)
-                                  .add("sensitive", GetInputEventSensitivity())
-                                  .add("focused", focused())
-                                  .add("controlled_window", (guint64)controlled_window());
+  introspection.add(GetAbsoluteGeometry())
+               .add("monitor", monitor())
+               .add("opacity", opacity())
+               .add("visible", opacity() != 0.0f)
+               .add("sensitive", GetInputEventSensitivity())
+               .add("focused", focused())
+               .add("controlled_window", controlled_window());
 }
 
 } // unity namespace
